<?xml version="1.0" encoding="UTF-8"?>
<?altova_samplexml C:\Documents and Settings\Ashley_Cheung\Desktop\date dropdown\hyfinityXSLSample.xml?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:out="http://www.hyfinity.com/fm/preprocessout" xmlns:exsl="http://exslt.org/common" exclude-result-prefixes="exsl" extension-element-prefixes="exsl">
    <!-- Import modular XSLs for perform generation of specific application areas such as events scripts. -->
    <xsl:import href="genExternalEvents.xsl"/>
    <!-- <xsl:output indent="yes" method="xml" cdata-section-elements="script" encoding="UTF-8"/> -->
    <xsl:output indent="yes" method="xml" encoding="UTF-8"/>
    <xsl:namespace-alias result-prefix="xsl" stylesheet-prefix="out"/>
    <!-- Inserts the XSL/ sxforms specific information into the generated HTML to
         produce the final stylesheet.  The transform is performed on the HTML result,
         with the structure, bindings, and application config files loaded in as a variables.
         The existing file is also loaded in to keep inserted templates and custom sections -->
    <xsl:variable name="formStructure" select="/fm:transform_structure/fm:structure" xmlns:fm="http://www.hyfinity.com/formmaker"/>
    <xsl:variable name="bindings" select="/fm:transform_structure/fm:bindings" xmlns:fm="http://www.hyfinity.com/formmaker"/>
    <xsl:variable name="appConfig" select="/fm:transform_structure/fm:app_config" xmlns:fm="http://www.hyfinity.com/formmaker"/>
    <xsl:variable name="existingFile" select="/fm:transform_structure/fm:existing_file" xmlns:fm="http://www.hyfinity.com/formmaker"/>
    <xsl:variable name="eventConfig" select="/fm:transform_structure/fm:event_config" xmlns:fm="http://www.hyfinity.com/formmaker"/>
    <xsl:variable name="htmlResult" select="/fm:transform_structure/fm:html_gen_output" xmlns:fm="http://www.hyfinity.com/formmaker"/>
    <xsl:variable name="page_id" select="/fm:transform_structure/@page_id" xmlns:fm="http://www.hyfinity.com/formmaker"/>


    <!-- Find the names of every action that this page could participate in
         These will be placed in a semi colon seperated string for later processing. -->
    <xsl:variable name="actionNames" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:for-each select="$appConfig/fm:application/fm:links/fm:link[fm:source[(@type = 'Page') and (. = $page_id)]]" >
            <xsl:text>;</xsl:text>
            <xsl:value-of select="fm:action" />
        </xsl:for-each>

        <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[(@id = $page_id) and (@type = 'subsection')]">
            <xsl:call-template name="generateParentActionNames">
                <xsl:with-param name="currentPageName" select="$page_id"/>
            </xsl:call-template>
        </xsl:if>

        <xsl:text>;</xsl:text>
    </xsl:variable>

    <xsl:template name="generateParentActionNames" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="currentPageName"/>

        <!-- find the parent pages -->
        <xsl:for-each select="$appConfig/fm:application/fm:pages/fm:page[@id=$appConfig/fm:application/fm:links/fm:link[fm:target=$currentPageName and fm:source/@type='Page']/fm:source]">
            <xsl:variable name="parentPage" select="./@id"/>

            <xsl:if test="count($appConfig/fm:application/fm:links/fm:link[fm:source/@type = 'Page' and fm:source = $parentPage]/fm:action) != 0">
                <xsl:for-each select="$appConfig/fm:application/fm:links/fm:link[fm:source/@type = 'Page' and fm:source = $parentPage]">
                    <xsl:text>;</xsl:text>
                    <xsl:value-of select="fm:action" />
                </xsl:for-each>
            </xsl:if>
            <xsl:if test="./@type = 'subsection'">
                <xsl:call-template name="generateParentActionNames">
                    <xsl:with-param name="currentPageName" select="$parentPage"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>


    <!-- create the form_id variable to use in generated script fragments.
         For normal pages, this is set to the name of the form that has been created.
         For subsection pages however there is no form, as the content gets embedded within the calling page's form.
         Therefore, for now just set to reference the first form, but this is not an ideal solution.
         TODO: Work out how to change the script fragments to not require the actual form name. -->
    <xsl:variable name="form_id" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:choose>
            <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" >
                <xsl:text>forms[0]</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$htmlResult/div/form/@name"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="accessibility_level" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:choose>
            <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:accessibility_level[. != '']">
                <xsl:value-of select="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:accessibility_level"/>
            </xsl:when>
            <xsl:when test="$appConfig/fm:application/fm:defaults/fm:accessibility_level != ''">
                <xsl:value-of select="$appConfig/fm:application/fm:defaults/fm:accessibility_level"/>
            </xsl:when>
            <xsl:otherwise>none</xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <!-- The name of the XSL file that should be imported by the generated XSL if any value conversions are needed
         This XSL should define the 'convert-case' and 'parse-format-date' templates. -->
    <xsl:variable name="conversions_xsl_filename">conversions.xsl</xsl:variable>

    <!-- The value stored in the xpaths fields when they have not been entered on screen -->
    <xsl:variable name="blank_xpath_value">enter_xpath_here</xsl:variable>

    <!--

        Root template that starts the XSL processing.
        First checks whether an existing file exists to copy stuff from.

        -->
    <xsl:template match="/">
        <xsl:choose>
            <xsl:when test="$existingFile/*">
                <!-- Copy the stylesheet tag and templates from the existing file -->
                <xsl:apply-templates select="$existingFile/*" mode="copyExistingFile">
                    <xsl:with-param name="newContent" select="$htmlResult/*"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
                <!-- Existing file is empty - create everything -->
                <out:stylesheet version="1.0" exclude-result-prefixes="fm" xmlns:fm="http://www.hyfinity.com/formmaker">
                    <!-- Set up skin import statement -->
                    <xsl:choose>
                        <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:skin != ''">
                            <out:import href="{$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:skin}"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <!-- don't use the default skin if this is a subsection page -->
                            <xsl:if test="not($appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection')" >
                                <out:import href="{$appConfig/fm:application/fm:defaults/fm:skin}"/>
                            </xsl:if>
                        </xsl:otherwise>
                    </xsl:choose>
                    <!-- import the conversion utility XSL if needed -->
                    <xsl:variable name="conversionRequired">
                        <xsl:call-template name="checkConversionRequired"/>
                    </xsl:variable>
                    <xsl:if test="$conversionRequired = 'true'">
                        <out:import href="{$conversions_xsl_filename}"/>
                    </xsl:if>
                    <!--<out:output method="xhtml" indent="yes" doctype-public="-//W3C//DTD HTML 4.0 Transitional//EN" doctype-system="http://www.w3.org/TR/html4/loose.dtd" />-->
                    <out:output method="xml" indent="yes" encoding="UTF-8"/>

                    <xsl:call-template name="create-section-body-template">
                        <xsl:with-param name="contentToOutput" select="$htmlResult/*"/>
                    </xsl:call-template>

                    <out:template name="section_title">
                        <xsl:value-of select="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@id" xmlns:fm="http://www.hyfinity.com/formmaker"/>
                    </out:template>
                    <out:template name="page_scripts">
                        <xsl:call-template name="output-script-includes"/>
                    </out:template>
                    <xsl:call-template name="create-css-imports"/>
                    <out:template name="page_meta_tags">
                        <xsl:comment>Insert any required page specific meta tags in here</xsl:comment>
                    </out:template>


                    <!-- Output a root template if this is a subsection page without a skin defined.
                         In all other situations, the skin should provide this initial template.-->
                    <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id][@type = 'subsection' and not(fm:skin != '')]" xmlns:fm="http://www.hyfinity.com/formmaker">
                        <out:template match="/">
                            <div class="subsection_container" id="subsection_{$page_id}">
                                <out:call-template name="section_body"/>
                            </div>
                        </out:template>
                    </xsl:if>

                    <!-- check if we need to output the templates for maintaining dynamic data -->
                    <xsl:if test="/fm:transform_structure/fm:data_from_form/fm:actions/fm:action//xsl:apply-templates[@mode = 'maintain-dynamic-content']"  xmlns:fm="http://www.hyfinity.com/formmaker">
                        <xsl:call-template name="output-maintain-dynamic-data-templates" />
                    </xsl:if>
                </out:stylesheet>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Copy the stylesheet tag and templates from the existing output file -->
    <xsl:template match="xsl:stylesheet" mode="copyExistingFile">
        <xsl:param name="newContent"/>
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:comment>Some information has been retained from the existing file!</xsl:comment>
            <!-- Set up skin import statement -->
            <xsl:choose xmlns:fm="http://www.hyfinity.com/formmaker">
                <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:skin != ''">
                    <out:import href="{$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:skin}"/>
                </xsl:when>
                <xsl:otherwise>
                    <!-- don't use the default skin if this is a subsection page -->
                    <xsl:if test="not($appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection')" >
                        <out:import href="{$appConfig/fm:application/fm:defaults/fm:skin}"/>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>
            <!-- import the conversion utility XSL if needed -->
            <xsl:variable name="conversionRequired">
                <xsl:call-template name="checkConversionRequired"/>
            </xsl:variable>
            <xsl:if test="$conversionRequired = 'true'">
                <out:import href="{$conversions_xsl_filename}"/>
            </xsl:if>
            <xsl:copy-of select="*[(local-name() != 'template') and (local-name() != 'import')]"/>
            <!-- copy existing templates except the 'section_body' and css_imports.  Also exclude the script templates here, as they will be handled below-->
            <xsl:copy-of select="xsl:template[(@name and (@name != 'section_body') and (@name != 'css_imports') and (@name != 'page_scripts') and (@name != 'client_scripts')) or (@match and (@match != '/'))]"/>

            <!-- Work out whether to create or copy a default root template for subsection pages -->
            <xsl:choose>
                <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" xmlns:fm="http://www.hyfinity.com/formmaker">
                    <xsl:choose>
                        <!-- When a skin has been specified dont use a root template -->
                        <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:skin != ''">
                            <!-- do nothing -->
                        </xsl:when>
                        <!-- otherwise, if there is one in the existing file, use that -->
                        <xsl:when test="xsl:template[@match = '/']" >
                            <xsl:copy-of select="xsl:template[@match = '/']"/>
                        </xsl:when>
                        <!-- otherwise, create a new one -->
                        <xsl:otherwise>
                            <out:template match="/">
                                <div class="subsection_container" id="subsection_{$page_id}">
                                    <out:call-template name="section_body"/>
                                </div>
                            </out:template>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:copy-of select="xsl:template[@match = '/']"/>
                </xsl:otherwise>
            </xsl:choose>

            <!-- Check whether the CSS stuff has been overridden and if so copy
                 the template from the exisiting file - otherwise recreate using FM settings-->
            <xsl:choose>
                <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:css_override = 'true'" xmlns:fm="http://www.hyfinity.com/formmaker">
                    <xsl:copy-of select="xsl:template[@name = 'css_imports']"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="create-css-imports"/>
                </xsl:otherwise>
            </xsl:choose>


            <!-- Handle script templates
                 We now always generate a 'page_scripts' template including all the script files defined
                 on the FM app map screen relating to this page.
                 To handle backwards compatability, when a page is regenerated and the previous XSL contains
                 a (non empty) client_scripts template, then that template will be copied across and called
                 from the generated page_scripts template. -->
            <out:template name="page_scripts">
                <xsl:call-template name="output-script-includes"/>

                <xsl:if test="xsl:template[(@name = 'client_scripts') and ((. != '') or *)]">
                    <out:call-template name="client_scripts"/>
                </xsl:if>
            </out:template>

            <xsl:if test="xsl:template[(@name = 'client_scripts') and ((. != '') or *)]">
                <xsl:copy-of select="xsl:template[@name = 'client_scripts']"/>
            </xsl:if>



            <!-- Create the new 'section_body' template -->
            <xsl:call-template name="create-section-body-template">
                <xsl:with-param name="contentToOutput" select="$newContent"/>
            </xsl:call-template>

            <!-- check if we need to output the templates for maintaining dynamic data
                 We only do this for an existing page if they are needed, and dont already exist in the XSL -->
            <xsl:if test="/fm:transform_structure/fm:data_from_form/fm:actions/fm:action//xsl:apply-templates[@mode = 'maintain-dynamic-content'] and
                         not(xsl:template[@mode = 'maintain-dynamic-content'])" xmlns:fm="http://www.hyfinity.com/formmaker" >
                <xsl:call-template name="output-maintain-dynamic-data-templates" />
            </xsl:if>

        </xsl:copy>
    </xsl:template>

    <xsl:template name="create-css-imports" xmlns:fm="http://www.hyfinity.com/formmaker">
        <out:template name="css_imports">

            <!-- check if it is still the old style - hopefully this wont be used any more, but just in case.. -->
            <xsl:choose>
                <xsl:when test="$appConfig/fm:application/fm:defaults/fm:css[. != '']">
                    <link rel="stylesheet" type="text/css" href="css/{$appConfig/fm:application/fm:defaults/fm:css}"></link>
                    <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:css[(. != '') and (. != $appConfig/fm:application/fm:defaults/fm:css)]" >
                        <link rel="stylesheet" type="text/css" href="css/{$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:css}"></link>
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                    <!-- New format supporting multiple CSS files for both application and page.-->

                    <xsl:variable name="cssFiles">
                        <files>
                            <xsl:for-each select="$appConfig/fm:application/fm:defaults/fm:css_files/fm:css_file[. != '']" >
                                <link rel="stylesheet" type="text/css" href="{.}"></link>
                            </xsl:for-each>
                            <xsl:for-each select="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:css_files/fm:css_file[(. != '') and (. != $appConfig/fm:application/fm:defaults/fm:css_files/fm:css_file)]" >
                                <link rel="stylesheet" type="text/css" href="{.}"></link>
                            </xsl:for-each>

                            <!-- If this page has any partial pages on it, then we also need to include any css files
                                 assigned to the partial page -->
                            <xsl:call-template name="output-partial-page-css-includes" >
                                <xsl:with-param name="parentPageId" select="$page_id"/>
                            </xsl:call-template>
                        </files>
                    </xsl:variable>


                    <xsl:variable name="cssFileSet" select="exsl:node-set($cssFiles)"/>
                    <xsl:for-each select="$cssFileSet/files/link[not(@href = preceding-sibling::link/@href)]" >
                        <xsl:copy-of select="."/>
                    </xsl:for-each>

                </xsl:otherwise>
            </xsl:choose>

        </out:template>
    </xsl:template>

    <xsl:template name="output-partial-page-css-includes" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="parentPageId" />

        <!-- Loop through each partial page -->
        <xsl:for-each select="$appConfig/fm:application/fm:pages/fm:page[(@type = 'subsection') and (@id = $appConfig/fm:application/fm:links/fm:link[fm:source = $parentPageId and (fm:source/@type = 'Page') and (fm:target/@type = 'Page')]/fm:target)]" >

            <!-- Output each script include -->
            <xsl:for-each select="fm:css_files/fm:css_file">
                <link rel="stylesheet" type="text/css" href="{.}"></link>
            </xsl:for-each>

            <!-- Check for any more child partial pages -->
            <xsl:call-template name="output-partial-page-css-includes" >
                <xsl:with-param name="parentPageId" select="@id"/>
            </xsl:call-template>

        </xsl:for-each>

    </xsl:template>







    <!-- Outputs a script tag for every script file include that applies to this page.
         This takes both the application wide and page specific entries from the application file. -->
    <xsl:template name="output-script-includes" xmlns:fm="http://www.hyfinity.com/formmaker">

        <xsl:variable name="scriptFiles">
            <files>
                <xsl:for-each select="$appConfig/fm:application/fm:defaults/fm:script_files/fm:script_file">
                    <script type="text/javascript" src="{.}"> </script>
                </xsl:for-each>
                <xsl:for-each select="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/fm:script_files/fm:script_file">
                    <script type="text/javascript" src="{.}"> </script>
                </xsl:for-each>

                <!-- If this page has any partial pages on it, then we also need to include any scripts
                     assigned to the partial page -->
                <xsl:call-template name="output-partial-page-script-includes" >
                    <xsl:with-param name="parentPageId" select="$page_id"/>
                </xsl:call-template>
            </files>
        </xsl:variable>

        <xsl:variable name="scriptFileSet" select="exsl:node-set($scriptFiles)"/>
        <xsl:for-each select="$scriptFileSet/files/script[not(@src = preceding-sibling::script/@src)]" >
            <xsl:copy-of select="."/>
        </xsl:for-each>

    </xsl:template>

    <xsl:template name="output-partial-page-script-includes" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="parentPageId" />

        <!-- Loop through each partial page -->
        <xsl:for-each select="$appConfig/fm:application/fm:pages/fm:page[(@type = 'subsection') and (@id = $appConfig/fm:application/fm:links/fm:link[fm:source = $parentPageId and (fm:source/@type = 'Page') and (fm:target/@type = 'Page')]/fm:target)]" >

            <!-- Output each script include -->
            <xsl:for-each select="fm:script_files/fm:script_file">
                <script type="text/javascript" src="{.}"> </script>
            </xsl:for-each>

            <!-- Check for any more child partial pages -->
            <xsl:call-template name="output-partial-page-script-includes" >
                <xsl:with-param name="parentPageId" select="@id"/>
            </xsl:call-template>

        </xsl:for-each>

    </xsl:template>




    <!-- Creates the main section body tempalte that will output all the details of this page.
         For subsection pages this template will define all the namespaces, as the xform tags are not used-->
    <xsl:template name="create-section-body-template">
        <xsl:param name="contentToOutput"/>

        <xsl:choose>
            <xsl:when test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" xmlns:fm="http://www.hyfinity.com/formmaker">
                <!--<xsl:text disable-output-escaping="yes">&lt;xsl:template name="section_body" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" </xsl:text>
                <xsl:for-each select="$bindings/fm:xforms/namespace::*">
                    <xsl:if test="(name(.) != '') and (name(.) != 'xml') and (name(.) != 'xsl')">
                        <xsl:text>xmlns:</xsl:text>
                        <xsl:value-of select="name(.)"/>
                        <xsl:text disable-output-escaping="yes">="</xsl:text>
                        <xsl:value-of select="."/>
                        <xsl:text disable-output-escaping="yes">" </xsl:text>
                    </xsl:if>
                </xsl:for-each>
                <xsl:text disable-output-escaping="yes">&gt;</xsl:text>
                    <xsl:apply-templates select="$contentToOutput"/>
                    <script type="text/javascript">
                        <xsl:call-template name="createValidationAttributesScript"/>
                    </script>
                <xsl:text disable-output-escaping="yes">&lt;/xsl:template&gt;</xsl:text>-->

                <out:template name="section_body">
                    <xsl:call-template name="create-xform-wrapper">
                        <xsl:with-param name="content" select="$contentToOutput"/>
                    </xsl:call-template>
                    <script type="text/javascript">
                        <xsl:call-template name="createValidationAttributesScript"/>
                    </script>
                </out:template>

            </xsl:when>
            <xsl:otherwise>
                <out:template name="section_body">
                    <xsl:apply-templates select="$contentToOutput"/>
                    <script type="text/javascript">
                        <out:text disable-output-escaping="yes">
                            <xsl:call-template name="createFormSubmissionScript"/>
                        </out:text>
                        <xsl:call-template name="createValidationAttributesScript"/>
                    </script>
                </out:template>
            </xsl:otherwise>
        </xsl:choose>



    </xsl:template>



    <!--
        When using the maintain dynamic data option, we need to copy a chunk of data from the
        factbase into the xform instance structure.
        Unfortunately, there is a bug in xalan that means the namespaces of this data can get chanegd in
        some situations when using an xsl:copy-of statement.
        Instead we use a couple of templates that recursively output the fragment in a way that
        should maintain the correct namespaces.
        This just outputs the templates used for this recursive copy.
    -->
    <xsl:template name="output-maintain-dynamic-data-templates">
        <out:template match="node() | @*" mode="maintain-dynamic-content">
            <out:copy>
                <out:apply-templates select="@*" mode="maintain-dynamic-content"/>
                <out:apply-templates mode="maintain-dynamic-content"/>
            </out:copy>
        </out:template>
        <out:template match="*" mode="maintain-dynamic-content">
            <out:element name="{{name()}}" namespace="{{namespace-uri()}}">
                <out:apply-templates select="@*" mode="maintain-dynamic-content"/>
                <out:apply-templates mode="maintain-dynamic-content"/>
            </out:element>
        </out:template>
    </xsl:template>




    <xsl:template name="createFormSubmissionScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:text disable-output-escaping="yes">
            /*
             * Do NOT edit or add to the following script.
             * It will be recreated each time the page is regenerated!
             * Instead use the Application Map screen to add an external
             * script file to your page, and place your code in there.
             */

            hyf.validation.config =
            {
                form: document.</xsl:text>
                <xsl:value-of select="$form_id"/>
                <xsl:text disable-output-escaping="yes">,
                errorDisplayMethod: '</xsl:text>
                <xsl:choose>
                    <xsl:when test="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method != ''">
                        <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method"/>
                    </xsl:when>
                    <xsl:otherwise>none</xsl:otherwise>
                </xsl:choose>
                <xsl:text disable-output-escaping="yes">'</xsl:text>
                <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:field_error/@style != ''">
                    <xsl:text disable-output-escaping="yes">,
                errorDisplayFieldStyle: '</xsl:text>
                    <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:field_error/@style"/>
                    <xsl:text disable-output-escaping="yes">'</xsl:text>
                </xsl:if>
                <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:field_error/@class != ''">
                    <xsl:text disable-output-escaping="yes">,
                errorDisplayFieldClass: '</xsl:text>
                    <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:field_error/@class"/>
                    <xsl:text disable-output-escaping="yes">'</xsl:text>
                </xsl:if>
                <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:message_error/@style != ''">
                    <xsl:text disable-output-escaping="yes">,
                errorDisplayMessageStyle: '</xsl:text>
                    <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:message_error/@style"/>
                    <xsl:text disable-output-escaping="yes">'</xsl:text>
                </xsl:if>
                <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:message_error/@class != ''">
                    <xsl:text disable-output-escaping="yes">,
                errorDisplayMessageClass: '</xsl:text>
                    <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:message_error/@class"/>
                    <xsl:text disable-output-escaping="yes">'</xsl:text>
                </xsl:if>
                <xsl:text disable-output-escaping="yes">,
                errorDisplayShowMessage: </xsl:text>
                <xsl:choose>
                    <xsl:when test="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method/@show_message = 'true'">
                        <xsl:text disable-output-escaping="yes">true</xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text disable-output-escaping="yes">false</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:text disable-output-escaping="yes">,
                errorDisplayShowAlerts: </xsl:text>
                <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method/@alert"/>
                <xsl:text disable-output-escaping="yes">,
                errorDisplayValidationMode: '</xsl:text>
                <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method/@validation_mode"/>
                <xsl:text disable-output-escaping="yes">'</xsl:text>

                <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:top_level_message/@required = 'true'">
                    <xsl:text disable-output-escaping="yes">,
                errorDisplayMessageLocation: document.getElementById("</xsl:text>
                    <xsl:value-of select="$form_id"/>
                    <xsl:text disable-output-escaping="yes">MessageDiv")</xsl:text>
                    <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:top_level_message != ''">
                        <xsl:text disable-output-escaping="yes">,
                errorDisplayMessageString: '</xsl:text>
                        <xsl:value-of select="$formStructure/fm:structure/fm:form_details/fm:top_level_message"/>
                        <xsl:text disable-output-escaping="yes">'</xsl:text>
                    </xsl:if>
                </xsl:if>
                <xsl:text disable-output-escaping="yes">
            }
        </xsl:text>




        <xsl:if test="$formStructure/fm:structure//fm:repeat/fm:sort/@default = 'true'">
            <xsl:text disable-output-escaping="yes"><![CDATA[
            function sortResults(repeatName, sortField)
            {
                /* update the value in the sort control hidden field*/
                var textBox = eval('document.forms[0].'+repeatName+'_sort_field');
                textBox.value = sortField;

                /*submit the form*/
                hyf.FMAction.handleFormSubmission();
            }
        ]]></xsl:text>
        </xsl:if>

    </xsl:template>


    <!--

    ***********************************************************************
    START templates for outputting javascript to create validation attributes
    ***********************************************************************

    -->
    <!-- Create script to output all the _.... attributes created for javascript validation.
         This process is needed as outputting the attributes directly creates invalid html -->
    <xsl:template name="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">

        <!-- for each created just to ensure the correct namespaces are present -->
        <xsl:text disable-output-escaping="yes">&lt;xsl:for-each select="." xmlns:xsl="http://www.w3.org/1999/XSL/Transform"</xsl:text>
        <xsl:for-each select="$bindings/fm:xforms/namespace::*">
            <xsl:if test="(name(.) != '') and (name(.) != 'xml') and (name(.) != 'xsl')">
                <xsl:text> xmlns:</xsl:text>
                <xsl:value-of select="name(.)"/>
                <xsl:text disable-output-escaping="yes">="</xsl:text>
                <xsl:value-of select="."/>
                <xsl:text disable-output-escaping="yes">"</xsl:text>
            </xsl:if>
        </xsl:for-each>
        <xsl:text disable-output-escaping="yes">&gt;</xsl:text>

        <xsl:call-template name="createEventsScript"/>

        <xsl:apply-templates select="$htmlResult/*" mode="createValidationAttributesScript"/>

        <xsl:text disable-output-escaping="yes">&lt;/xsl:for-each&gt;</xsl:text>
    </xsl:template>
    <!--<xsl:template match="repeat_code_placeholder[.//input[@type != 'button'] or .//select or .//textarea]" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">-->
    <xsl:template match="repeat_code_placeholder" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="repeatName" select="@name"/>
        <!-- Create for-each statement -->
        <out:for-each select="{$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath}">
            <xsl:if test="not($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath = '')">
                <xsl:attribute name="select">repeat_xpath_not_specified</xsl:attribute>
            </xsl:if>
            <out:variable name="{$repeatName}LoopEntry" select="."/>
            <xsl:choose>
                <xsl:when test="ancestor::repeat_code_placeholder">
                    <out:variable name="{$repeatName}LoopCount">
                        <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        <out:text>
                            <xsl:value-of select="$repeatName"/>
                        </out:text>
                        <out:value-of select="position()"/>
                    </out:variable>
                </xsl:when>
                <xsl:otherwise>
                    <out:variable name="{$repeatName}LoopCount">
                        <out:text>
                            <xsl:value-of select="$repeatName"/>
                        </out:text>
                        <out:value-of select="position()"/>
                    </out:variable>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates select="*" mode="createValidationAttributesScript"/>
        </out:for-each>
    </xsl:template>
    <xsl:template match="repeat_code_default_row_placeholder" xmlns:fm="http://www.hyfinity.com/formmaker" mode="createValidationAttributesScript">
        <xsl:variable name="repeatName" select="@name"/>
        <out:for-each select="/*[1]">
            <out:variable name="{$repeatName}LoopEntry" select="."/>
            <out:variable name="{$repeatName}LoopCount">
                <xsl:if test="ancestor::repeat_code_placeholder">
                    <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                </xsl:if>
                <out:text>
                    <xsl:value-of select="$repeatName"/>
                    <xsl:text>BlankEntry</xsl:text>
                </out:text>
            </out:variable>

            <xsl:apply-templates select="preceding::repeat_code_placeholder[@name = $repeatName]/*" mode="createValidationAttributesScript"/>
        </out:for-each>
    </xsl:template>
    <!-- match each group to see if it should be hidden - if so dont output any attribute creation javascript
         for the contents of the group, becuase it will fail as teh elements wont exist. -->
    <xsl:template match="*[@_use = 'group']" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="groupName" select="@id"/>
        <xsl:variable name="configGroup" select="$formStructure/fm:structure//fm:group[@name = $groupName]"/>

        <xsl:variable name="tabField">
            <xsl:if test="($configGroup/@conditional_display = 'true') and ($formStructure/fm:structure//fm:element[not(ancestor::fm:path) and (@name = $configGroup/@control_field)]/fm:field/fm:input = 'tabs')">
                <xsl:value-of select="$configGroup/@control_field"/>
                <xsl:text>_tab_</xsl:text>
                <xsl:value-of select="$configGroup/@control_field_value"/>
            </xsl:if>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="($configGroup/@hide = 'true') and ($bindings/fm:xforms/fm:group[@name = $groupName]/fm:hide_xpath != '')">
                <!-- Create runtime hide test -->
                <out:if test="not({$bindings/fm:xforms/fm:group[@name = $groupName]/fm:hide_xpath})">

                    <!-- check for any custom attributes -->
                    <xsl:if test="$configGroup/fm:custom_attributes/fm:attribute or ($tabField != '')" >
                        <out:text>field = document.getElementById('</out:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                            <xsl:value-of select="$groupName"/>
                            <xsl:text>');</xsl:text>
                        </out:text>
                        <xsl:call-template name="createCustomAttributeAtributesScript">
                            <xsl:with-param name="configElement" select="$configGroup"/>
                        </xsl:call-template>
                        <xsl:if test="$tabField != ''">
                            <out:text>
                            <xsl:text>field.setAttribute('_use', 'tabPane');</xsl:text>
                            <xsl:text>field.setAttribute('_tabField', "</xsl:text>
                            </out:text>
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                            </xsl:if>
                            <out:text>
                            <xsl:value-of select="$tabField"/>
                            <xsl:text>");</xsl:text>
                            </out:text>
                        </xsl:if>
                    </xsl:if>

                    <xsl:apply-templates select="*" mode="createValidationAttributesScript"/>
                </out:if>
            </xsl:when>
            <xsl:otherwise>

                <!-- check for any custom attributes -->
                <xsl:if test="$configGroup/fm:custom_attributes/fm:attribute or ($tabField != '')" >
                    <out:text>field = document.getElementById('</out:text>
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                    </xsl:if>
                    <out:text>
                        <xsl:value-of select="$groupName"/>
                        <xsl:text>');</xsl:text>
                    </out:text>
                    <xsl:call-template name="createCustomAttributeAtributesScript">
                        <xsl:with-param name="configElement" select="$configGroup"/>
                    </xsl:call-template>
                    <xsl:if test="$tabField != ''">
                        <out:text>
                        <xsl:text>field.setAttribute('_use', 'tabPane');</xsl:text>
                        <xsl:text>field.setAttribute('_tabField', "</xsl:text>
                        </out:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                        <xsl:value-of select="$tabField"/>
                        <xsl:text>");</xsl:text>
                        </out:text>
                    </xsl:if>
                </xsl:if>

                <xsl:apply-templates select="*" mode="createValidationAttributesScript"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- match any container being used to possibly hide a field. for these we need to output a check to stop
        outputting the attribute creation script if the field has been hidden -->
    <xsl:template match="*[@_use = 'elementHideContainer']" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elemName" select="@_element"/>
        <xsl:variable name="configElem" select="$formStructure/fm:structure//fm:element[@name = $elemName and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <xsl:when test="($configElem/@hide = 'true') and ($bindings/fm:xforms/fm:element[@name = $elemName]/fm:hide_xpath != '')">
                <!-- Create runtime hide test -->
                <out:if test="not({$bindings/fm:xforms/fm:element[@name = $elemName]/fm:hide_xpath})">
                    <xsl:apply-templates select="*" mode="createValidationAttributesScript"/>
                </out:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="*" mode="createValidationAttributesScript"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="input[@type != 'button'] | select | textarea" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="fieldID" select="@id"/>
        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $fieldID and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <xsl:when test="(($configElement/fm:field/fm:input = 'radio') or ($configElement/fm:field/fm:input = 'multipleCheckbox')) and ($configElement/fm:field/fm:input/@options = 'dynamic')">
                <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:repeat_xpath}">
                    <xsl:call-template name="createFieldValidationAttributesScript">
                        <xsl:with-param name="multiple">true</xsl:with-param>
                        <xsl:with-param name="configElement" select="$configElement"/>
                    </xsl:call-template>
                </out:for-each>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="createFieldValidationAttributesScript">
                    <xsl:with-param name="multiple">false</xsl:with-param>
                    <xsl:with-param name="configElement" select="$configElement"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:for-each select="option[@_editable = 'true']">
            <out:text>
            field = document.getElementById('</out:text>
            <xsl:if test="ancestor::repeat_code_placeholder">
                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
            </xsl:if>
            <out:text>
                <xsl:value-of select="@id"/>
                <xsl:text>');field.setAttribute('_editable', 'true'); field.setAttribute('_initialValue', '</xsl:text>
                <xsl:value-of select="@_initialValue"/>
                <xsl:text>');</xsl:text>
            </out:text>
        </xsl:for-each>
    </xsl:template>
    <!-- match output fields to check if they need any custom attributes -->
    <xsl:template match="span[@_use = 'output'] | a" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="fieldID" select="@id"/>
        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $fieldID and not(ancestor::fm:path)]"/>

        <xsl:if test="$configElement/fm:custom_attributes/fm:attribute">

            <out:text>
            field = document.getElementById('</out:text>
            <xsl:if test="ancestor::repeat_code_placeholder">
                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
            </xsl:if>
            <out:text>
                <xsl:value-of select="$fieldID"/>
                <xsl:text>');</xsl:text>
            </out:text>

            <xsl:call-template name="createCustomAttributeAtributesScript">
                <xsl:with-param name="configElement" select="$configElement"/>
            </xsl:call-template>

        </xsl:if>

    </xsl:template>

    <xsl:template name="createFieldValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="multiple"/>
        <xsl:param name="configElement"/>
        <xsl:if test="@*[starts-with(local-name(), '_')]">
            <out:text>
            field = document.getElementById('</out:text>
            <xsl:if test="ancestor::repeat_code_placeholder">
                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
            </xsl:if>
            <out:text>
                <xsl:value-of select="@id"/>
            </out:text>
            <xsl:if test="$multiple = 'true'">
                <out:value-of select="position()"/>
            </xsl:if>
            <out:text>');</out:text>
            <xsl:for-each select="@*[starts-with(local-name(), '_') and (local-name() != '_validate') and (local-name() != '_element') and (local-name() != '_spanClass') and (local-name() != '_captionClass')]">
                <out:text disable-output-escaping="yes">
                    <xsl:text>field.setAttribute('</xsl:text>
                    <xsl:value-of select="local-name()"/>
                    <xsl:text>','</xsl:text>
                    <!-- escape any backslash \ characters as this is the Javascript escape character  -->
                    <xsl:variable name="escapedSlash">
                        <xsl:call-template name="replace-string">
                            <xsl:with-param name="text" select="."/>
                            <xsl:with-param name="replace" select="'\'"/>
                            <xsl:with-param name="with" select="'\\'"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <!-- now escape any apostraphe characters as we are using that as the string container -->
                    <xsl:call-template name="replace-string">
                        <xsl:with-param name="text" select="$escapedSlash"/>
                        <xsl:with-param name="replace">'</xsl:with-param>
                        <xsl:with-param name="with">\'</xsl:with-param>
                    </xsl:call-template>
                    <xsl:text>');</xsl:text>
                </out:text>
            </xsl:for-each>
            <!-- Check if there is an _element attribute (ie this is a checkbox within a select many)
                 and if so add the repeat count to the value-->
            <xsl:if test="@_element">
                <out:text disable-output-escaping="yes">field.setAttribute('_element','</out:text>
                <xsl:if test="ancestor::repeat_code_placeholder">
                    <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                </xsl:if>
                <out:text disable-output-escaping="yes">
                    <xsl:value-of select="@_element"/>
                    <xsl:text>');</xsl:text>
                </out:text>
            </xsl:if>
            <!-- handle the validate control attribute.  Only output the attribute if validation should be turned off -->
            <xsl:choose>
                <xsl:when test="($configElement/fm:constraints/@validate = 'dynamic') and ($bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:validate_xpath != '')">
                    <out:choose>
                        <out:when test="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:validate_xpath}">
                            <!--<out:text>field.setAttribute('_validate', 'true');</out:text>-->
                        </out:when>
                        <out:otherwise>
                            <out:text disable-output-escaping="yes">field.setAttribute('_validate', 'false');</out:text>
                        </out:otherwise>
                    </out:choose>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:if test="@_validate = 'false'">
                        <out:text disable-output-escaping="yes">field.setAttribute('_validate', '<xsl:value-of select="@_validate"/>');</out:text>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>

            <xsl:call-template name="createCustomAttributeAtributesScript">
                <xsl:with-param name="configElement" select="$configElement"/>
            </xsl:call-template>

        </xsl:if>

        <!-- check if we need to initialise this field as an expandable text area. -->
        <xsl:if test="$configElement/fm:field/fm:input[ . = 'textarea' and @expandable = 'true']">
            <out:text>field.onkeyup();</out:text>
        </xsl:if>

    </xsl:template>

    <!-- Outputs the script lines to create all the custom attributes defined for the given field/group.
        This assumes that the javascript 'field' object has already been defined to point to the correct object.
    -->
    <xsl:template name="createCustomAttributeAtributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>

        <!-- Check for custom attributes -->
        <xsl:if test="$configElement/fm:custom_attributes/fm:attribute">
            <out:text disable-output-escaping="yes">
                <xsl:for-each select="$configElement/fm:custom_attributes/fm:attribute">
                    <xsl:text>field.setAttribute('</xsl:text>
                    <xsl:value-of select="@name"/>
                    <xsl:text>', '</xsl:text>
                    <xsl:value-of select="@value"/>
                    <xsl:text>');</xsl:text>
                </xsl:for-each>

                <!-- Check for a dojoType custom attribute, and if so look to see if we need to
                     convert any of our constraints to the dojo format attributes -->
                <xsl:if test="(local-name($configElement) = 'element') and $configElement/fm:custom_attributes/fm:attribute[@name = 'dojoType']" >
                    <!-- output the dojo required attribute -->
                    <xsl:if test="not($configElement/fm:custom_attributes/fm:attribute[@name = 'required'])" >
                        <xsl:text>field.setAttribute('required', '</xsl:text>
                        <xsl:value-of select="$configElement/fm:constraints/fm:required"/>
                        <xsl:text>');</xsl:text>
                    </xsl:if>
                    <!-- output the dojo constraints attribute -->
                    <xsl:if test="not($configElement/fm:custom_attributes/fm:attribute[@name = 'constraints'])" >
                        <xsl:if test="$configElement/fm:constraints[(fm:minInclusive != '') or (fm:minExclusive != '') or (fm:maxInclusive != '') or (fm:maxExclusive != '')]" >
                            <xsl:text>field.setAttribute('constraints', '{</xsl:text>
                            <xsl:choose>
                                <xsl:when test="$configElement/fm:constraints/fm:minInclusive[. != '']" >
                                    <xsl:text>min:</xsl:text>
                                    <xsl:value-of select="$configElement/fm:constraints/fm:minInclusive" />
                                    <xsl:text>,</xsl:text>
                                </xsl:when>
                                <xsl:when test="$configElement/fm:constraints/fm:minExclusive[. != '']" >
                                    <xsl:text>min:</xsl:text>
                                    <xsl:value-of select="($configElement/fm:constraints/fm:minExclusive) + 1" />
                                    <xsl:text>,</xsl:text>
                                </xsl:when>
                            </xsl:choose>
                            <xsl:choose>
                                <xsl:when test="$configElement/fm:constraints/fm:maxInclusive[. != '']" >
                                    <xsl:text>max:</xsl:text>
                                    <xsl:value-of select="$configElement/fm:constraints/fm:maxInclusive" />
                                    <xsl:text>,</xsl:text>
                                </xsl:when>
                                <xsl:when test="$configElement/fm:constraints/fm:maxExclusive[. != '']" >
                                    <xsl:text>max:</xsl:text>
                                    <xsl:value-of select="($configElement/fm:constraints/fm:maxExclusive) - 1" />
                                    <xsl:text>,</xsl:text>
                                </xsl:when>
                            </xsl:choose>
                            <xsl:choose>
                                <xsl:when test="contains($configElement/fm:constraints/*[(local-name() = 'minInclusive') or (local-name() = 'minExclusive') or (local-name() = 'maxInclusive') or (local-name() = 'maxExclusive')], '.')" >
                                    <xsl:text>places:</xsl:text>
                                    <xsl:value-of select="string-length(substring-after($configElement/fm:constraints/*[(local-name() = 'minInclusive') or (local-name() = 'minExclusive') or (local-name() = 'maxInclusive') or (local-name() = 'maxExclusive')][contains(., '.')], '.'))"/>
                                </xsl:when>
                                <xsl:otherwise>places:0</xsl:otherwise>
                            </xsl:choose>
                            <xsl:text>}');</xsl:text>
                        </xsl:if>
                    </xsl:if>

                    <!-- Output the placeholder text for select fields if required -->
                    <xsl:if test="not($configElement/fm:custom_attributes/fm:attribute[@name = 'placeHolder'])" >
                        <xsl:if test="$configElement/fm:field/fm:input[(. = 'select') and (@dropdown_default != '')]" >
                            <xsl:text>field.setAttribute('placeHolder', '</xsl:text>
                            <xsl:value-of select="$configElement/fm:field/fm:input/@dropdown_default"/>
                            <!-- Also remove the initial 'option' tag that will be created as we dont want
                                 the user to be able to select this when the dojo control is rendered. -->
                            <xsl:text>');hyf.util.initFilteringSelect(field);</xsl:text>
                        </xsl:if>
                    </xsl:if>

                    <xsl:if test="$configElement/fm:field/fm:input/@size[. != '']" >
                        <xsl:text>if (field.style.cssText.indexOf('width') == -1) </xsl:text>
                        <xsl:text>field.style.cssText += ';width:</xsl:text>
                        <xsl:choose>
                            <xsl:when test="contains($configElement/fm:custom_attributes/fm:attribute[@name = 'dojoType']/@value, 'NumberSpinner')" >
                                <xsl:value-of select="$configElement/fm:field/fm:input/@size + 2" />
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$configElement/fm:field/fm:input/@size + 1" />
                            </xsl:otherwise>
                        </xsl:choose>
                        <xsl:text>em;';</xsl:text>
                    </xsl:if>

                </xsl:if>

            </out:text>
        </xsl:if>
    </xsl:template>

    <!-- match any message containers to create javascript to copy across the hint messages etc. -->
    <xsl:template match="*[substring(@id, string-length(@id) - string-length('_message_container') + 1) = '_message_container']" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elementID" select="substring(@id, 1, string-length(@id) - string-length('_message_container'))"/>
        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]"/>
        <xsl:if test="$configElement/fm:error/fm:hint/@use = 'true' or $configElement/fm:error/fm:server/@use = 'true'">
            <out:text disable-output-escaping="yes">field = document.getElementById('</out:text>
            <xsl:if test="ancestor::repeat_code_placeholder">
                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
            </xsl:if>
            <out:text disable-output-escaping="yes">
                <xsl:value-of select="@id"/>
                <xsl:text>');
                </xsl:text>
                <xsl:for-each select="@*[starts-with(local-name(), '_')]">
                    <xsl:text>field.setAttribute('</xsl:text>
                    <xsl:value-of select="local-name()"/>
                    <xsl:text>','</xsl:text>
                    <xsl:value-of select="."/>
                    <xsl:text>');</xsl:text>
                </xsl:for-each>
            </out:text>
        </xsl:if>
        <!-- Check if any server message inforation needs to be added to the element -->
        <xsl:if test="$configElement/fm:error/fm:server/@use = 'true'">
            <!-- check if the server message exists -->
            <xsl:if test="$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath != ''">
                <out:if test="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath} != ''">
                    <!-- set server message attribute -->
                    <out:text disable-output-escaping="yes">field.setAttribute('_serverMessage','</out:text>
                    <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath}">
                        <out:if test="position() != 1">
                            <out:text disable-output-escaping="yes">
                                <xsl:text>&lt;br/&gt;</xsl:text>
                            </out:text>
                        </out:if>
                        <out:value-of select="."/>
                    </out:for-each>
                    <out:text disable-output-escaping="yes">');
                        <xsl:text>field.setAttribute('_serverMessageShow','true');</xsl:text>
                        <xsl:text>field.setAttribute('_serverMessageRetain','</xsl:text>
                        <xsl:value-of select="$configElement/fm:error/fm:server/@retain"/>
                        <xsl:text>');</xsl:text>
                        <!-- set the container styling to the server message styles -->
                        <xsl:if test="$configElement/fm:error/fm:server/@style != ''">
                            <xsl:text>field.setAttribute('_serverStyle','</xsl:text>
                            <xsl:value-of select="$configElement/fm:error/fm:server/@style"/>
                            <xsl:text>');</xsl:text>
                        </xsl:if>
                        <xsl:if test="$configElement/fm:error/fm:server/@class != ''">
                            <xsl:text>field.setAttribute('_serverClass','</xsl:text>
                            <xsl:value-of select="$configElement/fm:error/fm:server/@class"/>
                            <xsl:text>');</xsl:text>
                        </xsl:if>
                    </out:text>
                    <!-- check the message display format for this form and put the server message in the relevant place -->
                    <xsl:if test="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method = 'tooltip'">
                        <out:text disable-output-escaping="yes">field.setAttribute('_tipMessage', field.getAttribute('_serverMessage'));</out:text>
                    </xsl:if>
                </out:if>
            </xsl:if>
        </xsl:if>
    </xsl:template>
    <xsl:template match="@* | node()" mode="createValidationAttributesScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:apply-templates select="@*" mode="createValidationAttributesScript"/>
        <xsl:apply-templates select="*" mode="createValidationAttributesScript"/>
    </xsl:template>
    <!--

    ***********************************************************************
    END templates for outputting javascript to create validation attributes
    **********************************************************************

    -->
    <!-- Create the javascript needed for the combobbox for the given element -->
    <xsl:template match="fm:element" mode="createComboBoxScript" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="repeatName"/>
        <xsl:param name="elementName"/>
        <out:variable name="{@name}comboBoxName">
            <xsl:if test="$repeatName != ''">
                <out:value-of select="${$repeatName}LoopCount"/>
            </xsl:if>
            <xsl:value-of select="$elementName"/>
        </out:variable>
        <script type="text/javascript">
            <out:text>var </out:text>
            <out:value-of select="${@name}comboBoxName"/>
            <out:text>_dropdown = document.getElementById('</out:text>
            <out:value-of select="${@name}comboBoxName"/>
            <out:text>');&#10;var </out:text>
            <out:value-of select="${@name}comboBoxName"/>
            <out:text>_selectedIndex = </out:text>
            <out:value-of select="${@name}comboBoxName"/>
            <out:text>_dropdown.selectedIndex;&#10;var </out:text>
            <out:value-of select="${@name}comboBoxName"/>
            <out:text>_changeCause = 'MANUAL_CLICK';&#10;</out:text>
            <!-- Check if the event handlers that are about to be defined below have already been
                         specified for the element, and if so create a new function so that both the existing,
                         and new event handlers are called-->
            <xsl:choose>
                <xsl:when test="fm:events/fm:onkeypress != ''">
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.oldOnKeyPress = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeypress;&#10;function </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnKeyPress(evt)&#10;{&#10;comboBoxKeyPress(evt);&#10;this.oldOnKeyPress(evt);&#10;}&#10;</out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeypress = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnKeyPress;&#10;</out:text>
                </xsl:when>
                <xsl:otherwise>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeypress = comboBoxKeyPress;&#10;</out:text>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="fm:events/fm:onkeyup != ''">
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.oldOnKeyUp = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeyup;&#10;function </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnKeyUp(evt)&#10;{&#10;comboBoxKeyUp(evt);&#10;this.oldOnKeyUp(evt);&#10;}&#10;</out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeyup = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnKeyUp;&#10;</out:text>
                </xsl:when>
                <xsl:otherwise>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeyup = comboBoxKeyUp;&#10;</out:text>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="fm:events/fm:onkeydown != ''">
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.oldOnKeyDown = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeydown;&#10;function </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnKeyDown(evt)&#10;{&#10;comboBoxKeyDown(evt);&#10;this.oldOnKeyDown(evt);&#10;}&#10;</out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeydown = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnKeyDown;&#10;</out:text>
                </xsl:when>
                <xsl:otherwise>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onkeydown = comboBoxKeyDown;&#10;</out:text>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="fm:events/fm:onchange != ''">
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.oldOnChange = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onchange;&#10;function </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnChange(evt)&#10;{&#10;comboBoxChange(evt);&#10;this.oldOnChange(evt);&#10;}&#10;</out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onchange = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnChange;&#10;</out:text>
                </xsl:when>
                <xsl:otherwise>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onchange = comboBoxChange;&#10;</out:text>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="fm:events/fm:onmousedown != ''">
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.oldOnMouseDown = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onmousedown;&#10;function </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnMouseDown(evt)&#10;{&#10;comboBoxMouseDown(evt);&#10;this.oldOnMouseDown(evt);&#10;}&#10;</out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onmousedown = </out:text>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdownOnMouseDown;&#10;</out:text>
                </xsl:when>
                <xsl:otherwise>
                    <out:value-of select="${@name}comboBoxName"/>
                    <out:text>_dropdown.onmousedown = comboBoxMouseDown;&#10;</out:text>
                </xsl:otherwise>
            </xsl:choose>
        </script>
    </xsl:template>
    <!-- Check if the value entered by the user for this element should be converted in any way
         Currently only checks for date or case conversions.

         If the configElement is specified, this checks if that particular element requires conversions.
         If not present, this checks if any element in the structure requires conversion.
         -->
    <xsl:template name="checkConversionRequired" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:choose>
            <xsl:when test="$configElement">
                <xsl:choose>
                    <xsl:when test="($configElement/fm:constraints/fm:type = 'date') and ($configElement/fm:data_formats/fm:date[(@data != @display) and (@data != '') and (@display != '')])">true</xsl:when>
                    <xsl:when test="($configElement/fm:constraints/fm:type = 'string') and ($configElement/fm:data_formats/fm:case[(@display != 'preserve') and (@display != '')])">true</xsl:when>
                    <xsl:otherwise>false</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="$formStructure/fm:structure//fm:element[(fm:constraints/fm:type = 'date') and (fm:data_formats/fm:date[(@data != @display) and (@data != '') and (@display != '')])]">true</xsl:when>
                    <xsl:when test="$formStructure/fm:structure//fm:element[(fm:constraints/fm:type = 'string') and (fm:data_formats/fm:case[(@display != 'preserve') and (@display != '')])]">true</xsl:when>
                    <xsl:otherwise>false</xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Create a date picker link providing a calendar popup for this field if required -->
    <xsl:template match="fm:element" mode="createDatePicker" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="repeatName"/>
        <!-- check the data value is a date type and that a date picker has been requested -->
        <xsl:if test="fm:constraints/fm:type = 'date'">
            <xsl:if test="fm:field/fm:date_picker/@required = 'true'">

                <xsl:choose>
                    <!-- Do we need to consider other types, eg select boxes -->
                    <xsl:when test="(fm:field/fm:input = 'text') or (fm:field/fm:input = 'textarea')">
                        <xsl:variable name="calendarConfigString">
                            <xsl:text>hyf.calendar.config['</xsl:text>
                            <xsl:value-of select="@name"/>
                            <xsl:text>'] = {popup: </xsl:text>
                            <xsl:value-of select="fm:field/fm:date_picker/@popup" />
                            <xsl:text>, type: '</xsl:text>
                            <xsl:value-of select="fm:field/fm:date_picker/@type" />
                            <xsl:text>', hasDropDowns: </xsl:text>
                            <xsl:value-of select="contains(fm:data_formats/fm:date/@display,'#')" />
                            <xsl:text>, dataFormat: '</xsl:text>

                            <!-- escape any apostraphes or back slashes in the format string, as
                                 we are placing it in a javascript string -->
                            <xsl:call-template name="replace-string">
                                <xsl:with-param name="text">
                                    <xsl:call-template name="replace-string">
                                        <xsl:with-param name="text" select="fm:data_formats/fm:date/@data"/>
                                        <xsl:with-param name="replace" select="'\'"/>
                                        <xsl:with-param name="with" select="'\\'"/>
                                    </xsl:call-template>
                                </xsl:with-param>
                                <xsl:with-param name="replace">'</xsl:with-param>
                                <xsl:with-param name="with">\'</xsl:with-param>
                            </xsl:call-template>

                            <!--<xsl:value-of select="fm:data_formats/fm:date/@data" />-->
                            <xsl:text>'</xsl:text>
                            <xsl:if test="fm:constraints/fm:maxExclusive != ''" >
                                <xsl:text>, maxExclusive: '</xsl:text>
                                <xsl:value-of select="fm:constraints/fm:maxExclusive" />
                                <xsl:text>'</xsl:text>
                            </xsl:if>
                            <xsl:if test="fm:constraints/fm:minExclusive != ''" >
                                <xsl:text>, minExclusive: '</xsl:text>
                                <xsl:value-of select="fm:constraints/fm:minExclusive" />
                                <xsl:text>'</xsl:text>
                            </xsl:if>
                            <xsl:text>};</xsl:text>
                        </xsl:variable>

                        <xsl:choose>
                            <xsl:when test="$repeatName != ''">
                                <out:if test="position() = 1">
                                    <script type="text/javascript">
                                        <xsl:value-of select="$calendarConfigString"/>
                                    </script>
                                </out:if>
                            </xsl:when>
                            <xsl:otherwise>
                                <script type="text/javascript">
                                    <xsl:value-of select="$calendarConfigString"/>
                                </script>
                            </xsl:otherwise>
                        </xsl:choose>

                        <a id="{@name}_calendar_anchor" name="{@name}_calendar_anchor" href="#" onclick="hyf.calendar.showCalendar('{@name}');return false;" class="{fm:field/fm:date_picker/@class}">
                            <xsl:if test="$repeatName != ''">
                                <xsl:attribute name="id">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="$repeatName"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                    <xsl:value-of select="@name"/>
                                    <xsl:text>_calendar_anchor</xsl:text>
                                </xsl:attribute>
                                <xsl:attribute name="name">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="$repeatName"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                    <xsl:value-of select="@name"/>
                                    <xsl:text>_calendar_anchor</xsl:text>
                                </xsl:attribute>
                                <xsl:attribute name="onclick">
                                    <xsl:text>hyf.calendar.showCalendar('</xsl:text>
                                    <xsl:value-of select="@name"/>
                                    <xsl:text>', '{$</xsl:text>
                                    <xsl:value-of select="$repeatName"/>
                                    <xsl:text>LoopCount}');return false;</xsl:text>
                                </xsl:attribute>
                            </xsl:if>
                            <!-- check if this should be disabled (by removing these two event triggers)
                                 TODO: should really not output the attributes above if they are not needed-->
                            <xsl:if test="(@disable = 'true') and ($bindings/fm:xforms/fm:element[@name = current()/@name]/fm:disable_xpath != '')">
                                <out:if test="{$bindings/fm:xforms/fm:element[@name = current()/@name]/fm:disable_xpath}">
                                    <out:attribute name="onclick" namespace=""/>
                                </out:if>
                            </xsl:if>
                            <!--<out:text>-->
                                <xsl:choose>
                                    <xsl:when test="fm:field/fm:date_picker/@content != ''">
                                        <xsl:value-of select="fm:field/fm:date_picker/@content"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:text disable-output-escaping="yes">&#160;&#160;&#160;&#160;&#160;</xsl:text>
                                    </xsl:otherwise>
                                </xsl:choose>
                            <!--</out:text>-->
                        </a>
                    </xsl:when>
                </xsl:choose>
            </xsl:if>
        </xsl:if>
    </xsl:template>
    <!-- Match the form div and insert the xform:instance tag -->
    <xsl:template match="div[@class='form']" xmlns:fm="http://www.hyfinity.com/formmaker">

        <xsl:call-template name="create-xform-wrapper">
            <xsl:with-param name="content" select="."/>
        </xsl:call-template>

    </xsl:template>

    <!-- This tempalte adds and xform tag around the given content, and outputs the instance docuemnt
         content that applies to the current page. -->
    <xsl:template name="create-xform-wrapper" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="content" />

        <!-- Create the xform tag with all the namespaces from the root of the bindings config file
             These set of namespaces will be used for all the actions and message structures coming out from the page -->
        <xsl:text disable-output-escaping="yes">&lt;xform:xform xmlns:xform="http://www.w3.org/2001/08/xforms" xmlns="" </xsl:text>
        <xsl:for-each select="$bindings/fm:xforms/namespace::*">
            <xsl:if test="(name(.) != '') and (name(.) != 'xml') and (name(.) != 'xform')">
                <xsl:text>xmlns:</xsl:text>
                <xsl:value-of select="name(.)"/>
                <xsl:text disable-output-escaping="yes">="</xsl:text>
                <xsl:value-of select="."/>
                <xsl:text disable-output-escaping="yes">" </xsl:text>
            </xsl:if>
        </xsl:for-each>
        <xsl:text disable-output-escaping="yes">&gt;</xsl:text>
        <!-- Bug 92: Only output the xforms:instance if this is a standard mvc application
            This is so screen only applications (eg Miaftr) wont see the instance data appearing on the screen. -->
        <xsl:if test="$appConfig/fm:application/fm:display_type[. = 'mvc']">

            <!-- loop through each unique action instance info -->
            <xsl:for-each select="/fm:transform_structure/fm:data_from_form/fm:actions/fm:action">
                <xform:instance xmlns:xform="http://www.w3.org/2001/08/xforms">
                    <xsl:if test="@name != ''">
                        <xsl:attribute name="id">
                            <xsl:value-of select="@name"/>
                        </xsl:attribute>
                    </xsl:if>
                    <xsl:copy-of select="*"/>
                </xform:instance>
            </xsl:for-each>
        </xsl:if>

        <!-- for-each used to ensure the current context is the content element provided -->
        <xsl:for-each select="$content">
            <xsl:copy>
                <xsl:copy-of select="@*"/>
                <xsl:apply-templates select="*"/>
            </xsl:copy>
        </xsl:for-each>

        <xsl:text disable-output-escaping="yes">&lt;/xform:xform&gt;</xsl:text>


    </xsl:template>


    <!-- Add the check to create the language field underneath the form if needed -->
    <xsl:template match="form">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>

            <out:if test="/mvc:eForm/mvc:Control/mvc:Language != ''">
                <input type="hidden" name="Language" value="{{/mvc:eForm/mvc:Control/mvc:Language}}"/>
            </out:if>

            <xsl:apply-templates/>

        </xsl:copy>
    </xsl:template>

    <!-- match repeat to insert xpaths -->
    <xsl:template match="repeat_code_placeholder" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="rpc" select="."/>
        <xsl:variable name="repeatName" select="@name"/>
        <xsl:variable name="configEntry" select="$formStructure/fm:structure//fm:repeat[@name = $repeatName]"/>
        <!-- Handle the sort order and data-type options
             Create variables so we can check in advance if the dynamic xpaths evaluate to an appropriate value-->
        <xsl:if test="$configEntry/fm:sort and ($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_xpath != '')">
            <out:variable name="{$repeatName}_sort_order">
                <xsl:choose>
                    <xsl:when test="$configEntry/fm:sort/@order != 'dynamic'">
                        <xsl:value-of select="$configEntry/fm:sort/@order"/>
                    </xsl:when>
                    <xsl:when test="($configEntry/fm:sort/@order = 'dynamic') and not($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_order_xpath != '')">ascending</xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="sort_order_xpath">
                            <xsl:choose>
                                <xsl:when test="starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_order_xpath, '/')">
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_order_xpath"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath"/>
                                    <xsl:text>/</xsl:text>
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_order_xpath"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <out:choose>
                            <out:when test="{$sort_order_xpath} = 'ascending' or {$sort_order_xpath} = 'descending'">
                                <out:value-of select="{$sort_order_xpath}"/>
                            </out:when>
                            <out:otherwise>ascending</out:otherwise>
                        </out:choose>
                    </xsl:otherwise>
                </xsl:choose>
            </out:variable>
            <out:variable name="{$repeatName}_sort_data-type">
                <xsl:choose>
                    <xsl:when test="$configEntry/fm:sort/@data-type != 'dynamic'">
                        <xsl:value-of select="$configEntry/fm:sort/@data-type"/>
                    </xsl:when>
                    <xsl:when test="($configEntry/fm:sort/@data-type = 'dynamic') and not($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_data-type_xpath != '')">text</xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="sort_data-type_xpath">
                            <xsl:choose>
                                <xsl:when test="starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_data-type_xpath, '/')">
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_data-type_xpath"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath"/>
                                    <xsl:text>/</xsl:text>
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_data-type_xpath"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <out:choose>
                            <out:when test="{$sort_data-type_xpath} = 'text' or {$sort_data-type_xpath} = 'number'">
                                <out:value-of select="{$sort_data-type_xpath}"/>
                            </out:when>
                            <out:otherwise>text</out:otherwise>
                        </out:choose>
                    </xsl:otherwise>
                </xsl:choose>
            </out:variable>
        </xsl:if>
        <!-- Create for-each statement -->
        <out:for-each select="{$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath}">
            <!-- if the repeat xpath is not specified put dummy string in the for-each statement -->
            <xsl:if test="not($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath = '')">
                <xsl:attribute name="select">repeat_xpath_not_specified</xsl:attribute>
            </xsl:if>
            <!-- Check if the entries should be sorted -->
            <xsl:if test="$configEntry/fm:sort and ($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_xpath != '')">
                <out:sort select="{$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:sort_xpath}" order="{{${$repeatName}_sort_order}}" data-type="{{${$repeatName}_sort_data-type}}">
                </out:sort>
            </xsl:if>
            <!-- Create loop variables -->
            <out:variable name="{$repeatName}LoopEntry" select="."/>
            <xsl:choose>
                <xsl:when test="ancestor::repeat_code_placeholder">
                    <out:variable name="{$repeatName}LoopCount">
                        <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        <out:text>
                            <xsl:value-of select="$repeatName"/>
                        </out:text>
                        <out:value-of select="position()"/>
                    </out:variable>
                </xsl:when>
                <xsl:otherwise>
                    <out:variable name="{$repeatName}LoopCount">
                        <out:text>
                            <xsl:value-of select="$repeatName"/>
                        </out:text>
                        <out:value-of select="position()"/>
                    </out:variable>
                </xsl:otherwise>
            </xsl:choose>
            <!-- Build up xform xpath, used for hidden fields for all elements within this repeat -->

            <!-- If sorting has been enabled, then we need to loop through the data again to work out the position of the
                 current enty in the repeat in document order, not sorted order.
                 This used to happen within the xform_xpath variable definition below, but as there are now (potentially)
                 multiple xform xpaths for each repeat we now do the loop once here, and create a variable to save having
                 to loop through the data multiple times. -->

            <out:variable name="{$repeatName}-repeat-position-number">
                <xsl:choose>
                    <xsl:when test="$configEntry/fm:sort">
                        <xsl:variable name="repeatXPath">
                            <xsl:call-template name="findRepeatXPath">
                                <xsl:with-param name="repeatEntry" select="$configEntry"/>
                                <xsl:with-param name="currentXPath"/>
                            </xsl:call-template>
                        </xsl:variable>
                        <out:for-each select="{$repeatXPath}">
                            <out:if test="generate-id(.) = generate-id(${$repeatName}LoopEntry)">
                                <out:value-of select="position()"/>
                            </out:if>
                        </out:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:value-of select="position()"/>
                    </xsl:otherwise>
                </xsl:choose>
            </out:variable>

            <!-- Process repeat contents -->
            <xsl:apply-templates select="*"/>
        </out:for-each>
    </xsl:template>

    <xsl:template match="repeat_code_default_row_placeholder" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="repeatName" select="@name"/>
        <out:for-each select="/*[1]">
            <out:variable name="{$repeatName}-repeat-position-number">xxxxx</out:variable>
            <out:variable name="{$repeatName}LoopEntry" select="."/>
            <out:variable name="{$repeatName}LoopCount">
                <xsl:if test="ancestor::repeat_code_placeholder">
                    <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                </xsl:if>
                <out:text>
                    <xsl:value-of select="$repeatName"/>
                    <xsl:text>BlankEntry</xsl:text>
                </out:text>
            </out:variable>

            <xsl:apply-templates select="preceding::repeat_code_placeholder[@name = $repeatName]/*"/>
        </out:for-each>
    </xsl:template>

    <xsl:template match="repeat_code_hidden_fields_placeholder" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="repeatName" select="@name"/>
        <xsl:variable name="configEntry" select="$formStructure/fm:structure//fm:repeat[@name = $repeatName]"/>

        <!-- Create hidden fields storing the Page to Hanlder XPath for this Repeat, and the count of how many
             entries are within this repeat at runtime.

             These can then be used by SXForms to clone structure to ensure the correct number of repeat entries
             are present in the data to be bound to.
             -->
        <!-- Only do this for repeats that have some data entry componenets contained within them -->
        <xsl:if test="$configEntry//fm:element/fm:field/fm:input[. != 'output' and . != 'button' and . != 'htmlButton']">

            <xsl:variable name="repeatNumber">
                <!-- if this is a partial page, then include the page name in the id to stop possible conflicts
                     with data on the main page -->
                <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" >
                    <xsl:value-of select="$page_id" />
                    <xsl:text>-</xsl:text>
                </xsl:if>
                <xsl:if test="$configEntry/ancestor::fm:repeat">
                    <xsl:text>{$</xsl:text>
                    <xsl:value-of select="$configEntry/ancestor::fm:repeat[1]/@name"/>
                    <xsl:text>LoopCount}-</xsl:text>
                </xsl:if>
                <xsl:value-of select="count($configEntry/ancestor::fm:repeat) + count($configEntry/preceding::fm:repeat) + 1"/>
            </xsl:variable>

            <xsl:variable name="repeatBindings">
                <xsl:call-template name="find-field-xpath-bindings">
                    <xsl:with-param name="configElement" select="$configEntry"/>
                </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="repeatBindingsSet" select="exsl:node-set($repeatBindings)"/>

            <!-- loop through each unique xpath found -->
            <xsl:for-each select="$repeatBindingsSet/bindings/action[not(. = preceding-sibling::action)]">
                <input type="hidden" value="{.}">
                    <xsl:attribute name="name">
                        <!-- only add the action naming stuff if we have more than one binding -->
                        <xsl:if test="count($repeatBindingsSet/bindings/action[not(. = preceding-sibling::action)]) != 1" >
                            <xsl:text>xgf_</xsl:text>
                        </xsl:if>

                        <xsl:text>hyfrepeat-</xsl:text>
                        <xsl:value-of select="$repeatNumber"/>
                        <xsl:text>-location</xsl:text>

                        <!-- only add the action naming stuff if we have more than one binding -->
                        <xsl:if test="count($repeatBindingsSet/bindings/action[not(. = preceding-sibling::action)]) != 1" >
                            <xsl:for-each select="$repeatBindingsSet/bindings/action[. = current()]" >
                                <xsl:text>_xga_</xsl:text>
                                <xsl:value-of select="@name"/>
                            </xsl:for-each>
                            <!-- add the names of any actions based on this one -->
                            <xsl:for-each select="$bindings/fm:xforms/fm:instance_configuration/fm:action[@base = current()/@name]" >
                                <xsl:text>_xga_</xsl:text>
                                <xsl:value-of select="@name"/>
                            </xsl:for-each>
                        </xsl:if>
                    </xsl:attribute>
                </input>

            </xsl:for-each>


            <xsl:if test="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath != ''">
                <input type="hidden" name="hyfrepeat-{$repeatNumber}-count" value="{{count({$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath})}}"/>
            </xsl:if>
        </xsl:if>

        <!-- Check if default sorting behaviour should be assumed, and if so create hidden fields to store information -->
        <xsl:if test="$configEntry/fm:sort/@default = 'true'">
            <!-- TODO: FOR MVC 2.8: This default sorting functionality will not work anymore, because of the changes to the page to handler xpath structure. -->
            <input type="hidden" name="{$repeatName}_sort_field" value="{$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath}/../{$repeatName}_sort_field">
                <xsl:attribute name="value"><xsl:text>{</xsl:text><xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath"/><xsl:text>/../</xsl:text><xsl:value-of select="$repeatName"/><xsl:text>_sort_field</xsl:text><xsl:text>}</xsl:text></xsl:attribute>
            </input>
            <input type="hidden" name="{$repeatName}_sort_field_xpath" value="">
                <!-- Set xform xpath value so that the sorting control fields are assumed to be at the same level as the repeating nodeset, with specifc element names that dont have a namespace-->
                <xsl:choose>
                    <xsl:when test="starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath, '/')">
                        <xsl:attribute name="value"><xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath"/><xsl:text>/../</xsl:text><xsl:value-of select="$repeatName"/><xsl:text>_sort_field</xsl:text></xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:attribute name="value" namespace="">
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}_xform_xpath"/>
                                <out:text>
                                    <xsl:text>/</xsl:text>
                                </out:text>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath"/>
                                <xsl:text>/../</xsl:text>
                                <xsl:value-of select="$repeatName"/>
                                <xsl:text>_sort_field</xsl:text>
                            </out:text>
                        </out:attribute>
                    </xsl:otherwise>
                </xsl:choose>
            </input>
            <xsl:if test="$configEntry/fm:sort/@order = 'dynamic'">
                <input type="hidden" name="{$repeatName}_sort_order" value="{$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath}/../{$repeatName}_sort_order">
                    <xsl:attribute name="value"><xsl:text>{</xsl:text><xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath"/><xsl:text>/../</xsl:text><xsl:value-of select="$repeatName"/><xsl:text>_sort_order</xsl:text><xsl:text>}</xsl:text></xsl:attribute>
                </input>
                <input type="hidden" name="{$repeatName}_sort_order_xpath" value="/{$repeatName}_sort_order">
                    <xsl:choose>
                        <xsl:when test="starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath, '/')">
                            <xsl:attribute name="value"><xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath"/><xsl:text>/../</xsl:text><xsl:value-of select="$repeatName"/><xsl:text>_sort_order</xsl:text></xsl:attribute>
                        </xsl:when>
                        <xsl:otherwise>
                            <out:attribute name="value" namespace="">
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}_xform_xpath"/>
                                    <out:text>
                                        <xsl:text>/</xsl:text>
                                    </out:text>
                                </xsl:if>
                                <out:text>
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath"/>
                                    <xsl:text>/../</xsl:text>
                                    <xsl:value-of select="$repeatName"/>
                                    <xsl:text>_sort_order</xsl:text>
                                </out:text>
                            </out:attribute>
                        </xsl:otherwise>
                    </xsl:choose>
                </input>
            </xsl:if>
            <xsl:if test="$configEntry/fm:sort/@data-type = 'dynamic'">
                <input type="hidden" name="{$repeatName}_sort_data_type" value="{$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath}/../{$repeatName}_sort_data_type">
                    <xsl:attribute name="value"><xsl:text>{</xsl:text><xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath"/><xsl:text>/../</xsl:text><xsl:value-of select="$repeatName"/><xsl:text>_sort_data_type</xsl:text><xsl:text>}</xsl:text></xsl:attribute>
                </input>
                <input type="hidden" name="{$repeatName}_sort_data_type_xpath" value="/{$repeatName}_sort_data_type">
                    <xsl:choose>
                        <xsl:when test="starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath, '/')">
                            <xsl:attribute name="value"><xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath"/><xsl:text>/../</xsl:text><xsl:value-of select="$repeatName"/><xsl:text>_sort_data_type</xsl:text></xsl:attribute>
                        </xsl:when>
                        <xsl:otherwise>
                            <out:attribute name="value" namespace="">
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}_xform_xpath"/>
                                    <out:text>
                                        <xsl:text>/</xsl:text>
                                    </out:text>
                                </xsl:if>
                                <out:text>
                                    <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:xform_xpath"/>
                                    <xsl:text>/../</xsl:text>
                                    <xsl:value-of select="$repeatName"/>
                                    <xsl:text>_sort_data_type</xsl:text>
                                </out:text>
                            </out:attribute>
                        </xsl:otherwise>
                    </xsl:choose>
                </input>
            </xsl:if>
        </xsl:if>

    </xsl:template>

    <!-- The 'repeat_code_empty_content' element is inserted in the HTML content to wrap data that
        should only be displayed if the repeat matches no elements -->
    <xsl:template match="repeat_code_empty_content" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="repeatName" select="@name"/>
        <xsl:variable name="configEntry" select="$formStructure/fm:structure//fm:repeat[@name = $repeatName]"/>

        <!-- create the check on the runtime data -->
        <out:if test="not({$bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath})">
            <!-- if the repeat xpath is not specified put dummy string in the for-each statement -->
            <xsl:if test="not($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:repeat[@name = $repeatName]/fm:repeat_xpath = '')">
                <xsl:attribute name="test">not(repeat_xpath_not_specified)</xsl:attribute>
            </xsl:if>
            <!-- copy the existing content -->
            <xsl:apply-templates />
        </out:if>

    </xsl:template>


    <xsl:template name="findRepeatXPath" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="repeatEntry"/>
        <xsl:param name="currentXPath"/>
        <xsl:choose>
            <xsl:when test="$repeatEntry/ancestor::fm:repeat">
                <!-- check if repeat xpath for this repeat is absolute -->
                <xsl:choose>
                    <xsl:when test="starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath, '/')">
                        <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath"/>
                        <xsl:if test="$currentXPath != ''">
                            <xsl:text>/</xsl:text>
                            <xsl:value-of select="$currentXPath"/>
                        </xsl:if>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="newXPath">
                            <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath"/>
                            <xsl:if test="$currentXPath != ''">
                                <xsl:text>/</xsl:text>
                                <xsl:value-of select="$currentXPath"/>
                            </xsl:if>
                        </xsl:variable>
                        <xsl:call-template name="findRepeatXPath">
                            <xsl:with-param name="repeatEntry" select="$repeatEntry/ancestor::fm:repeat[1]"/>
                            <xsl:with-param name="currentXPath" select="$newXPath"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath"/>
                <xsl:if test="$currentXPath != ''">
                    <xsl:text>/</xsl:text>
                    <xsl:value-of select="$currentXPath"/>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- match group to check if hide needed -->
    <xsl:template match="*[@_use='group']" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="groupName" select="@id"/>
        <xsl:variable name="configGroup" select="$formStructure/fm:structure//fm:group[@name = $groupName]"/>
        <xsl:choose>
            <xsl:when test="($configGroup/@hide = 'true') and ($bindings/fm:xforms/fm:group[@name = $groupName]/fm:hide_xpath != '')">
                <!-- group needs to be conditionally hidden -->

                <!-- Check if it is in a repeat table, to determine whether the element should be removed completely
                     or if the td element if present should be retained-->
                <xsl:choose>
                    <xsl:when test="$configGroup/parent::fm:group[(@gen_method='table') or (@gen_method='horizontal_table')]/parent::fm:repeat" >
                        <!-- Check if the xpath is absolute or not -->
                        <xsl:variable name="isAbsolute" select="starts-with($bindings/fm:xforms/fm:group[@name = $groupName]/fm:hide_xpath, '/')"/>
                        <xsl:choose>
                            <xsl:when test="$isAbsolute" >
                                <xsl:apply-templates select="." mode="process-group-hide">
                                    <xsl:with-param name="leave-tds">false</xsl:with-param>
                                    <xsl:with-param name="configGroup" select="$configGroup" />
                                </xsl:apply-templates>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:apply-templates select="." mode="process-group-hide">
                                    <xsl:with-param name="leave-tds">true</xsl:with-param>
                                    <xsl:with-param name="configGroup" select="$configGroup" />
                                </xsl:apply-templates>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:apply-templates select="." mode="process-group-hide">
                            <xsl:with-param name="leave-tds">true</xsl:with-param>
                            <xsl:with-param name="configGroup" select="$configGroup" />
                        </xsl:apply-templates>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <!-- no conditional hiding -->
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <!-- Check for dynamic group styling -->
                    <xsl:if test="$configGroup/fm:style/@options = 'dynamic'" >
                        <out:attribute name="class">
                            <xsl:if test="@class != ''">
                                <out:text>
                                    <xsl:value-of select="@class"/>
                                </out:text>
                            </xsl:if>
                            <xsl:call-template name="outputDynamicStylingImpl">
                                <xsl:with-param name="configElement" select="$configGroup/fm:style"/>
                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $groupName]"/>
                                <xsl:with-param name="styleType">group</xsl:with-param>
                            </xsl:call-template>
                        </out:attribute>
                    </xsl:if>

                    <xsl:apply-templates select="*"/>
                </xsl:copy>
                <xsl:call-template name="create-group-display-details">
                    <xsl:with-param name="configGroup" select="$configGroup"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="*[@_use='group']" mode="process-group-hide" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="leave-tds">true</xsl:param>
        <xsl:param name="configGroup"/>

        <xsl:choose>
            <xsl:when test="($leave-tds = 'true') and (local-name(.) = 'td')">
                <!-- always output the td container and styling -->
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <!-- Check for dynamic group styling -->
                    <xsl:if test="$configGroup/fm:style/@options = 'dynamic'" >
                        <out:attribute name="class">
                            <xsl:if test="@class != ''">
                                <out:text>
                                    <xsl:value-of select="@class"/>
                                </out:text>
                            </xsl:if>
                            <xsl:call-template name="outputDynamicStylingImpl">
                                <xsl:with-param name="configElement" select="$configGroup/fm:style"/>
                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $configGroup/@name]"/>
                                <xsl:with-param name="styleType">group</xsl:with-param>
                            </xsl:call-template>
                        </out:attribute>
                    </xsl:if>

                    <!-- check if submit if hidden is false, and so additional hidden fields requried to
                         indicate how to put structure back when needed -->
                    <xsl:choose>
                        <xsl:when test="($configGroup/@hide = 'true') and ($configGroup/@submit_if_hidden = 'false') and ($bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath != '')">
                            <out:choose>
                                <out:when test="not({$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath})">
                                    <xsl:apply-templates select="*"/>

                                    <xsl:call-template name="create-group-display-details">
                                        <xsl:with-param name="configGroup" select="$configGroup"/>
                                        <xsl:with-param name="showHideGroup">true</xsl:with-param>
                                    </xsl:call-template>
                                </out:when>
                                <out:otherwise>
                                    <xsl:call-template name="create-group-display-details">
                                        <xsl:with-param name="configGroup" select="$configGroup"/>
                                        <xsl:with-param name="showHideGroup">false</xsl:with-param>
                                    </xsl:call-template>
                                </out:otherwise>
                            </out:choose>
                        </xsl:when>
                        <xsl:when test="($configGroup/@hide = 'true') and ($bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath != '')">
                            <out:if test="not({$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath})">
                                <xsl:apply-templates select="*"/>
                            </out:if>
                            <xsl:call-template name="create-group-display-details">
                                <xsl:with-param name="configGroup" select="$configGroup"/>
                            </xsl:call-template>
                        </xsl:when>
                    </xsl:choose>

                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <!-- remove the whole group container when hidden -->

                <!-- check if submit if hidden is false, and so additional hidden fields requried to
                     indicate how to put structure back when needed -->
                <xsl:choose>
                    <xsl:when test="($configGroup/@hide = 'true') and ($configGroup/@submit_if_hidden = 'false') and ($bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath != '')">
                        <out:choose>
                            <out:when test="not({$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath})">
                                <xsl:copy>
                                    <xsl:apply-templates select="@*"/>
                                    <!-- Check for dynamic group styling -->
                                    <xsl:if test="$configGroup/fm:style/@options = 'dynamic'" >
                                        <out:attribute name="class">
                                            <xsl:if test="@class != ''">
                                                <out:text>
                                                    <xsl:value-of select="@class"/>
                                                </out:text>
                                            </xsl:if>
                                            <xsl:call-template name="outputDynamicStylingImpl">
                                                <xsl:with-param name="configElement" select="$configGroup/fm:style"/>
                                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $configGroup/@name]"/>
                                                <xsl:with-param name="styleType">group</xsl:with-param>
                                            </xsl:call-template>
                                        </out:attribute>
                                    </xsl:if>


                                    <xsl:apply-templates select="*"/>
                                </xsl:copy>
                                <xsl:call-template name="create-group-display-details">
                                    <xsl:with-param name="configGroup" select="$configGroup"/>
                                    <xsl:with-param name="showHideGroup">true</xsl:with-param>
                                </xsl:call-template>
                            </out:when>
                            <out:otherwise>
                                <xsl:call-template name="create-group-display-details">
                                    <xsl:with-param name="configGroup" select="$configGroup"/>
                                    <xsl:with-param name="showHideGroup">false</xsl:with-param>
                                </xsl:call-template>
                            </out:otherwise>
                        </out:choose>
                    </xsl:when>
                    <xsl:when test="($configGroup/@hide = 'true') and ($bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath != '')">
                        <out:if test="not({$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath})">
                            <xsl:copy>
                                <xsl:apply-templates select="@*"/>

                                <!-- Check for dynamic group styling -->
                                <xsl:if test="$configGroup/fm:style/@options = 'dynamic'" >
                                    <out:attribute name="class">
                                        <xsl:if test="@class != ''">
                                            <out:text>
                                                <xsl:value-of select="@class"/>
                                            </out:text>
                                        </xsl:if>
                                        <xsl:call-template name="outputDynamicStylingImpl">
                                            <xsl:with-param name="configElement" select="$configGroup/fm:style"/>
                                            <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $configGroup/@name]"/>
                                            <xsl:with-param name="styleType">group</xsl:with-param>
                                        </xsl:call-template>
                                    </out:attribute>
                                </xsl:if>

                                <xsl:apply-templates select="*"/>
                            </xsl:copy>
                        </out:if>
                        <xsl:call-template name="create-group-display-details">
                            <xsl:with-param name="configGroup" select="$configGroup"/>
                        </xsl:call-template>
                    </xsl:when>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <xsl:template name="create-group-display-details" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configGroup"/>
        <xsl:param name="showHideGroup">true</xsl:param>
        <xsl:variable name="groupName" select="$configGroup/@name"/>
        <xsl:if test="$configGroup/@hide = 'true' or $configGroup/@conditional_display = 'true'">
            <!-- if submit_if_hidden is set to false, output the hidden fields required by sxforms -->
            <xsl:if test="$configGroup/@submit_if_hidden = 'false'">
                <xsl:variable name="groupNumber">
                    <!-- if this is a partial page, then include the page name in the id to stop possible conflicts
                         with data on the main page -->
                    <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" >
                        <xsl:value-of select="$page_id" />
                        <xsl:text>-</xsl:text>
                    </xsl:if>

                    <xsl:if test="$configGroup/ancestor::fm:repeat">
                        <xsl:text>{$</xsl:text>
                        <xsl:value-of select="$configGroup/ancestor::fm:repeat[1]/@name"/>
                        <xsl:text>LoopCount}-</xsl:text>
                    </xsl:if>
                    <xsl:value-of select="count($configGroup/ancestor::fm:group[@hide = 'true' or @conditional_display = 'true']) + count($configGroup/preceding::fm:group[@hide = 'true' or @conditional_display = 'true']) + 1"/>
                </xsl:variable>

                <xsl:choose>
                    <xsl:when test="$showHideGroup = 'false'">
                        <input type="hidden" name="hyfgroup-{$groupNumber}-control" value="NONE_REQUIRED_HIDE"/>
                    </xsl:when>
                    <xsl:when test="$configGroup/@conditional_display = 'true'">
                        <xsl:variable name="controlField" select="$formStructure//fm:element[@name = $configGroup/@control_field]"/>
                        <input type="hidden" name="hyfgroup-{$groupNumber}-control" value="{$configGroup/@control_field}">
                            <xsl:if test="$controlField/ancestor::fm:repeat">
                                <xsl:attribute name="value" namespace=""><xsl:text>{$</xsl:text><xsl:value-of select="$controlField/ancestor::fm:repeat[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$configGroup/@control_field"/></xsl:attribute>
                            </xsl:if>
                        </input>
                        <input type="hidden" name="hyfgroup-{$groupNumber}-values" value="{$configGroup/@control_field_value}"/>
                    </xsl:when>
                    <xsl:when test="$showHideGroup = 'true'">
                        <input type="hidden" name="hyfgroup-{$groupNumber}-control" value="NONE_REQUIRED_SHOW"/>
                    </xsl:when>
                </xsl:choose>


                <!-- find the set of action location bindings for this group-->
                <xsl:variable name="groupBindings">
                    <xsl:call-template name="find-field-xpath-bindings">
                        <xsl:with-param name="configElement" select="$configGroup"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="groupBindingsSet" select="exsl:node-set($groupBindings)"/>

                <!-- work out the set of unique bindings to output, and the actions that each applies to -->
                <xsl:for-each select="$groupBindingsSet/bindings/action[not(. = preceding-sibling::action)]">
                    <input type="hidden" value="{.}">
                        <xsl:attribute name="name">
                            <!-- only add the action naming stuff if we have more than one binding -->
                            <xsl:if test="count($groupBindingsSet/bindings/action[not(. = preceding-sibling::action)]) != 1" >
                                <xsl:text>xgf_</xsl:text>
                            </xsl:if>

                            <xsl:text>hyfgroup-</xsl:text>
                            <xsl:value-of select="$groupNumber"/>
                            <xsl:text>-location</xsl:text>

                            <!-- only add the action naming stuff if we have more than one binding -->
                            <xsl:if test="count($groupBindingsSet/bindings/action[not(. = preceding-sibling::action)]) != 1" >
                                <xsl:for-each select="$groupBindingsSet/bindings/action[. = current()]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="@name"/>
                                </xsl:for-each>
                                <!-- add the names of any actions based on this one -->
                                <xsl:for-each select="$bindings/fm:xforms/fm:instance_configuration/fm:action[@base = current()/@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="@name"/>
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:attribute>
                    </input>
                </xsl:for-each>

                <!-- work out the set of different group fragments -->
                <xsl:variable name="groupFragments">
                    <xsl:call-template name="find-hide-group-fragments">
                        <xsl:with-param name="configGroup" select="$configGroup"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="groupFragmentsSet" select="exsl:node-set($groupFragments)"/>

                <!-- output each unique fragment -->
                <xsl:for-each select="$groupFragmentsSet/bindings/action/fragment[not(same_as)]">
                    <textarea rows="1" cols="1" style="display:none;">
                        <xsl:attribute name="name">
                            <!-- only add the action naming stuff if we have more than one conflict set -->
                            <xsl:if test="(count($groupFragmentsSet/bindings/action/fragment[not(same_as)]) != 1) or $groupFragmentsSet/bindings/action[not(fragment)]" >
                                <xsl:text>xgf_</xsl:text>
                            </xsl:if>

                            <xsl:text>hyfgroup-</xsl:text>
                            <xsl:value-of select="$groupNumber"/>
                            <xsl:text>-structure</xsl:text>

                            <!-- only add the action naming stuff if we have more than one conflict set -->
                            <xsl:if test="(count($groupFragmentsSet/bindings/action/fragment[not(same_as)]) != 1) or $groupFragmentsSet/bindings/action[not(fragment)]" >
                                <xsl:text>_xga_</xsl:text>
                                <xsl:value-of select="../@name"/>
                                <xsl:for-each select="$groupFragmentsSet/bindings/action/fragment[same_as = current()/../@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="../@name"/>
                                </xsl:for-each>
                                <!-- add the names of any actions based on this one -->
                                <xsl:for-each select="$bindings/fm:xforms/fm:instance_configuration/fm:action[@base = current()/../@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="@name"/>
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:attribute>
                        <xsl:copy-of select="$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:action[@name = current()/../@name]/fm:choice_fragment"/>
                    </textarea>
                </xsl:for-each>

                <!-- check for any conflicting fields and groups -->
                <xsl:variable name="groupConflicts">
                    <xsl:call-template name="find-hide-group-conflicts">
                        <xsl:with-param name="configGroup" select="$configGroup"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="groupConflictsSet" select="exsl:node-set($groupConflicts)"/>
                <xsl:variable name="groupConflictChecks">
                    <xsl:call-template name="check-hide-group-conflict-uniqueness">
                        <xsl:with-param name="conflicts" select="$groupConflictsSet"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="groupConflictChecksSet" select="exsl:node-set($groupConflictChecks)"/>

                <!-- loop through all the actions that have interacting fields found -->
                <xsl:for-each select="$groupConflictChecksSet/conflict_checks/action/interacting_fields[not(same_as)]">
                    <input type="hidden">
                        <xsl:attribute name="name">
                            <!-- only add the action naming stuff if we have more than one conflict set -->
                            <xsl:if test="(count($groupConflictChecksSet/conflict_checks/action/interacting_fields[not(same_as)]) != 1) or $groupConflictChecksSet/conflict_checks/action[not(interacting_fields)]" >
                                <xsl:text>xgf_</xsl:text>
                            </xsl:if>

                            <xsl:text>hyfgroup-</xsl:text>
                            <xsl:value-of select="$groupNumber"/>
                            <xsl:text>-interacting_fields</xsl:text>

                            <!-- only add the action naming stuff if we have more than one conflict set -->
                            <xsl:if test="(count($groupConflictChecksSet/conflict_checks/action/interacting_fields[not(same_as)]) != 1) or $groupConflictChecksSet/conflict_checks/action[not(interacting_fields)]" >
                                <xsl:text>_xga_</xsl:text>
                                <xsl:value-of select="../@name"/>
                                <xsl:for-each select="$groupConflictChecksSet/conflict_checks/action/interacting_fields[same_as = current()/../@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="../@name"/>
                                </xsl:for-each>
                                <!-- add the names of any actions based on this one -->
                                <xsl:for-each select="$bindings/fm:xforms/fm:instance_configuration/fm:action[@base = current()/../@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="@name"/>
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:attribute>
                        <out:attribute name="value">
                            <xsl:for-each select="$groupConflictsSet/conflicts/action[@name = current()/../@name]/interacting_fields/*">
                                <xsl:copy-of select="." />
                            </xsl:for-each>
                        </out:attribute>
                    </input>
                </xsl:for-each>
                <!-- loop through all the actions that have interacting groups found -->
                <xsl:for-each select="$groupConflictChecksSet/conflict_checks/action/interacting_groups[not(same_as)]">
                    <input type="hidden">
                        <xsl:attribute name="name">
                            <!-- only add the action naming stuff if we have more than one conflict set -->
                            <xsl:if test="(count($groupConflictChecksSet/conflict_checks/action/interacting_groups[not(same_as)]) != 1) or $groupConflictChecksSet/conflict_checks/action[not(interacting_groups)]" >
                                <xsl:text>xgf_</xsl:text>
                            </xsl:if>

                            <xsl:text>hyfgroup-</xsl:text>
                            <xsl:value-of select="$groupNumber"/>
                            <xsl:text>-interacting_groups</xsl:text>

                            <!-- only add the action naming stuff if we have more than one conflict set -->
                            <xsl:if test="(count($groupConflictChecksSet/conflict_checks/action/interacting_groups[not(same_as)]) != 1) or $groupConflictChecksSet/conflict_checks/action[not(interacting_groups)]" >
                                <xsl:text>_xga_</xsl:text>
                                <xsl:value-of select="../@name"/>
                                <xsl:for-each select="$groupConflictChecksSet/conflict_checks/action/interacting_groups[same_as = current()/../@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="../@name"/>
                                </xsl:for-each>
                                <!-- add the names of any actions based on this one -->
                                <xsl:for-each select="$bindings/fm:xforms/fm:instance_configuration/fm:action[@base = current()/../@name]" >
                                    <xsl:text>_xga_</xsl:text>
                                    <xsl:value-of select="@name"/>
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:attribute>
                        <out:attribute name="value">
                            <xsl:for-each select="$groupConflictsSet/conflicts/action[@name = current()/../@name]/interacting_groups/*">
                                <xsl:copy-of select="." />
                            </xsl:for-each>
                        </out:attribute>
                    </input>
                </xsl:for-each>

            </xsl:if>
            <!-- if conditional_display is true, and this is the last group using this control field
                 then create the javascript function that will be used to toggle display-->
            <xsl:if test="$configGroup/@conditional_display = 'true' and not($configGroup/following::fm:group[@conditional_display = 'true' and @control_field = $configGroup/@control_field])">
                <xsl:variable name="controlField" select="$formStructure//fm:element[@name = $configGroup/@control_field]"/>

                <xsl:variable name="controlFieldRepeatPrefix">
                    <xsl:if test="$controlField/ancestor::fm:repeat">
                        <xsl:call-template name="find-control-field-repeat-prefix-from-context">
                            <xsl:with-param name="context" select="$controlField"/>
                            <xsl:with-param name="configGroup" select="$configGroup"/>
                        </xsl:call-template>
                    </xsl:if>
                </xsl:variable>

                <!-- Create javascript function for showing and hiding relevant groups as control value changes -->
                <script type="text/javascript">
                    <out:text>function </out:text>
                    <xsl:if test="$controlFieldRepeatPrefix != ''">
                        <out:value-of select="{$controlFieldRepeatPrefix}"/>
                    </xsl:if>
                    <out:text>
                        <xsl:value-of select="$controlField/@name"/>
                        <xsl:text>ValueChanged(field)
                    {
                    </xsl:text>
                        <xsl:choose>
                            <xsl:when test="$controlField/fm:field/fm:input = 'multipleCheckbox'">
                            var elemName = field.getAttribute('_element');
                            var cont = document.getElementById(elemName + '_container');
                            var inputs = cont.getElementsByTagName('input');
                            var checkValues = new Array();
                            for (var i=0; i != inputs.length; ++i) {
                                if (inputs[i].getAttribute('_element') == elemName)
                                    if (inputs[i].checked)
                                        checkValues[checkValues.length] = inputs[i].value;
                            }
                        </xsl:when>
                            <xsl:when test="$controlField/fm:field/fm:input = 'checkbox'">
                                <xsl:text>
                            if (field.checked)
                                var checkValue = field.value;
                            else
                                var checkValue = '</xsl:text>
                                <xsl:value-of select="$controlField/fm:field/fm:input/@unchecked_value"/>
                                <xsl:text>';</xsl:text>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:text>
                            var checkValue = field.value;
                            </xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </out:text>
                    <xsl:for-each select="$formStructure/fm:structure//fm:group[@conditional_display = 'true' and @control_field = $controlField/@name]">
                        <xsl:variable name="choiceGroupName" select="@name"/>
                        <out:text>
                            <xsl:choose>
                                <xsl:when test="$controlField/fm:field/fm:input = 'multipleCheckbox'">
                                    <xsl:text>
                                    match = false;
                                    for (var val=0; val != checkValues.length; ++val) {
                                    </xsl:text>
                                    <xsl:text>if (';</xsl:text>
                                    <xsl:value-of select="@control_field_value"/>
                                    <xsl:text>;'.indexOf(';'+checkValues[val]+';') != -1) {
                                        match = true; break;
                                      }
                                    }
                                    if (match) {</xsl:text>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:text>if (';</xsl:text>
                                    <xsl:value-of select="@control_field_value"/>
                                    <xsl:text>;'.indexOf(';'+checkValue+';') != -1) {</xsl:text>
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:text>
                            hyf.util.showComponent(document.getElementById('</xsl:text>
                        </out:text>
                        <xsl:if test="ancestor::fm:repeat">
                            <out:value-of select="${ancestor::fm:repeat[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                            <xsl:value-of select="$choiceGroupName"/>
                            <xsl:text>'));
                            </xsl:text>
                        </out:text>
                        <!-- Check if this group has a label which needs to be shown or hidden
                             NOTE: this will no longer be needed if the group labels are moved within the group itself!!-->
                        <xsl:if test="fm:label[@required = 'true']">
                            <out:text>hyf.util.showComponent(document.getElementById('</out:text>
                            <xsl:if test="ancestor::fm:repeat">
                                <out:value-of select="${ancestor::fm:repeat[1]/@name}LoopCount"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$choiceGroupName"/>
                                <xsl:text>_label_container'));
                                </xsl:text>
                            </out:text>
                        </xsl:if>
                        <out:text>} else {
                            hyf.util.hideComponent(document.getElementById('</out:text>
                        <xsl:if test="ancestor::fm:repeat">
                            <out:value-of select="${ancestor::fm:repeat[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                            <xsl:value-of select="$choiceGroupName"/>
                            <xsl:text>'));
                            </xsl:text>
                        </out:text>
                        <!-- Check if this group has a label which needs to be shown or hidden
                             NOTE: this will no longer be needed if the group labels are moved within the group itself!!-->
                        <xsl:if test="fm:label[@required = 'true']">
                            <out:text>hyf.util.hideComponent(document.getElementById('</out:text>
                            <xsl:if test="ancestor::fm:repeat">
                                <out:value-of select="${ancestor::fm:repeat[1]/@name}LoopCount"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$choiceGroupName"/>
                                <xsl:text>_label_container'));
                                </xsl:text>
                            </out:text>
                        </xsl:if>
                        <out:text>} </out:text>
                    </xsl:for-each>
                    <out:text>}
                    </out:text>
                    <!-- Call this new function now to make sure only the correct group sections are initially hidden-->

                    <xsl:variable name="controlFieldExtraRepeatString">
                        <xsl:call-template name="find-control-field-static-repeat-prefix">
                            <xsl:with-param name="context" select="$controlField"/>
                            <xsl:with-param name="configGroup" select="$configGroup"/>
                        </xsl:call-template>
                    </xsl:variable>

                    <xsl:choose>
                        <xsl:when test="$controlField/fm:field/fm:input = 'radio'">
                            <out:text>
                                <xsl:text>found = false; cont = document.getElementById('</xsl:text>
                            </out:text>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlFieldExtraRepeatString"/>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>_container');
                            var inputs = cont.getElementsByTagName('input');
                            for (var i=0; i != inputs.length; ++i) {
                                if (inputs[i].type == 'radio')
                                    if (inputs[i].checked)
                                    {</xsl:text>
                            </out:text>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>ValueChanged(inputs[i]); found = true; break;}
                            } if (!found)</xsl:text>
                            </out:text>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>ValueChanged(document);</xsl:text>
                            </out:text>
                        </xsl:when>
                        <xsl:when test="$controlField/fm:field/fm:input = 'multipleCheckbox'">
                            <!-- The multipleCheckbox support relies on the _element attribtues being present.
                                 These do not get created until the end though so rather than calling the function
                                 now we attach an onload event to call it.
                                 NOTE: attachEvent doesn't work in Mozilla/Safari (and maybe others) so switched to
                                 call a new method which handles the complexities of how to actually attach the event.
                                 This is defined in the DisplayUtils JS file.-->
                            <out:text>
                                <!--<xsl:text>window.attachEvent('onload', function() {</xsl:text>-->
                                <xsl:text>hyf.attachEventHandler(window, 'onload', function() {</xsl:text>
                            </out:text>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>ValueChanged(document.getElementById('</xsl:text>
                            </out:text>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlFieldExtraRepeatString"/>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>1')); });</xsl:text>
                            </out:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>ValueChanged(document.getElementById('</xsl:text>
                            </out:text>
                            <xsl:if test="$controlFieldRepeatPrefix != ''">
                                <out:value-of select="{$controlFieldRepeatPrefix}"/>
                            </xsl:if>
                            <out:text>
                                <xsl:value-of select="$controlFieldExtraRepeatString"/>
                                <xsl:value-of select="$controlField/@name"/>
                                <xsl:text>'));</xsl:text>
                            </out:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </script>
            </xsl:if>
        </xsl:if>
    </xsl:template>

    <!-- Returns the name of the loop count variable for the first repeat that is a parent of both the supplied group
         and the supplied context element (control field)
         eg. Given structure        RA
                                        RB
                                            F
                                        G
         This will return '$RALoopCount', and the next method would return 'RB1'
         -->
    <xsl:template name="find-control-field-repeat-prefix-from-context" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configGroup" />
        <xsl:param name="context" />

        <xsl:choose>
            <xsl:when test="not($context)">
            </xsl:when>
            <xsl:when test="$configGroup/ancestor::fm:repeat/@name = $context/ancestor::fm:repeat[1]/@name">
                <xsl:text>$</xsl:text>
                <xsl:value-of select="$context/ancestor::fm:repeat[1]/@name" />
                <xsl:text>LoopCount</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="find-control-field-repeat-prefix-from-context">
                    <xsl:with-param name="configGroup" select="$configGroup"/>
                    <xsl:with-param name="context" select="$context/ancestor::fm:repeat[1]"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Returns a static string that should be added to the front of the given fields name to find an ID value
         that handles all the repeats between the group and the field (context).
         eg. Given structure        RA
                                        RB
                                            F
                                        G
         This will return 'RB1', and the above method would return '$RALoopCount'
         -->
    <xsl:template name="find-control-field-static-repeat-prefix" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configGroup" />
        <xsl:param name="context" />

        <xsl:choose>
            <xsl:when test="not($context) or not($context/ancestor::fm:repeat)">
            </xsl:when>
            <xsl:when test="$configGroup/ancestor::fm:repeat/@name = $context/ancestor::fm:repeat[1]/@name">

            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="find-control-field-static-repeat-prefix">
                    <xsl:with-param name="configGroup" select="$configGroup"/>
                    <xsl:with-param name="context" select="$context/ancestor::fm:repeat[1]"/>
                </xsl:call-template>

                <xsl:value-of select="$context/ancestor::fm:repeat[1]/@name" />
                <xsl:text>1</xsl:text>

            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <!-- checks through the set of xml fragments defined for the given group for each action,
         to find out whihc ones are the same or not. -->
    <xsl:template name="find-hide-group-fragments" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configGroup"/>

        <bindings>
            <!-- loop through each action binding that we have for this field in the bindings file that matches a still valid action-->
            <xsl:for-each select="$bindings/fm:xforms/fm:group[(@name = $configGroup/@name)]/fm:action[contains($actionNames, concat(';', @name, ';'))]" >
                <action name="{@name}">
                    <xsl:if test="fm:choice_fragment/*">
                        <fragment>
                            <xsl:variable name="loopCheck" select="fm:choice_fragment"/>
                            <xsl:for-each select="preceding-sibling::fm:action[fm:choice_fragment/*]">
                                <xsl:variable name="diff" >
                                    <xsl:call-template name="compare-fragments">
                                        <xsl:with-param name="nodeA" select="fm:choice_fragment"/>
                                        <xsl:with-param name="nodeB" select="$loopCheck"/>
                                    </xsl:call-template>
                                </xsl:variable>
                                <xsl:if test="$diff = ''">
                                    <same_as>
                                        <xsl:value-of select="@name"/>
                                    </same_as>
                                </xsl:if>
                            </xsl:for-each>
                        </fragment>
                    </xsl:if>
                </action>
            </xsl:for-each>
        </bindings>
    </xsl:template>

    <!-- finds the set of conflicting fields and group for the given group -->
    <xsl:template name="find-hide-group-conflicts" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configGroup"/>

        <conflicts>
            <!-- loop through each action that is valid for this page and is not based on another-->
            <xsl:for-each select="$appConfig/fm:application/fm:links/fm:link[contains($actionNames, concat(';', fm:action, ';')) and not(@base)]" >
                <xsl:variable name="actionName" select="fm:action"/>

                <action name="{$actionName}">
                    <!-- Now check for any contained elements or groups that could conflict.
                         We need to pass the names of these through to SXforms to process.
                         TODO: The current approach used to check for conflicts is very basic.
                         It really needs to handle the full xpath for each, including parent repeats!!!!-->
                    <!-- Check if we do have any fields within this group that could possibly conflict with others.
                         At this point we just want to know if there are any-->
                    <xsl:variable name="groupRelatedFields">
                        <!-- create a list of any fields within this group that have the same binding as another field
                             These will need to be disabled when this group is hidden to stop them obverriding correct data-->
                        <xsl:for-each select="$configGroup//fm:element[not(ancestor::fm:path)]">
                            <!-- check if the binding is the same as any other -->
                            <xsl:if test="$bindings/fm:xforms/fm:element[@name = current()/@name]/fm:action[@name = $actionName]/fm:xform_xpath = $bindings/fm:xforms/fm:element[@name != current()/@name]/fm:action[@name = $actionName][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                                <xsl:text>yes</xsl:text>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:variable>
                    <!-- if we have found any conflicting fields, we now need to create a hidden field containing
                         a list of all the field names so that SXForms can filter them out -->
                    <xsl:if test="$groupRelatedFields != ''">
                        <interacting_fields>
                            <!-- first find all the conflicting ones that are not within a repeat between the group and the element
                                 For these elements we know the names now so can enter them directly.-->
                            <xsl:for-each select="$configGroup//fm:element[not(ancestor::fm:path) and not(ancestor::fm:repeat[ancestor::fm:group[@name = $configGroup/@name]])]">
                                <!-- check if the binding is the same as any other -->
                                <xsl:if test="$bindings/fm:xforms/fm:element[@name = current()/@name]/fm:action[@name = $actionName]/fm:xform_xpath = $bindings/fm:xforms/fm:element[@name != current()/@name]/fm:action[@name = $actionName][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                                    <!-- handle repeats above the config group -->
                                    <xsl:if test="ancestor::fm:repeat">
                                        <out:value-of select="${ancestor::fm:repeat[1]/@name}LoopCount"/>
                                    </xsl:if>
                                    <out:text>
                                        <xsl:value-of select="@name"/>
                                        <xsl:text>;</xsl:text>
                                    </out:text>
                                </xsl:if>
                            </xsl:for-each>
                            <!-- Iterate through each repeat contained by this group to check if it has any conflicting elements.
                                 Only look at the first level of contained repeats at this point. -->
                            <xsl:for-each select="$configGroup//fm:repeat[not(ancestor::fm:repeat/ancestor::fm:group[@name = $configGroup/@name])]">
                                <xsl:call-template name="choiceGroupProcessInteracting">
                                    <xsl:with-param name="group" select="$configGroup"/>
                                    <xsl:with-param name="repeat" select="."/>
                                    <xsl:with-param name="mode">field</xsl:with-param>
                                    <xsl:with-param name="action" select="$actionName"/>
                                </xsl:call-template>
                            </xsl:for-each>
                        </interacting_fields>
                    </xsl:if>
                    <!-- list any sub groups within this group that would need to be disabled if it isnt visible-->
                    <xsl:variable name="groupRelatedGroups">
                        <!-- list any sub groups (that can be hidden) that have the same context element as this one -->
                        <xsl:for-each select="$configGroup//fm:group[not(ancestor::fm:path) and ((@conditional_display = 'true' or @hide = 'true') and @submit_if_hidden = 'false')]">
                            <!-- check if this choice has the same context as any others -->
                            <xsl:if test="$bindings/fm:xforms/fm:group[@name = current()/@name]/fm:action[@name = $actionName]/fm:xform_xpath = $bindings/fm:xforms/fm:group[@name != current()/@name]/fm:action[@name = $actionName][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                                <xsl:text>yes</xsl:text>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:if test="$groupRelatedGroups != ''">
                        <interacting_groups>
                            <!-- first find all the conflicting ones that are not within a repeat between the group and the element
                                 For these elements we know the names now so can enter them directly.-->
                            <xsl:for-each select="$configGroup//fm:group[not(ancestor::fm:path) and ((@conditional_display = 'true' or @hide = 'true') and @submit_if_hidden = 'false') and not(ancestor::fm:repeat[ancestor::fm:group[@name = $configGroup/@name]])]">
                                <!-- check if the binding is the same as any other -->
                                <xsl:if test="$bindings/fm:xforms/fm:group[@name = current()/@name]/fm:action[@name = $actionName]/fm:xform_xpath = $bindings/fm:xforms/fm:group[@name != current()/@name]/fm:action[@name = $actionName][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                                    <out:text>
                                        <xsl:text>hyfgroup-</xsl:text>
                                        <!-- if this is a partial page, then include the page name in the id to stop possible conflicts
                                             with data on the main page -->
                                        <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" >
                                            <xsl:value-of select="$page_id" />
                                            <xsl:text>-</xsl:text>
                                        </xsl:if>
                                    </out:text>
                                    <!-- handle repeats above the config group -->
                                    <xsl:if test="ancestor::fm:repeat">
                                        <out:value-of select="${ancestor::fm:repeat[1]/@name}LoopCount"/>
                                    </xsl:if>
                                    <out:text>
                                        <xsl:value-of select="count(ancestor::fm:group[@hide = 'true' or @conditional_display = 'true']) + count(preceding::fm:group[@hide = 'true' or @conditional_display = 'true']) + 1"/>
                                        <xsl:text>-control;</xsl:text>
                                    </out:text>
                                </xsl:if>
                            </xsl:for-each>
                            <!-- Iterate through each repeat contained by this group to check if it has any conflicting groups.
                                 Only look at the first level of contained repeats at this point. -->
                            <xsl:for-each select="$configGroup//fm:repeat[not(ancestor::fm:repeat/ancestor::fm:group[@name = $configGroup/@name])]">
                                <xsl:call-template name="choiceGroupProcessInteracting">
                                    <xsl:with-param name="repeat" select="."/>
                                    <xsl:with-param name="mode">group</xsl:with-param>
                                    <xsl:with-param name="action" select="$actionName"/>
                                </xsl:call-template>
                            </xsl:for-each>
                        </interacting_groups>
                    </xsl:if>
                </action>
            </xsl:for-each>
        </conflicts>
    </xsl:template>

    <!-- checks through the output from the above template to determine which actions have the same set of
         conflicting fileds or groups -->
    <xsl:template name="check-hide-group-conflict-uniqueness">
        <xsl:param name="conflicts"/>
        <conflict_checks>
            <xsl:for-each select="$conflicts/conflicts/action">
                <action name="{@name}">
                    <xsl:if test="interacting_fields/*">
                        <interacting_fields>
                            <xsl:variable name="loopCheck" select="interacting_fields"/>
                            <xsl:for-each select="preceding-sibling::action[interacting_fields/*]">
                                <xsl:variable name="diff" >
                                    <xsl:call-template name="compare-fragments">
                                        <xsl:with-param name="nodeA" select="interacting_fields"/>
                                        <xsl:with-param name="nodeB" select="$loopCheck"/>
                                    </xsl:call-template>
                                </xsl:variable>
                                <xsl:if test="$diff = ''">
                                    <same_as>
                                        <xsl:value-of select="@name"/>
                                    </same_as>
                                </xsl:if>
                            </xsl:for-each>
                        </interacting_fields>
                    </xsl:if>
                    <xsl:if test="interacting_groups/*">
                        <interacting_groups>
                            <xsl:variable name="loopCheck" select="interacting_groups"/>
                            <xsl:for-each select="preceding-sibling::action[interacting_groups/*]">
                                <xsl:variable name="diff" >
                                    <xsl:call-template name="compare-fragments">
                                        <xsl:with-param name="nodeA" select="interacting_groups"/>
                                        <xsl:with-param name="nodeB" select="$loopCheck"/>
                                    </xsl:call-template>
                                </xsl:variable>
                                <xsl:if test="$diff = ''">
                                    <same_as>
                                        <xsl:value-of select="@name"/>
                                    </same_as>
                                </xsl:if>
                            </xsl:for-each>
                        </interacting_groups>
                    </xsl:if>
                </action>
            </xsl:for-each>
        </conflict_checks>
    </xsl:template>


    <!-- template for processing the list of interacting fields or groups.  This handles fields that are within
         repeats, and creates the required loop commands -->
    <xsl:template name="choiceGroupProcessInteracting" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="repeat"/>
        <xsl:param name="mode"/>
        <xsl:param name="action"/>
        <!-- check if this repeat contains any conflicting elements -->
        <xsl:variable name="conflicting">
            <xsl:choose>
                <xsl:when test="$mode = 'field'">
                    <!-- find any fields within this group that have the same binding as another field-->
                    <xsl:for-each select="$repeat//fm:element[not(ancestor::fm:path)]">
                        <!-- check if the binding is the same as any other -->
                        <xsl:if test="$bindings/fm:xforms/fm:element[@name = current()/@name]/fm:action[@name = $action]/fm:xform_xpath = $bindings/fm:xforms/fm:element[@name != current()/@name]/fm:action[@name = $action][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                            <xsl:text>yes</xsl:text>
                        </xsl:if>
                    </xsl:for-each>
                </xsl:when>
                <xsl:when test="$mode= 'group'">
                    <!-- find any fields within this group that have the same binding as another field-->
                    <xsl:for-each select="$repeat//fm:group[not(ancestor::fm:path) and ((@conditional_display = 'true' or @hide = 'true') and @submit_if_hidden = 'false')]">
                        <!-- check if the binding is the same as any other -->
                        <xsl:if test="$bindings/fm:xforms/fm:group[@name = current()/@name]/fm:action[@name = $action]/fm:xform_xpath = $bindings/fm:xforms/fm:group[@name != current()/@name]/fm:action[@name = $action][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                            <xsl:text>yes</xsl:text>
                        </xsl:if>
                    </xsl:for-each>
                </xsl:when>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="$conflicting != ''">
            <out:for-each select="{$bindings/fm:xforms/fm:repeat[@name = $repeat/@name]/fm:repeat_xpath}">
                <!-- if the repeat xpath is not specified put dummy string in the for-each statement -->
                <xsl:if test="not($bindings/fm:xforms/fm:repeat[@name = $repeat/@name]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:repeat[@name = $repeat/@name]/fm:repeat_xpath = '')">
                    <xsl:attribute name="select">repeat_xpath_not_specified</xsl:attribute>
                </xsl:if>
                <!-- Create loop variables -->
                <xsl:choose>
                    <xsl:when test="$repeat/ancestor::fm:repeat">
                        <out:variable name="{$repeat/@name}LoopCount">
                            <out:value-of select="${$repeat/ancestor::fm:repeat[1]/@name}LoopCount"/>
                            <out:text>
                                <xsl:value-of select="$repeat/@name"/>
                            </out:text>
                            <out:value-of select="position()"/>
                        </out:variable>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:variable name="{$repeat/@name}LoopCount">
                            <out:text>
                                <xsl:value-of select="$repeat/@name"/>
                            </out:text>
                            <out:value-of select="position()"/>
                        </out:variable>
                    </xsl:otherwise>
                </xsl:choose>
                <!-- process any contained repeats -->
                <xsl:for-each select="$repeat//fm:repeat[not(ancestor::fm:repeat/ancestor::fm:repeat[@name = $repeat/@name])]">
                    <xsl:call-template name="choiceGroupProcessInteracting">
                        <xsl:with-param name="repeat" select="."/>
                        <xsl:with-param name="mode" select="$mode"/>
                        <xsl:with-param name="action" select="$action"/>
                    </xsl:call-template>
                </xsl:for-each>

                <xsl:choose>
                    <xsl:when test="$mode = 'field'">
                        <!-- output the names of all conflicting elements directly within this repeat -->
                        <xsl:for-each select="$repeat//fm:element[not(ancestor::fm:path) and not(ancestor::fm:repeat[ancestor::fm:repeat[@name = $repeat/@name]])]">
                            <xsl:if test="$bindings/fm:xforms/fm:element[@name = current()/@name]/fm:action[@name = $action]/fm:xform_xpath = $bindings/fm:xforms/fm:element[@name != current()/@name]/fm:action[@name = $action][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                                <out:value-of select="${$repeat/@name}LoopCount"/>
                                <out:text>
                                    <xsl:value-of select="@name"/>
                                    <xsl:text>;</xsl:text>
                                </out:text>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:when test="$mode = 'group'">
                        <!-- output the names of all conflicting elements directly within this repeat -->
                        <xsl:for-each select="$repeat//fm:group[not(ancestor::fm:path) and ((@conditional_display = 'true' or @hide = 'true') and @submit_if_hidden = 'false') and not(ancestor::fm:repeat[ancestor::fm:repeat[@name = $repeat/@name]])]">
                            <xsl:if test="$bindings/fm:xforms/fm:group[@name = current()/@name]/fm:action[@name = $action]/fm:xform_xpath = $bindings/fm:xforms/fm:group[@name != current()/@name]/fm:action[@name = $action][normalize-space(fm:xform_xpath) != '' and fm:xform_xpath != $blank_xpath_value]/fm:xform_xpath">
                                <out:text>
                                    <xsl:text>hyfgroup-</xsl:text>
                                    <!-- if this is a partial page, then include the page name in the id to stop possible conflicts
                                         with data on the main page -->
                                    <xsl:if test="$appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection'" >
                                        <xsl:value-of select="$page_id" />
                                        <xsl:text>-</xsl:text>
                                    </xsl:if>
                                </out:text>
                                <out:value-of select="${$repeat/@name}LoopCount"/>
                                <out:text>
                                    <xsl:value-of select="count(ancestor::fm:group[@hide = 'true' or @conditional_display = 'true']) + count(preceding::fm:group[@hide = 'true' or @conditional_display = 'true']) + 1"/>
                                    <xsl:text>-control;</xsl:text>
                                </out:text>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:when>
                </xsl:choose>
            </out:for-each>
        </xsl:if>
    </xsl:template>
    <!-- Checks if the given config element is a control field for a choice.
          If so this adds an event handler to cope with changing field values,
          and so ensuring the correct choice sections are visible -->
    <xsl:template name="CheckChoiceControlAttachEvent" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:variable name="controlledGroup" select="$formStructure/fm:structure//fm:group[@conditional_display = 'true' and @control_field = $configElement/@name][last()]"/>

        <xsl:if test="$controlledGroup">
            <xsl:variable name="methodName">

                <xsl:variable name="repeatPrefix">
                    <xsl:call-template name="find-control-field-repeat-prefix-from-context">
                        <xsl:with-param name="context" select="$configElement"/>
                        <xsl:with-param name="configGroup" select="$controlledGroup"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:if test="$repeatPrefix != ''" >
                    <xsl:text>{</xsl:text>
                    <xsl:value-of select="$repeatPrefix" />
                    <xsl:text>}</xsl:text>
                </xsl:if>
                <!--<xsl:if test="$configElement/ancestor::fm:repeat and
                                ($controlledGroup/ancestor::fm:repeat/@name = $configElement/ancestor::fm:repeat[1]/@name)">
                    <xsl:text>{$</xsl:text>
                    <xsl:value-of select="$configElement/ancestor::fm:repeat[1]/@name"/>
                    <xsl:text>LoopCount}</xsl:text>
                </xsl:if>-->
                <xsl:value-of select="$configElement/@name"/>
                <xsl:text>ValueChanged(this);</xsl:text>
            </xsl:variable>
            <xsl:variable name="eventTrigger">
                <xsl:choose>
                    <xsl:when test="$configElement/fm:field/fm:input = 'radio' or $configElement/fm:field/fm:input = 'checkbox' or $configElement/fm:field/fm:input = 'multipleCheckbox'">
                        <xsl:text>onclick</xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>onchange</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <!-- check if the element already defines an event for the given trigger so that we don't override it -->
            <xsl:choose>
                <xsl:when test="$configElement/@*[local-name() = $eventTrigger]" >
                    <xsl:apply-templates select="$configElement/@*[local-name() = $eventTrigger]">
                        <xsl:with-param name="extraContent" select="$methodName"/>
                    </xsl:apply-templates>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="{$eventTrigger}">
                        <xsl:value-of select="$methodName"/>
                    </xsl:attribute>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>
    <!-- ensure that any script text gets wrapped with xsl:text tags -->
    <xsl:template match="script/text()">
        <out:text>
            <xsl:copy/>
        </out:text>
    </xsl:template>
    <!-- Matches each form control
        If the control defines a form submission event we need to output a standard submit button for non
        javascript browsers, and then switch it to the correct format if javascript is enabled.
        Otherwise we just output the control normally.
        -->
    <xsl:template match="input | select | textarea | a | img" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="configName">
            <xsl:choose>
                <xsl:when test="@_originalFieldName">
                    <xsl:value-of select="@_originalFieldName"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="@name"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $configName and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <!--<xsl:when test="@name = $firstSubmissionElement/@name">-->
            <xsl:when test="$configElement/fm:events/*/fm:action[@id = 'formSubmission']">
                <xsl:choose>
                    <xsl:when test="$configElement/fm:field/fm:input = 'button' or $configElement/fm:field/fm:input = 'htmlButton'">
                        <!-- replace the button with a submit button -->
                        <!-- QUESTION: what if one control defines multiple form submission events for different actions? -->
                        <input name="xgate_action_{($configElement/fm:events/*/fm:action[@id = 'formSubmission'])[1]/fm:param[@name = 'Action']/@value}" id="{@name}_submit_button" type="submit" value="{$configElement/fm:field/fm:input/@caption}">
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <xsl:attribute name="id">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                    <xsl:value-of select="@name"/>
                                    <xsl:text>_submit_button</xsl:text>
                                </xsl:attribute>
                            </xsl:if>
                        </input>
                        <span id="{@name}_temp_container" style="display:none; visibility:hidden;">
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <xsl:attribute name="id"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="@name"/><xsl:text>_temp_container</xsl:text></xsl:attribute>
                            </xsl:if>
                            <xsl:apply-templates select="." mode="copycontrol"/>
                        </span>
                        <script type="text/javascript">
                            <out:text disable-output-escaping="yes">
                                <xsl:text>document.getElementById('</xsl:text>
                            </out:text>
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                            </xsl:if>
                            <out:text disable-output-escaping="yes">
                                <xsl:value-of select="@name"/>
                                <xsl:text>_submit_button').parentNode.innerHTML=</xsl:text>
                                <xsl:text>document.getElementById('</xsl:text>
                            </out:text>
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                            </xsl:if>
                            <out:text disable-output-escaping="yes">
                                <xsl:value-of select="@name"/>
                                <xsl:text>_temp_container').innerHTML;</xsl:text>
                            </out:text>
                        </script>
                    </xsl:when>
                    <xsl:when test="$configElement/fm:field/fm:input != 'radio' and $configElement/fm:field/fm:input != 'multipleCheckbox'">
                        <!-- Output the control -->
                        <xsl:apply-templates select="." mode="copycontrol"/>
                        <!-- Output a new button to perform the submit action for that control -->
                        <noscript>
                            <input name="xgate_action_{($configElement/fm:events/*/fm:action[@id = 'formSubmission'])[1]/fm:param[@name = 'Action']/@value}" id="{@name}_submit_button" type="submit" value="Submit">
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <xsl:attribute name="id"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="@name"/><xsl:text>_submit_button</xsl:text></xsl:attribute>
                                </xsl:if>
                            </input>
                        </noscript>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- Output the control -->
                        <xsl:apply-templates select="." mode="copycontrol"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="." mode="copycontrol"/>
            </xsl:otherwise>
        </xsl:choose>

        <!-- add in the event calls -->
        <!--<xsl:if test="@*[name() = $eventConfig/fm:event_config/fm:events/*/*/fm:event[@function_name != 'href']/@id]">
            <script type="text/javascript">
                --> <!--only use an onload call if this is not a subsection page, as for a subsection
                     the onload will have already fired.--> <!--
                <xsl:if test="not($appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection')">
                    <out:text>hyf.attachEventHandler(window, 'onload', function() {</out:text>
                </xsl:if>
                <xsl:for-each select="@*[name() = $eventConfig/fm:event_config/fm:events/*/*/fm:event[@function_name != 'href']/@id]">
                    <out:text>
                        <xsl:text>hyf.attachEventHandler(document.getElementById('</xsl:text>
                    </out:text>
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                    </xsl:if>
                    <out:text>
                        <xsl:value-of select="../@id"/>
                        <xsl:text>'), '</xsl:text>
                        <xsl:value-of select="name()"/>
                        <xsl:text>', </xsl:text>
                        <xsl:value-of select="$configName"/>
                        <xsl:value-of select="name()"/>
                        <xsl:text>);</xsl:text>
                    </out:text>
                </xsl:for-each>
                <xsl:if test="not($appConfig/fm:application/fm:pages/fm:page[@id = $page_id]/@type = 'subsection')">
                    <out:text>});</out:text>
                </xsl:if>
            </script>
        </xsl:if>-->

    </xsl:template>
    <xsl:template mode="copycontrol" match="input[@type ='checkbox' and (not(@_use) or (@_use != 'selectMany'))]" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="inputName" select="./@name"/>
        <xsl:variable name="inputEntry" select="$formStructure/fm:structure//fm:element[@name = $inputName and not(ancestor::fm:path)]"/>
        <xsl:copy>
            <xsl:apply-templates select="@*[local-name(.) != '_regularExpression']"/>
            <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
            <xsl:if test="ancestor::repeat_code_placeholder">
                <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$inputName"/></xsl:attribute>
            </xsl:if>
            <!-- Check if this is a choice control field -->
            <xsl:call-template name="CheckChoiceControlAttachEvent">
                <xsl:with-param name="configElement" select="$inputEntry"/>
            </xsl:call-template>
            <xsl:choose>
                <!-- If the checkbox value is dynamic, insert xpath to get the value -->
                <xsl:when test="$inputEntry/fm:field/fm:input/@options = 'dynamic'">
                    <!-- Makes sure the value_xpath has been specified to stop empty AVTs {} or sides of expressions being output-->
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:value_xpath != ''">
                        <xsl:attribute name="value"><xsl:text>{</xsl:text><xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:value_xpath"/><xsl:text>}</xsl:text></xsl:attribute>
                        <!-- Check if the checkbox should be checked -->
                        <!-- Makes sure the transform_xpath has been specified to stop empty AVTs {} or sides of expressions being output-->
                        <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != ''">
                            <out:if test="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:value_xpath} = {$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}">
                                <out:attribute name="checked" namespace="">checked</out:attribute>
                            </out:if>
                            <!-- check if a default value has been provided -->
                            <xsl:if test="$inputEntry/fm:field/fm:input/@default != ''">
                                <out:if test="not({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '')">
                                    <!-- transform_xpath doesnt match anything, so check if default should be used -->
                                    <out:if test="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:value_xpath} = '{$inputEntry/fm:field/fm:input/@default}'">
                                        <out:attribute name="checked" namespace="">checked</out:attribute>
                                    </out:if>
                                </out:if>
                            </xsl:if>
                        </xsl:if>
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                    <!-- Check if the checkbox should be checked -->
                    <!-- Makes sure the transform_xpath has been specified to stop empty AVTs {} or sides of expressions being output-->
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != ''">
                        <out:if test="'{@value}' = {$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}">
                            <out:attribute name="checked" namespace="">checked</out:attribute>
                        </out:if>
                        <!-- check if a default value has been provided -->
                        <xsl:if test="$inputEntry/fm:field/fm:input/@default != ''">
                            <out:if test="not({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '')">
                                <!-- transform_xpath doesnt match anything, so check if default should be used -->
                                <xsl:if test="@value = $inputEntry/fm:field/fm:input/@default">
                                    <out:attribute name="checked" namespace="">checked</out:attribute>
                                </xsl:if>
                            </out:if>
                        </xsl:if>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>
            <!-- Check if this element needs a disable check -->
            <xsl:call-template name="checkElementDisable">
                <xsl:with-param name="configElement" select="$inputEntry"/>
            </xsl:call-template>
        </xsl:copy>
        <!-- Create the hidden field to store the xpath -->
        <xsl:call-template name="create_xform_xpath_field">
            <xsl:with-param name="configElement" select="$inputEntry"/>
        </xsl:call-template>
    </xsl:template>
    <!-- Process each checkbox for use in a selectMany control -->
    <xsl:template mode="copycontrol" match="input[@type ='checkbox' and @_use = 'selectMany']" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="inputName" select="@_element"/>
        <xsl:variable name="inputEntry" select="$formStructure/fm:structure//fm:element[@name = $inputName and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <xsl:when test="$inputEntry/fm:field/fm:input/@options = 'dynamic'">
                <!-- Create a variable to store the total number of controls to create -->
                <xsl:choose>
                    <xsl:when test="not($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath = '')">
                        <out:variable name="totalCount">0</out:variable>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:variable name="totalCount" select="count({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath})"/>
                    </xsl:otherwise>
                </xsl:choose>
                <!-- Create loop to generate checkboxes -->
                <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath}">
                    <!-- if the repeat xpath is not specified put dummy string in the for-each statement -->
                    <xsl:if test="not($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath = '')">
                        <xsl:attribute name="select">repeat_xpath_not_specified</xsl:attribute>
                    </xsl:if>
                    <out:variable name="number" select="position()"/>
                    <xsl:apply-templates select="." mode="displayEnumEntry">
                        <xsl:with-param name="element" select="$inputEntry"/>
                    </xsl:apply-templates>
                </out:for-each>
            </xsl:when>
            <xsl:otherwise>
                <!-- Add check to check box to see if it should be selected -->
                <xsl:copy>
                    <xsl:apply-templates select="@*[(local-name(.) != '_regularExpression') ]"/>
                    <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name">
                            <xsl:text>{$</xsl:text>
                            <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                            <xsl:text>LoopCount}</xsl:text>
                            <xsl:value-of select="@name"/>
                        </xsl:attribute>
                    </xsl:if>
                    <!-- Check if this is a choice control field -->
                    <xsl:call-template name="CheckChoiceControlAttachEvent">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- Check if this element needs dynamic styles -->
                    <xsl:call-template name="checkElementDynamicStyle">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- Check if it should be selected -->
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != ''">

                        <!-- Check the binding format selected for this field, to determine how the data will be stored
                             If not provided, then the 'structure' option is assumed for multiple checkbox fields for backward compatability-->
                        <xsl:choose>
                            <xsl:when test="$bindings/fm:xforms/fm:element[@name = $inputName]/@binding_format[. = 'csv']">
                                <xsl:variable name="separator">
                                    <xsl:choose>
                                        <xsl:when test="$bindings/fm:xforms/fm:element[@name = $inputName]/@value_separator[(. != '') and (. != ',')]" >
                                            <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $inputName]/@value_separator" />
                                        </xsl:when>
                                        <xsl:otherwise>,</xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                <out:if test="contains(concat('{$separator}', {$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}, '{$separator}'), '{$separator}{@value}{$separator}')">
                                    <out:attribute name="checked" namespace="">checked</out:attribute>
                                </out:if>
                            </xsl:when>
                            <xsl:otherwise>
                                <out:if test="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}[. = '{@value}']">
                                    <out:attribute name="checked" namespace="">checked</out:attribute>
                                </out:if>
                            </xsl:otherwise>
                        </xsl:choose>

                        <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                        <xsl:if test="contains(concat(';', $inputEntry/fm:field/fm:input/@default, ';'), concat(';', @value, ';'))">
                            <out:if test="not({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '')">
                                <out:attribute name="checked" namespace="">checked</out:attribute>
                            </out:if>
                        </xsl:if>
                    </xsl:if>
                    <!-- Check if this element needs a disable check -->
                    <xsl:call-template name="checkElementDisable">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>

        <!-- If this is the last checkbox in the set, then create the hidden fields to store the binding information-->
        <xsl:if test="count(following::input[@name= $inputName]) = 0">
            <!-- Check the binding format selected for this field.
                 If not provided, then the 'structure' option is assumed for multiple checkbox fields for backward compatability-->
            <xsl:choose>
                <xsl:when test="$bindings/fm:xforms/fm:element[@name = $inputName]/@binding_format[. = 'csv']">
                    <input type="hidden" value="csv" name="{$inputName}_binding_format">
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:attribute name="name">
                                <xsl:text>{$</xsl:text>
                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                <xsl:text>LoopCount}</xsl:text>
                                <xsl:value-of select="$inputName"/>
                                <xsl:text>_binding_format</xsl:text>
                            </xsl:attribute>
                        </xsl:if>
                        <!-- Check if this element needs a disable check -->
                        <xsl:call-template name="checkElementDisable">
                            <xsl:with-param name="configElement" select="$inputEntry"/>
                        </xsl:call-template>
                    </input>
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/@value_separator[(. != '') and (. != ',')]" >
                        <input type="hidden" value="{$bindings/fm:xforms/fm:element[@name = $inputName]/@value_separator}" name="{$inputName}_value_separator">
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <xsl:attribute name="name">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                    <xsl:value-of select="$inputName"/>
                                    <xsl:text>_value_separator</xsl:text>
                                </xsl:attribute>
                            </xsl:if>
                        </input>
                    </xsl:if>

                    <xsl:call-template name="create_xform_xpath_field">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>

                </xsl:when>
                <xsl:otherwise>
                    <input type="hidden" value="structure" name="{$inputName}_binding_format">
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:attribute name="name">
                                <xsl:text>{$</xsl:text>
                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                <xsl:text>LoopCount}</xsl:text>
                                <xsl:value-of select="$inputName"/>
                                <xsl:text>_binding_format</xsl:text>
                            </xsl:attribute>
                        </xsl:if>
                        <!-- Check if this element needs a disable check -->
                        <xsl:call-template name="checkElementDisable">
                            <xsl:with-param name="configElement" select="$inputEntry"/>
                        </xsl:call-template>
                    </input>

                    <xsl:call-template name="create_xform_xpath_field">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <xsl:call-template name="create_xform_xpath_field">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                        <xsl:with-param name="nameSuffix">_value</xsl:with-param>
                        <xsl:with-param name="keepRelative">true</xsl:with-param>
                        <xsl:with-param name="xpathName">value_xpath</xsl:with-param>
                        <xsl:with-param name="defaultXPathValue">.</xsl:with-param>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>

        </xsl:if>


    </xsl:template>
    <xsl:template mode="copycontrol" match="input[@type != 'radio' and @type != 'checkbox'] | select[@_originalFieldName!='']" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:choose>
            <xsl:when test="not(@_type)">
                <!-- just copy the field -->
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="@name"/></xsl:attribute>
                    </xsl:if>

                    <!-- If this is the hidden field storing the not checked value for a checkbox,
                         then we need to see if it also needs a disable check -->
                    <xsl:if test="substring(@id, string-length(@id) - 22) = '_value_if_not_submitted'">
                        <xsl:call-template name="checkElementDisable">
                            <xsl:with-param name="configElement" select="$formStructure/fm:structure//fm:element[@name = substring-before(current()/@id, '_value_if_not_submitted') and not(ancestor::fm:path)]"/>
                        </xsl:call-template>
                    </xsl:if>

                    <xsl:copy-of select="*"/>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="fullInputName">
                    <xsl:choose>
                        <xsl:when test="./@_originalFieldName[.!='']">
                            <xsl:value-of select="./@_originalFieldName"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="./@name"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="inputName">
                    <xsl:choose>
                        <xsl:when test="@_display_only = 'true'">
                            <xsl:value-of select="substring($fullInputName, 1, string-length($fullInputName) - string-length('_display'))"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$fullInputName"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="inputEntry" select="$formStructure/fm:structure//fm:element[@name = $inputName and not(ancestor::fm:path)]"/>
                <xsl:copy>
                    <xsl:apply-templates select="@*[local-name(.) != '_regularExpression']"/>
                    <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$fullInputName"/></xsl:attribute>
                    </xsl:if>
                    <!-- Check if this is a choice control field -->
                    <xsl:call-template name="CheckChoiceControlAttachEvent">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- set the value attribute to the dynamic xpath if needed -->
                    <xsl:if test="(($inputEntry/fm:field/fm:input/@options = 'dynamic') or ($inputEntry/fm:field/fm:input = 'tabs')) and not(local-name() = 'select')">
                        <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != ''">
                            <xsl:attribute name="value"><xsl:text>{</xsl:text><xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath"/><xsl:text>}</xsl:text></xsl:attribute>
                        </xsl:if>

                        <!-- Insert commands to check if default value should be used instead -->
                        <xsl:variable name="defaultValue" select="$inputEntry/fm:field/fm:input/@default"/>
                        <xsl:if test="$defaultValue != ''">
                            <xsl:choose>
                                <xsl:when test="($bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != '')">
                                    <out:if test="not({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '')">
                                        <out:attribute name="value" namespace="">
                                            <xsl:value-of select="$defaultValue"/>
                                        </out:attribute>
                                    </out:if>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="value">
                                        <xsl:value-of select="$defaultValue"/>
                                    </xsl:attribute>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:if>
                    </xsl:if>

                    <!-- Check if this element needs dynamic styles -->
                    <xsl:call-template name="checkElementDynamicStyle">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- Check if this element needs a disable check -->
                    <xsl:call-template name="checkElementDisable">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- check if the value should be converted in any way - only dynamic values are converted -->
                    <xsl:if test="($inputEntry/fm:field/fm:input/@options = 'dynamic')">
                        <xsl:variable name="conversionRequired">
                            <xsl:call-template name="checkConversionRequired">
                                <xsl:with-param name="configElement" select="$inputEntry"/>
                            </xsl:call-template>
                        </xsl:variable>
                        <xsl:if test="$conversionRequired = 'true'">
                            <xsl:choose>
                                <xsl:when test="./@_selectBoxFormat[.!='']">
                                    <xsl:call-template name="convertValue">
                                        <xsl:with-param name="configElement" select="$inputEntry"/>
                                    </xsl:call-template>
                                </xsl:when>
                                <!-- only do non date dropdown conversions when a bindings xpath has been provided
                                     We always need to do the date dropdown stuff as this creates the entries for the user to select-->
                                <xsl:when test="($bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != '')" >
                                    <out:attribute name="value" namespace="">
                                        <xsl:call-template name="convertValue">
                                            <xsl:with-param name="configElement" select="$inputEntry"/>
                                        </xsl:call-template>
                                    </out:attribute>
                                </xsl:when>
                            </xsl:choose>
                        </xsl:if>
                    </xsl:if>
                </xsl:copy>
                <!-- Create the hidden field to store the xpath -->
                <xsl:if test="($inputEntry/fm:field/fm:input != 'htmlButton') and not(preceding-sibling::*[@_originalFieldName=$fullInputName])">
                    <xsl:call-template name="create_xform_xpath_field">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                </xsl:if>
                <xsl:variable name="test">
                    <xsl:value-of select="$fullInputName"/>
                </xsl:variable>
                <xsl:if test="not(following-sibling::*[@_originalFieldName=$fullInputName])">
                <xsl:apply-templates mode="createDatePicker" select="$inputEntry">
                    <xsl:with-param name="repeatName">
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                        </xsl:if>
                    </xsl:with-param>
                </xsl:apply-templates>
                </xsl:if>
                <!-- if this is a tabs field, create the needed script function to handle tab clicks-->
                <xsl:if test="$inputEntry/fm:field/fm:input = 'tabs'">
                    <script type="text/javascript">
                        <out:text>function </out:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                            <xsl:value-of select="$inputName"/>
                            <xsl:text>TabChange(value) {
                        document.getElementById('</xsl:text>
                        </out:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                            <xsl:value-of select="$inputName"/>
                            <xsl:text>').value = value;
                        document.getElementById('</xsl:text>
                        </out:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
                        </xsl:if>
                        <out:text>
                            <xsl:value-of select="$inputName"/>
                            <xsl:text>').onchange();
                        </xsl:text>
                        </out:text>

                        <!-- first make all the tabs appear unselected -->
                        <xsl:choose>
                            <xsl:when test="$inputEntry/fm:field/fm:input/@options = 'dynamic'">
                                <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath != ''">
                                    <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath}">
                                        <out:text>
                                            <xsl:text>document.getElementById('</xsl:text>
                                            <xsl:if test="ancestor::repeat_code_placeholder">
                                                <xsl:attribute name="id">
                                                    <xsl:text>{$</xsl:text>
                                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                                    <xsl:text>LoopCount}</xsl:text>
                                                </xsl:attribute>
                                            </xsl:if>
                                            <xsl:value-of select="$inputName"/>
                                            <xsl:text>_tab_</xsl:text>
                                        </out:text>
                                        <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:value_xpath}"/>
                                        <out:text>
                                            <xsl:text>').className = '</xsl:text>
                                            <xsl:value-of select="$inputEntry/fm:field/fm:input/@unselected_tab_class"/>
                                            <xsl:text>';
                            </xsl:text>
                                        </out:text>
                                    </out:for-each>
                                </xsl:if>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:for-each select="..//a[@_use = 'tab'] " >
                                    <out:text>
                                        <xsl:text>document.getElementById('</xsl:text>
                                        <xsl:if test="ancestor::repeat_code_placeholder">
                                            <xsl:attribute name="id">
                                                <xsl:text>{$</xsl:text>
                                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                                <xsl:text>LoopCount}</xsl:text>
                                            </xsl:attribute>
                                        </xsl:if>
                                        <xsl:value-of select="@id"/>
                                        <xsl:text>').className = '</xsl:text>
                                        <xsl:value-of select="$inputEntry/fm:field/fm:input/@unselected_tab_class"/>
                                        <xsl:text>';
                            </xsl:text>
                                    </out:text>
                                </xsl:for-each>
                            </xsl:otherwise>
                        </xsl:choose>

                        <out:text>
                        <!-- now make the selected tab appear correctly -->
                        <xsl:text>document.getElementById('</xsl:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:attribute name="id">
                                <xsl:text>{$</xsl:text>
                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                <xsl:text>LoopCount}</xsl:text>
                            </xsl:attribute>
                        </xsl:if>
                        <xsl:value-of select="$inputName"/>
                        <xsl:text>_tab_</xsl:text>
                        <xsl:text>' + value).className = '</xsl:text>
                        <xsl:value-of select="$inputEntry/fm:field/fm:input/@selected_tab_class"/>
                        <xsl:text>';
                        return false; }</xsl:text>
                        </out:text>
                    </script>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template mode="copycontrol" match="textarea" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="inputName" select="./@name"/>
        <xsl:variable name="inputEntry" select="$formStructure/fm:structure//fm:element[@name = $inputName and not(ancestor::fm:path)]"/>
        <xsl:copy>
            <xsl:apply-templates select="@*[(local-name(.) != '_regularExpression')]"/>
            <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
            <xsl:if test="ancestor::repeat_code_placeholder">
                <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$inputName"/></xsl:attribute>
            </xsl:if>
            <!-- Check if this is a choice control field -->
            <xsl:call-template name="CheckChoiceControlAttachEvent">
                <xsl:with-param name="configElement" select="$inputEntry"/>
            </xsl:call-template>
            <!-- Check if this element needs a disable check -->
            <xsl:call-template name="checkElementDisable">
                <xsl:with-param name="configElement" select="$inputEntry"/>
            </xsl:call-template>
            <!-- Check if this element needs dynamic styles -->
            <xsl:call-template name="checkElementDynamicStyle">
                <xsl:with-param name="configElement" select="$inputEntry"/>
            </xsl:call-template>
            <!-- Output the value -->
            <xsl:choose>
                <xsl:when test="$inputEntry/fm:field/fm:input/@options = 'dynamic'">
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != ''">
                        <xsl:call-template name="convertValue">
                            <xsl:with-param name="configElement" select="$inputEntry"/>
                        </xsl:call-template>
                        <!-- Insert commands to check if default value should be used instead -->
                        <xsl:variable name="defaultValue" select="$inputEntry/fm:field/fm:input/@default"/>
                        <xsl:if test="$defaultValue != ''">
                            <out:if test="not({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '')">
                                <out:text>
                                    <xsl:value-of select="$defaultValue"/>
                                </out:text>
                            </out:if>
                        </xsl:if>
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="."/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
        <!-- Create the hidden field to store the xpath -->
        <xsl:call-template name="create_xform_xpath_field">
            <xsl:with-param name="configElement" select="$inputEntry"/>
        </xsl:call-template>
        <xsl:apply-templates mode="createDatePicker" select="$inputEntry">
            <xsl:with-param name="repeatName">
                <xsl:if test="ancestor::repeat_code_placeholder">
                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                </xsl:if>
            </xsl:with-param>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template mode="copycontrol" match="select" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="selectName" select="@name"/>
        <xsl:variable name="selectEntry" select="$formStructure/fm:structure//fm:element[@name = $selectName and not(ancestor::fm:path)]"/>
        <xsl:copy>
            <xsl:apply-templates select="@*[(local-name(.) != '_regularExpression')]"/>
            <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
            <xsl:if test="ancestor::repeat_code_placeholder">
                <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$selectName"/></xsl:attribute>
            </xsl:if>
            <!-- Check if this is a choice control field -->
            <xsl:call-template name="CheckChoiceControlAttachEvent">
                <xsl:with-param name="configElement" select="$selectEntry"/>
            </xsl:call-template>
            <!-- Check if this element needs a disable check -->
            <xsl:call-template name="checkElementDisable">
                <xsl:with-param name="configElement" select="$selectEntry"/>
            </xsl:call-template>
            <!-- Check if this element needs dynamic styles -->
            <xsl:call-template name="checkElementDynamicStyle">
                <xsl:with-param name="configElement" select="$selectEntry"/>
            </xsl:call-template>
            <!-- Copy the default entry (eg containing ' Please Select One ') if present -->
            <xsl:apply-templates select="option[@_use = 'default']"/>
            <xsl:choose>
                <xsl:when test="$selectEntry/fm:field/fm:input/@options = 'dynamic'">
                    <!-- Create loop to generate the options -->
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $selectName]/fm:repeat_xpath != ''">
                        <out:variable name="checkDefault" select="(not({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} = ''))"/>

                        <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:repeat_xpath}">
                            <xsl:choose>
                                <!-- Check if multiple selection is allowed -->
                                <xsl:when test="$selectEntry/fm:field/fm:input/@multiple_selection = 'true'">


                                    <!-- Check the binding format for this field to see how the data is stored
                                         For the multiple select control we assume csv if not specified for backwards compatability -->
                                    <xsl:choose>
                                        <xsl:when test="$bindings/fm:xforms/fm:element[@name = $selectName]/@binding_format[. = 'structure']">
                                            <option value="{{{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath}}}">
                                                <out:if test="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath} = {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}">
                                                    <out:attribute name="selected" namespace="">selected</out:attribute>
                                                </out:if>
                                                <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                                                <xsl:if test="$selectEntry/fm:field/fm:input/@default != ''">
                                                    <out:if test="$checkDefault and contains(';{$selectEntry/fm:field/fm:input/@default};', concat(';', {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath}, ';'))">
                                                        <out:attribute name="selected" namespace="">selected</out:attribute>
                                                    </out:if>
                                                </xsl:if>
                                                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:text_xpath}"/>
                                            </option>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:variable name="separator">
                                                <xsl:choose>
                                                    <xsl:when test="$bindings/fm:xforms/fm:element[@name = $selectName]/@value_separator[(. != '') and (. != ',')]" >
                                                        <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $selectName]/@value_separator" />
                                                    </xsl:when>
                                                    <xsl:otherwise>,</xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:variable>
                                            <option value="{{{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath}}}">
                                                <out:if test="contains(concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath},'{$separator}'), concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath},'{$separator}'))">
                                                    <out:attribute name="selected" namespace="">selected</out:attribute>
                                                </out:if>
                                                <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                                                <xsl:if test="$selectEntry/fm:field/fm:input/@default != ''">
                                                    <out:if test="$checkDefault and contains('{$separator}{translate($selectEntry/fm:field/fm:input/@default, ';', $separator)}{$separator}', concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath},'{$separator}'))">
                                                        <out:attribute name="selected" namespace="">selected</out:attribute>
                                                    </out:if>
                                                </xsl:if>
                                                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:text_xpath}"/>
                                            </option>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <option value="{{{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath}}}">
                                        <out:if test="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath} = {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}">
                                            <out:attribute name="selected" namespace="">selected</out:attribute>
                                        </out:if>
                                        <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                                        <xsl:if test="$selectEntry/fm:field/fm:input/@default != ''">
                                            <out:if test="$checkDefault and ({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath} = '{$selectEntry/fm:field/fm:input/@default}')">
                                                <out:attribute name="selected" namespace="">selected</out:attribute>
                                            </out:if>
                                        </xsl:if>
                                        <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:text_xpath}"/>
                                    </option>
                                </xsl:otherwise>
                            </xsl:choose>
                        </out:for-each>
                    </xsl:if>
                    <!-- Copy any other options, eg the editable option for a combobox -->
                    <xsl:for-each select="option[not(@_use) or (@_use != 'default')]">
                        <xsl:copy>
                            <xsl:copy-of select="@*"/>
                            <xsl:if test="@_editable = 'true'">
                                <out:choose>
                                    <out:when test="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} != ''">
                                        <out:choose>
                                            <xsl:choose>
                                                <xsl:when test="starts-with($bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath, '/')">
                                                    <out:when test="not({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} = {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath})">
                                                        <out:attribute name="selected" namespace="">selected</out:attribute>
                                                        <out:attribute name="value" namespace="">
                                                            <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}"/>
                                                        </out:attribute>
                                                        <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}"/>
                                                    </out:when>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <out:when test="not({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} = {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:repeat_xpath}/{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:value_xpath})">
                                                        <out:attribute name="selected" namespace="">selected</out:attribute>
                                                        <out:attribute name="value" namespace="">
                                                            <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}"/>
                                                        </out:attribute>
                                                        <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}"/>
                                                    </out:when>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                            <out:otherwise>
                                                <xsl:apply-templates/>
                                            </out:otherwise>
                                        </out:choose>
                                    </out:when>
                                    <out:otherwise>
                                        <xsl:apply-templates/>
                                    </out:otherwise>
                                </out:choose>
                            </xsl:if>
                        </xsl:copy>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <!-- Create command to check which option should be selected -->
                    <xsl:for-each select="option[not(@_use) or (@_use != 'default')]">
                        <xsl:copy>
                            <xsl:apply-templates select="@*"/>
                            <xsl:choose>
                                <xsl:when test="$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath != ''">
                                    <xsl:choose>
                                        <xsl:when test="@_editable = 'true'">
                                            <out:choose>
                                                <out:when test="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} != ''">
                                                    <out:choose>
                                                        <out:when test="">
                                                            <xsl:attribute name="test">
                                                                <xsl:for-each select="../option[not(@_use = 'default') and not(@_editable = 'true')]">
                                                                    <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath"/>
                                                                    <xsl:text> != '</xsl:text>
                                                                    <xsl:value-of select="@value"/>
                                                                    <xsl:text>'</xsl:text>
                                                                    <xsl:if test="position() != last()">
                                                                        <xsl:text> and </xsl:text>
                                                                    </xsl:if>
                                                                </xsl:for-each>
                                                            </xsl:attribute>
                                                            <out:attribute name="selected" namespace="">selected</out:attribute>
                                                            <out:attribute name="value" namespace="">
                                                                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}"/>
                                                            </out:attribute>
                                                            <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}"/>
                                                        </out:when>
                                                        <out:otherwise>
                                                            <xsl:apply-templates/>
                                                        </out:otherwise>
                                                    </out:choose>
                                                </out:when>
                                                <out:otherwise>
                                                    <xsl:apply-templates/>
                                                </out:otherwise>
                                            </out:choose>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:choose>
                                                <!-- when multiple -->
                                                <xsl:when test="$selectEntry/fm:field/fm:input/@multiple_selection = 'true'">

                                                    <!-- Check the binding format for this field to see how the data is stored
                                                         For the multiple select control we assume csv if not specified for backwards compatability -->
                                                    <xsl:choose>
                                                        <xsl:when test="$bindings/fm:xforms/fm:element[@name = $selectName]/@binding_format[. = 'structure']">
                                                            <out:if test="'{@value}' = {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}">
                                                                <out:attribute name="selected" namespace="">selected</out:attribute>
                                                            </out:if>
                                                        </xsl:when>
                                                        <xsl:otherwise>
                                                            <xsl:variable name="separator">
                                                                <xsl:choose>
                                                                    <xsl:when test="$bindings/fm:xforms/fm:element[@name = $selectName]/@value_separator[(. != '') and (. != ',')]" >
                                                                        <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $selectName]/@value_separator" />
                                                                    </xsl:when>
                                                                    <xsl:otherwise>,</xsl:otherwise>
                                                                </xsl:choose>
                                                            </xsl:variable>

                                                            <out:if test="contains(concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath},'{$separator}'),'{concat($separator, @value, $separator)}')">
                                                                <out:attribute name="selected" namespace="">selected</out:attribute>
                                                            </out:if>
                                                        </xsl:otherwise>
                                                    </xsl:choose>

                                                    <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                                                    <xsl:if test="contains(concat(';',$selectEntry/fm:field/fm:input/@default,';'), concat(';', @value, ';'))">
                                                        <out:if test="not({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} = '')">
                                                            <out:attribute name="selected" namespace="">selected</out:attribute>
                                                        </out:if>
                                                    </xsl:if>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <out:if test="'{@value}' = {$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}">
                                                        <out:attribute name="selected" namespace="">selected</out:attribute>
                                                    </out:if>
                                                    <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                                                    <xsl:if test="@value = $selectEntry/fm:field/fm:input/@default">
                                                        <out:if test="not({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $selectName]/fm:transform_xpath} = '')">
                                                            <out:attribute name="selected" namespace="">selected</out:attribute>
                                                        </out:if>
                                                    </xsl:if>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                            <xsl:apply-templates/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:apply-templates/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:copy>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>

        <!-- Create the hidden field to store the xpath -->
        <xsl:call-template name="create_xform_xpath_field">
            <xsl:with-param name="configElement" select="$selectEntry"/>
        </xsl:call-template>

        <!-- Check if multiple selction is enabled, and if so output the additional hidden fields required -->
        <xsl:if test="$selectEntry/fm:field/fm:input/@multiple_selection = 'true'" >


            <!-- Check the binding format selected for this field.
                 If not provided, then the 'structure' option is assumed for select fields for backward compatability-->
            <xsl:choose>
                <xsl:when test="$bindings/fm:xforms/fm:element[@name = $selectName]/@binding_format[. = 'structure']">
                    <input type="hidden" value="structure" name="{$selectName}_binding_format">
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:attribute name="name">
                                <xsl:text>{$</xsl:text>
                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                <xsl:text>LoopCount}</xsl:text>
                                <xsl:value-of select="$selectName"/>
                                <xsl:text>_binding_format</xsl:text>
                            </xsl:attribute>
                        </xsl:if>
                        <!-- Check if this element needs a disable check -->
                        <xsl:call-template name="checkElementDisable">
                            <xsl:with-param name="configElement" select="$selectEntry"/>
                        </xsl:call-template>
                    </input>

                    <xsl:call-template name="create_xform_xpath_field">
                        <xsl:with-param name="configElement" select="$selectEntry"/>
                        <xsl:with-param name="nameSuffix">_value</xsl:with-param>
                        <xsl:with-param name="keepRelative">true</xsl:with-param>
                        <xsl:with-param name="xpathName">value_xpath</xsl:with-param>
                        <xsl:with-param name="defaultXPathValue">.</xsl:with-param>
                    </xsl:call-template>

                </xsl:when>
                <xsl:otherwise>
                    <input type="hidden" value="csv" name="{$selectName}_binding_format">
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:attribute name="name">
                                <xsl:text>{$</xsl:text>
                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                <xsl:text>LoopCount}</xsl:text>
                                <xsl:value-of select="$selectName"/>
                                <xsl:text>_binding_format</xsl:text>
                            </xsl:attribute>
                        </xsl:if>
                        <!-- Check if this element needs a disable check -->
                        <xsl:call-template name="checkElementDisable">
                            <xsl:with-param name="configElement" select="$selectEntry"/>
                        </xsl:call-template>
                    </input>
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $selectName]/@value_separator[(. != '') and (. != ',')]" >
                        <input type="hidden" value="{$bindings/fm:xforms/fm:element[@name = $selectName]/@value_separator}" name="{$selectName}_value_separator">
                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <xsl:attribute name="name">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                    <xsl:value-of select="$selectName"/>
                                    <xsl:text>_value_separator</xsl:text>
                                </xsl:attribute>
                            </xsl:if>
                        </input>
                    </xsl:if>

                </xsl:otherwise>
            </xsl:choose>

        </xsl:if>


        <!-- check if this is a combobox and so we need to create the required javascript -->
        <xsl:if test="$selectEntry/fm:field/fm:input = 'combobox'">
            <xsl:apply-templates select="$selectEntry" mode="createComboBoxScript">
                <xsl:with-param name="repeatName">
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                    </xsl:if>
                </xsl:with-param>
                <xsl:with-param name="elementName" select="$selectName"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <!-- wrap any text strings within option tags with a xsl:text element.
         This seems to stop tabs and newlines being output in the resulting HTML within each option tag.
         This isn't a problem functionally, but mackes the HTML look messier! -->
    <xsl:template match="option/text()">
        <out:text>
            <xsl:copy/>
        </out:text>
    </xsl:template>


    <!-- Insert the xpath details for radio buttons, whether static or dynamic -->
    <xsl:template mode="copycontrol" match="input[@type = 'radio']" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="inputName" select="@name"/>
        <xsl:variable name="inputEntry" select="$formStructure/fm:structure//fm:element[@name = $inputName and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <xsl:when test="$inputEntry/fm:field/fm:input/@options = 'dynamic'">
                <!-- Create a variable to store the total number of controls to create -->
                <xsl:choose>
                    <xsl:when test="not($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath = '')">
                        <out:variable name="totalCount">0</out:variable>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:variable name="totalCount" select="count({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath})"/>
                    </xsl:otherwise>
                </xsl:choose>
                <!-- Create loop to generate radio buttons -->
                <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath}">
                    <!-- if the repeat xpath is not specified put dummy string in the for-each statement -->
                    <xsl:if test="not($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath) or ($bindings/fm:xforms/fm:element[@name = $inputName]/fm:repeat_xpath = '')">
                        <xsl:attribute name="select">repeat_xpath_not_specified</xsl:attribute>
                    </xsl:if>
                    <out:variable name="number" select="position()"/>
                    <xsl:apply-templates select="." mode="displayEnumEntry">
                        <xsl:with-param name="element" select="$inputEntry"/>
                    </xsl:apply-templates>
                </out:for-each>
                <!-- Create the hidden field to store the xpath -->
                <xsl:call-template name="create_xform_xpath_field">
                    <xsl:with-param name="configElement" select="$inputEntry"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <!-- Add check to radio button to see if it should be selected -->
                <xsl:copy>
                    <xsl:apply-templates select="@*[(local-name(.) != '_regularExpression')]"/>
                    <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$inputName"/></xsl:attribute>
                    </xsl:if>
                    <!-- Check if this is a choice control field -->
                    <xsl:call-template name="CheckChoiceControlAttachEvent">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- Check if it should be selected -->
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath != ''">
                        <out:if test="{$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '{@value}'">
                            <out:attribute name="checked" namespace="">checked</out:attribute>
                        </out:if>
                        <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                        <xsl:if test="@value = $inputEntry/fm:field/fm:input/@default">
                            <out:if test="not({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $inputName]/fm:transform_xpath} = '')">
                                <out:attribute name="checked" namespace="">checked</out:attribute>
                            </out:if>
                        </xsl:if>
                    </xsl:if>
                    <!-- Check if this element needs a disable check -->
                    <xsl:call-template name="checkElementDisable">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                    <!-- Check if this element needs dynamic styles -->
                    <xsl:call-template name="checkElementDynamicStyle">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                </xsl:copy>
                <!-- If this is the last radio button in the group, create the hidden field to hold the xpath -->
                <xsl:if test="count(following::input[@name= $inputName]) = 0">
                    <xsl:call-template name="create_xform_xpath_field">
                        <xsl:with-param name="configElement" select="$inputEntry"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="input[@type = 'radio' or @type = 'checkbox']" mode="displayEnumEntry" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="element"/>
        <xsl:choose>
            <xsl:when test="(string(number($element/fm:field/fm:input/@size)) = 'NaN') and (string(number($element/fm:field/fm:input/@size-y)) = 'NaN')">
                <!-- if neither rows or colums defined, use old approach, of just using spans.  Therefore dont need
                    to output any starting section-->
            </xsl:when>
            <xsl:otherwise>
                <out:choose>
                    <out:when test="$number = 1">
                        <out:text disable-output-escaping="yes">&lt;table cellspacing="0" cellpadding="0" border="0"&gt;&lt;tr&gt;</out:text>
                    </out:when>
                    <xsl:if test="string(number($element/fm:field/fm:input/@size)) != 'NaN'">
                        <out:when test="({$element/fm:field/fm:input/@size} = 1) or (($number mod {$element/fm:field/fm:input/@size}) = 1)">
                            <out:text disable-output-escaping="yes">&lt;/tr&gt;&lt;tr&gt;</out:text>
                        </out:when>
                    </xsl:if>
                    <xsl:if test="string(number($element/fm:field/fm:input/@size-y)) != 'NaN'">
                        <out:when test="($totalCount &lt;= {$element/fm:field/fm:input/@size-y}) or  (($number mod ceiling($totalCount div {$element/fm:field/fm:input/@size-y})) = 1)">
                            <out:text disable-output-escaping="yes">&lt;/tr&gt;&lt;tr&gt;</out:text>
                        </out:when>
                    </xsl:if>
                </out:choose>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:variable name="container_tag">
            <xsl:choose>
                <xsl:when test="(string(number($element/fm:field/fm:input/@size)) = 'NaN') and (string(number($element/fm:field/fm:input/@size-y)) = 'NaN')">
                    <!-- if neither rows or colums defined, use old approach, of just using spans. -->
                    <xsl:text>span</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <!-- otherwise use a table to achieve the required layout -->
                    <xsl:text>td</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- Work out whether the label or the control should be output first -->
        <xsl:choose>
            <xsl:when test="$element/fm:field/fm:input/@option_labels = 'before'">
                <xsl:apply-templates select="." mode="createEnumerationLabel">
                    <xsl:with-param name="control_type" select="$element/fm:field/fm:input"/>
                    <xsl:with-param name="container_tag" select="$container_tag"/>
                    <xsl:with-param name="element" select="$element"/>
                </xsl:apply-templates>
                <xsl:apply-templates select="." mode="createEnumerationControl">
                    <xsl:with-param name="control_type" select="$element/fm:field/fm:input"/>
                    <xsl:with-param name="container_tag" select="$container_tag"/>
                    <xsl:with-param name="element" select="$element"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="." mode="createEnumerationControl">
                    <xsl:with-param name="control_type" select="$element/fm:field/fm:input"/>
                    <xsl:with-param name="container_tag" select="$container_tag"/>
                    <xsl:with-param name="element" select="$element"/>
                </xsl:apply-templates>
                <xsl:apply-templates select="." mode="createEnumerationLabel">
                    <xsl:with-param name="control_type" select="$element/fm:field/fm:input"/>
                    <xsl:with-param name="container_tag" select="$container_tag"/>
                    <xsl:with-param name="element" select="$element"/>
                </xsl:apply-templates>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="(string(number($element/fm:field/fm:input/@size)) = 'NaN') and (string(number($element/fm:field/fm:input/@size-y)) = 'NaN')">
                <!-- if neither rows or colums defined, use old approach, of just using spans.  Therefore dont need
                    to output any ending section-->
            </xsl:when>
            <xsl:otherwise>
                <!-- otherwise, if this is the last entry, close off the table -->
                <out:if test="$number = $totalCount">
                    <out:text disable-output-escaping="yes">&lt;/tr&gt;&lt;/table&gt;</out:text>
                </out:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="input[@type = 'radio' or @type = 'checkbox']" mode="createEnumerationControl" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="control_type"/>
        <xsl:param name="container_tag"/>
        <xsl:param name="element"/>
        <xsl:element name="{$container_tag}">
            <xsl:attribute name="class">
                <xsl:value-of select="@_spanClass"/>
            </xsl:attribute>
            <xsl:copy>
                <xsl:apply-templates select="@*[(local-name(.) != '_regularExpression') and (local-name(.) != '_spanClass') and (local-name(.) != '_captionClass')]"/>
                <xsl:choose>
                    <xsl:when test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name">
                            <xsl:text>{$</xsl:text>
                            <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                            <xsl:text>LoopCount}</xsl:text>
                            <xsl:value-of select="$element/@name"/>
                        </xsl:attribute>
                        <xsl:attribute name="id">
                            <xsl:text>{$</xsl:text>
                            <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                            <xsl:text>LoopCount}</xsl:text>
                            <xsl:value-of select="$element/@name"/>
                            <xsl:text>{$number}</xsl:text>
                        </xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:attribute name="name">
                            <xsl:value-of select="$element/@name"/>
                        </xsl:attribute>
                        <xsl:attribute name="id">
                            <xsl:value-of select="$element/@name"/>
                            <xsl:text>{$number}</xsl:text>
                        </xsl:attribute>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:if test="$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath != ''">
                    <xsl:attribute name="value">
                        <xsl:text>{</xsl:text>
                        <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath"/>
                        <xsl:text>}</xsl:text>
                    </xsl:attribute>
                </xsl:if>
                <!-- Check if this is a choice control field -->
                <xsl:call-template name="CheckChoiceControlAttachEvent">
                    <xsl:with-param name="configElement" select="$element"/>
                </xsl:call-template>
                <!-- Create check to see if this control should be selected -->
                <xsl:if test="($bindings/fm:xforms/fm:element[@name = $element/@name]/fm:transform_xpath != '') and ($bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath != '')">

                    <xsl:choose>
                        <xsl:when test="($control_type = multipleCheckbox) and ($bindings/fm:xforms/fm:element[@name = $element/@name]/@binding_format = 'csv')">
                            <xsl:variable name="separator">
                                <xsl:choose>
                                    <xsl:when test="$bindings/fm:xforms/fm:element[@name = $element/@name]/@value_separator[(. != '') and (. != ',')]" >
                                        <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $element/@name]/@value_separator" />
                                    </xsl:when>
                                    <xsl:otherwise>,</xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <out:if test="contains(concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:transform_xpath},'{$separator}'), concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath},'{$separator}'))">
                                <out:attribute name="checked" namespace="">checked</out:attribute>
                            </out:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <out:if test="{$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath} = {$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:transform_xpath}">
                                <out:attribute name="checked" namespace="">checked</out:attribute>
                            </out:if>
                        </xsl:otherwise>
                    </xsl:choose>


                    <!-- Check if the default value should be used -->
                    <xsl:if test="$element/fm:field/fm:input/@default != ''">
                        <out:if test="not({$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:transform_xpath} = '')">
                            <xsl:choose>
                                <xsl:when test="($control_type = multipleCheckbox)">
                                    <xsl:variable name="separator">
                                        <xsl:choose>
                                            <xsl:when test="$bindings/fm:xforms/fm:element[@name = $element/@name]/@value_separator[(. != '') and (. != ',')]" >
                                                <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $element/@name]/@value_separator" />
                                            </xsl:when>
                                            <xsl:otherwise>,</xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:variable>
                                    <out:if test="contains('{$separator}{translate($element/fm:field/fm:input/@default, ';', $separator)}{$separator}', concat('{$separator}',{$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath},'{$separator}'))">
                                        <out:attribute name="checked" namespace="">checked</out:attribute>
                                    </out:if>
                                </xsl:when>
                                <xsl:otherwise>
                                    <out:if test="{$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:value_xpath} = '{$element/fm:field/fm:input/@default}'">
                                        <out:attribute name="checked" namespace="">checked</out:attribute>
                                    </out:if>
                                </xsl:otherwise>
                            </xsl:choose>
                        </out:if>
                    </xsl:if>
                </xsl:if>
                <!-- Check if this element needs a disable check -->
                <xsl:call-template name="checkElementDisable">
                    <xsl:with-param name="configElement" select="$element"/>
                </xsl:call-template>
                <!-- Check if this element needs dynamic styles -->
                <xsl:call-template name="checkElementDynamicStyle">
                    <xsl:with-param name="configElement" select="$element"/>
                </xsl:call-template>
            </xsl:copy>
        </xsl:element>
    </xsl:template>
    <xsl:template match="input[@type = 'radio' or @type = 'checkbox']" mode="createEnumerationLabel" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="control_type"/>
        <xsl:param name="container_tag"/>
        <xsl:param name="element"/>
        <xsl:variable name="display_xpath">
            <xsl:choose>
                <xsl:when test="$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:text_xpath = ''">
                    <xsl:value-of select="$blank_xpath_value"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $element/@name]/fm:text_xpath"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="$container_tag = 'td'">
                <td class="{@_captionClass}">
                    <label for="{$element/@name}{{$number}}">
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:attribute name="for"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$element/@name"/><xsl:text>{$number}</xsl:text></xsl:attribute>
                        </xsl:if>
                        <out:value-of select="{$display_xpath}"/>
                    </label>
                </td>
            </xsl:when>
            <xsl:otherwise>
                <label for="{$element/@name}{{$number}}" class="{@_captionClass}">
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="for"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="$element/@name"/><xsl:text>{$number}</xsl:text></xsl:attribute>
                    </xsl:if>
                    <out:value-of select="{$display_xpath}"/>
                </label>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Creates the hidden field to store the xform xpath for the given element -->
    <xsl:template name="create_xform_xpath_field" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="xpathSuffix"/>
        <xsl:param name="nameSuffix"/>
        <xsl:param name="keepRelative">false</xsl:param>
        <xsl:param name="xpathName">xform_xpath</xsl:param>
        <xsl:param name="defaultXPathValue" select="$blank_xpath_value"/>

        <!-- create the bindings for this field for every action -->
        <xsl:variable name="fieldBindings">
            <xsl:call-template name="find-field-xpath-bindings">
                <xsl:with-param name="configElement" select="$configElement"/>
                <xsl:with-param name="keepRelative" select="$keepRelative"/>
                <xsl:with-param name="xpathName" select="$xpathName"/>
                <xsl:with-param name="defaultXPathValue" select="$defaultXPathValue"/>
            </xsl:call-template>
        </xsl:variable>

        <!--<test>
            <xsl:call-template name="find-field-xpath-bindings">
                <xsl:with-param name="configElement" select="$configElement"/>
            </xsl:call-template>
            <xsl:copy-of select="$configElement" />
        </test>-->

        <xsl:variable name="fieldBindingsSet" select="exsl:node-set($fieldBindings)"/>

        <!-- loop through each unique xpath found -->
        <xsl:for-each select="$fieldBindingsSet/bindings/action[not(. = preceding-sibling::action) and (. != $blank_xpath_value)]">


            <input type="hidden" value="{.}{$xpathSuffix}">
                <xsl:attribute name="name">
                    <!-- only add the action naming stuff if we have more than one binding -->
                    <xsl:if test="count($fieldBindingsSet/bindings/action[not(. = preceding-sibling::action)]) != 1" >
                        <xsl:text>xgf_</xsl:text>
                    </xsl:if>

                    <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
                    <xsl:if test="$configElement/ancestor::fm:repeat">
                        <xsl:text>{$</xsl:text>
                        <xsl:value-of select="$configElement/ancestor::fm:repeat[1]/@name"/>
                        <xsl:text>LoopCount}</xsl:text>
                    </xsl:if>
                    <xsl:value-of select="$configElement/@name"/>
                    <xsl:value-of select="$nameSuffix"/>
                    <xsl:text>_xpath</xsl:text>

                    <!-- only add the action naming stuff if we have more than one binding -->
                    <xsl:if test="count($fieldBindingsSet/bindings/action[not(. = preceding-sibling::action)]) != 1" >
                        <xsl:for-each select="$fieldBindingsSet/bindings/action[. = current()]" >
                            <xsl:text>_xga_</xsl:text>
                            <xsl:value-of select="@name"/>
                        </xsl:for-each>
                        <!-- add the names of any actions based on this one -->
                        <xsl:for-each select="$bindings/fm:xforms/fm:instance_configuration/fm:action[@base = current()/@name]" >
                            <xsl:text>_xga_</xsl:text>
                            <xsl:value-of select="@name"/>
                        </xsl:for-each>
                    </xsl:if>
                </xsl:attribute>

            </input>

        </xsl:for-each>

    </xsl:template>


    <xsl:template name="find-field-xpath-bindings" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="keepRelative">false</xsl:param>
        <xsl:param name="xpathName">xform_xpath</xsl:param>
        <xsl:param name="defaultXPathValue" select="$blank_xpath_value"/>

        <bindings>
            <!-- recursively loop through all actions that apply to this page to see if there are any bindings defined
                 for this field-->
            <xsl:call-template name="find-field-xpath-bindings-recursive" >
                <xsl:with-param name="configElement" select="$configElement"/>
                <xsl:with-param name="actionString" select="substring-after($actionNames, ';')"/>
                <xsl:with-param name="keepRelative" select="$keepRelative"/>
                <xsl:with-param name="xpathName" select="$xpathName"/>
                <xsl:with-param name="defaultXPathValue" select="$defaultXPathValue"/>
            </xsl:call-template>
        </bindings>
    </xsl:template>

    <xsl:template name="find-field-xpath-bindings-recursive" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="actionString"/>
        <xsl:param name="keepRelative">false</xsl:param>
        <xsl:param name="xpathName">xform_xpath</xsl:param>
        <xsl:param name="defaultXPathValue" select="$blank_xpath_value"/>

        <!-- check we still have actions to process -->
        <xsl:if test="$actionString != ''">

            <xsl:variable name="currentAction" select="substring-before($actionString, ';')"/>

            <!-- Check that this action is not based on another -->
            <xsl:if test="not($bindings/fm:xforms/fm:instance_configuration/fm:action[(@name = $currentAction) and (@base != '')])" >
                <!-- Check if there is a binding defined for this field or not -->
                <xsl:choose>
                    <xsl:when test="$bindings/fm:xforms/fm:*[(local-name() = local-name($configElement)) and (@name = $configElement/@name)]/fm:action[@name = $currentAction]/*[local-name() = $xpathName]">
                        <!-- loop through each binding that we have for this field in the bindings file-->
                        <xsl:for-each select="$bindings/fm:xforms/fm:*[(local-name() = local-name($configElement)) and (@name = $configElement/@name)]/fm:action[@name = $currentAction]/*[local-name() = $xpathName]" >
                            <action name="{$currentAction}">

                                <xsl:choose>
                                    <xsl:when test="(. = '') or (. = $blank_xpath_value)">
                                        <xsl:value-of select="$defaultXPathValue"/>
                                    </xsl:when>
                                    <xsl:otherwise>

                                        <!-- if this is in a repeat, then add the repeat part if necessary -->
                                        <xsl:if test="($keepRelative != 'true') and $configElement/ancestor::fm:repeat and not(starts-with(., '/'))">
                                            <xsl:call-template name="find-field-xpath-bindings-repeat">
                                                <xsl:with-param name="repeatElement" select="$configElement/ancestor::fm:repeat[1]"/>
                                                <xsl:with-param name="actionName" select="../@name"/>
                                            </xsl:call-template>
                                        </xsl:if>

                                        <xsl:value-of select="." />
                                    </xsl:otherwise>
                                </xsl:choose>
                            </action>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <action name="{$currentAction}">
                            <xsl:value-of select="$defaultXPathValue"/>
                        </action>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>


            <!-- process next action -->
            <xsl:call-template name="find-field-xpath-bindings-recursive" >
                <xsl:with-param name="configElement" select="$configElement"/>
                <xsl:with-param name="actionString" select="substring-after($actionString, ';')"/>
                <xsl:with-param name="keepRelative" select="$keepRelative"/>
                <xsl:with-param name="xpathName" select="$xpathName"/>
                <xsl:with-param name="defaultXPathValue" select="$defaultXPathValue"/>
            </xsl:call-template>
        </xsl:if>

    </xsl:template>

    <xsl:template name="find-field-xpath-bindings-repeat" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="repeatElement" />
        <xsl:param name="actionName"/>

        <xsl:if test="$repeatElement/ancestor::fm:repeat and not(starts-with($bindings/fm:xforms/fm:repeat[@name = $repeatElement/@name]/fm:action[@name = $actionName]/fm:xform_xpath, '/'))" >
            <xsl:call-template name="find-field-xpath-bindings-repeat">
                <xsl:with-param name="repeatElement" select="$repeatElement/ancestor::fm:repeat[1]"/>
                <xsl:with-param name="actionName" select="$actionName"/>
            </xsl:call-template>
        </xsl:if>

        <!-- ouput the repeat XPath -->
        <xsl:value-of select="$bindings/fm:xforms/fm:repeat[@name = $repeatElement/@name]/fm:action[@name = $actionName]/fm:xform_xpath"/>

        <!-- add the position indicator -->
        <xsl:text>[{$</xsl:text>
        <xsl:value-of select="$repeatElement/@name"/>
        <xsl:text>-repeat-position-number}]/</xsl:text>
    </xsl:template>




    <!-- Matches spans for displaying elements as 'output' due to extra attribute inserted
         Needed to check if the output is dynamic -->
    <xsl:template match="span[@_use='output']" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="outputName" select="@id"/>
        <xsl:variable name="outputEntry" select="$formStructure/fm:structure//fm:element[@name = $outputName and not(ancestor::fm:path)]"/>
        <xsl:copy>
            <xsl:apply-templates select="@*">
                <xsl:with-param name="configElement" select="$outputEntry"/>
            </xsl:apply-templates>
            <!-- Check if this element needs dynamic styles -->
            <xsl:call-template name="checkElementDynamicStyle">
                <xsl:with-param name="configElement" select="$outputEntry"/>
            </xsl:call-template>
            <xsl:choose>
                <xsl:when test="$outputEntry/fm:field/fm:input/@options = 'dynamic'">
                    <xsl:if test="$bindings/fm:xforms/fm:element[@name = $outputName]/fm:transform_xpath != ''">
                        <!-- check if the value is blank, and if so output nbsp to ensure table cells are shown in netscape-->
                        <out:choose>
                            <out:when test="not({$bindings/fm:xforms/fm:element[@name = $outputName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $outputName]/fm:transform_xpath} = '')">
                                <out:text disable-output-escaping="yes">&amp;#160;</out:text>
                            </out:when>
                            <out:otherwise>
                                <xsl:choose>
                                    <xsl:when test="$outputEntry/fm:constraints/fm:enumeration/@use_for_output = 'true'">
                                        <out:choose>
                                            <!-- process 'option' tags created by first stage to create runtime checks -->
                                            <xsl:apply-templates select="option" mode="createOutputChecks">
                                                <xsl:with-param name="outputEntry" select="$outputEntry"/>
                                            </xsl:apply-templates>
                                            <out:otherwise>
                                                <!-- perform any required date/case conversion -->
                                                <xsl:call-template name="convertValue">
                                                    <xsl:with-param name="configElement" select="$outputEntry"/>
                                                </xsl:call-template>
                                            </out:otherwise>
                                        </out:choose>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <!-- perform any required date/case conversion -->
                                        <xsl:call-template name="convertValue">
                                            <xsl:with-param name="configElement" select="$outputEntry"/>
                                        </xsl:call-template>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </out:otherwise>
                        </out:choose>
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
    </xsl:template>
    <!-- Matches the 'option' tags that are created by the first gen stage when the output value should be converted
         based on enumeration data.  This creates the runtime checks to perform this conversion -->
    <xsl:template match="option" mode="createOutputChecks" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="outputEntry"/>
        <out:when test="{$bindings/fm:xforms/fm:element[@name = $outputEntry/@name]/fm:transform_xpath} = '{@value}'">
            <xsl:value-of select="."/>
        </out:when>
    </xsl:template>
    <!-- Match all anchor tags being used as buttons (ie within a 'button' span)
         Needed to check if the button caption should be dynamic -->
    <!-- <xsl:template match="span[@class='button']/a"> -->
    <xsl:template mode="copycontrol" match="a[not(@_use = 'tab')]" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="buttonName" select="@name"/>
        <xsl:variable name="buttonEntry" select="$formStructure/fm:structure//fm:element[@name = $buttonName and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <xsl:when test="$buttonEntry">
                <!-- this anchor is used for a button type control -->
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <!-- prepend loop count to the name and id attributes if within a repeat -->
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="@name"/></xsl:attribute>
                    </xsl:if>
                    <!-- Check if this element needs a disable check -->
                    <xsl:call-template name="checkElementDisable">
                        <xsl:with-param name="configElement" select="$buttonEntry"/>
                    </xsl:call-template>
                    <!-- Check if this element needs dynamic styles -->
                    <xsl:call-template name="checkElementDynamicStyle">
                        <xsl:with-param name="configElement" select="$buttonEntry"/>
                    </xsl:call-template>
                    <!-- check if the value is dynamic -->
                    <xsl:choose>
                        <xsl:when test="$buttonEntry/fm:field/fm:input/@options = 'dynamic'">
                            <span><span>
                            <xsl:if test="$bindings/fm:xforms/fm:element[@name = $buttonName]/fm:transform_xpath != ''">
                                <!-- check if the value is blank, and if so output nbsp to ensure table cells are shown in netscape-->
                                <out:choose>
                                    <out:when test="not({$bindings/fm:xforms/fm:element[@name = $buttonName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $buttonName]/fm:transform_xpath} = '')">
                                        <out:text disable-output-escaping="yes">&amp;#160;</out:text>
                                    </out:when>
                                    <out:otherwise>
                                        <xsl:choose>
                                            <xsl:when test="$buttonEntry/fm:constraints/fm:enumeration/@use_for_output = 'true'">
                                                <out:choose>
                                                    <!-- process 'option' tags created by first stage to create runtime checks
                                                         This will create the <out:when> tags used to convert the data value to the
                                                         display value for each enumeration entry -->
                                                    <xsl:apply-templates select="option" mode="createOutputChecks">
                                                        <xsl:with-param name="outputEntry" select="$buttonEntry"/>
                                                    </xsl:apply-templates>
                                                    <out:otherwise>
                                                        <!-- perform any required date/case conversion -->
                                                        <xsl:call-template name="convertValue">
                                                            <xsl:with-param name="configElement" select="$buttonEntry"/>
                                                        </xsl:call-template>
                                                    </out:otherwise>
                                                </out:choose>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <!-- perform any required date/case conversion -->
                                                <xsl:call-template name="convertValue">
                                                    <xsl:with-param name="configElement" select="$buttonEntry"/>
                                                </xsl:call-template>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </out:otherwise>
                                </out:choose>
                            </xsl:if>
                            </span></span>
                        </xsl:when>
                        <xsl:otherwise>
                            <!-- check if the value is blank, and if so output nbsp to ensure table cells are shown in netscape-->
                            <xsl:choose>
                                <xsl:when test=". = ''">
                                    <xsl:text disable-output-escaping="yes">&amp;#160;</xsl:text>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:apply-templates/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:copy-of select="@*"/>
                    <!-- prepend loop count to the name and id attributes if within a repeat -->
                    <xsl:if test="ancestor::repeat_code_placeholder">
                        <xsl:attribute name="name"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="@name"/></xsl:attribute>
                    </xsl:if>
                    <!-- if this anchor is used for a calendar popup, check if it should be disabled -->
                    <xsl:if test="substring(@id, string-length(@id) - string-length('_calendar_anchor') + 1) = '_calendar_anchor'">
                        <xsl:apply-templates select="." mode="check_calendar_disable"/>
                    </xsl:if>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="img" mode="copycontrol" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="imageName" select="@name"/>
        <xsl:variable name="configEntry" select="$formStructure/fm:structure//fm:element[@name = $imageName and not(ancestor::fm:path)]"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:if test="$configEntry/fm:field/fm:input/@options = 'dynamic'">
                <xsl:if test="$bindings/fm:xforms/fm:element[@name = $imageName]/fm:transform_xpath != ''">
                    <xsl:attribute name="src">
                        <xsl:text>{</xsl:text>
                        <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $imageName]/fm:transform_xpath"/>
                        <xsl:text>}</xsl:text>
                    </xsl:attribute>
                </xsl:if>
            </xsl:if>
            <!-- Check if this element is within a repeat and if so create commands to prepend '$loopCount' to the name -->
            <xsl:if test="ancestor::repeat_code_placeholder">
                <xsl:attribute name="name">
                    <xsl:text>{$</xsl:text>
                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                    <xsl:text>LoopCount}</xsl:text>
                    <xsl:value-of select="$imageName"/>
                </xsl:attribute>
            </xsl:if>
            <!-- Check if this element needs dynamic styles -->
            <xsl:call-template name="checkElementDynamicStyle">
                <xsl:with-param name="configElement" select="$configEntry"/>
            </xsl:call-template>
            <!-- Insert commands to check if default value should be used instead -->
            <xsl:if test="$configEntry/fm:field/fm:input/@options = 'dynamic'">
                <xsl:variable name="defaultValue" select="$configEntry/fm:field/fm:input/@default"/>
                <xsl:if test="($defaultValue != '') and ($bindings/fm:xforms/fm:element[@name = $imageName]/fm:transform_xpath != '')">
                    <out:if test="not({$bindings/fm:xforms/fm:element[@name = $imageName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $imageName]/fm:transform_xpath} = '')">
                        <out:attribute name="src" namespace="">
                            <xsl:value-of select="$defaultValue"/>
                        </out:attribute>
                    </out:if>
                </xsl:if>
            </xsl:if>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="a[@_use = 'tab']" mode="copycontrol" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elementName" select="@_element"/>
        <xsl:variable name="configEntry" select="$formStructure/fm:structure//fm:element[@name = $elementName and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <xsl:when test="$configEntry/fm:field/fm:input/@options = 'dynamic'">
                <xsl:if test="$bindings/fm:xforms/fm:element[@name = $elementName]/fm:repeat_xpath != ''">
                    <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $elementName]/fm:repeat_xpath}">
                        <a href="#" class="{$configEntry/fm:field/fm:input/@unselected_tab_class}">
                            <xsl:apply-templates select="@*[local-name() != 'onclick']"/>
                            <xsl:attribute name="id">
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                </xsl:if>
                                <xsl:value-of select="$configEntry/@name"/>
                                <xsl:text>_tab_{</xsl:text>
                                <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $elementName]/fm:value_xpath"/>
                                <xsl:text>}</xsl:text>
                            </xsl:attribute>

                            <xsl:variable name="onclickContent">
                                <xsl:text>return </xsl:text>
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                </xsl:if>
                                <xsl:value-of select="$elementName"/>
                                <xsl:text>TabChange('{</xsl:text>
                                <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $elementName]/fm:value_xpath"/>
                                <xsl:text>}');</xsl:text>
                            </xsl:variable>
                            <xsl:choose>
                                <xsl:when test="@onclick">
                                    <xsl:apply-templates select="@onclick">
                                        <xsl:with-param name="extraContent" select="$onclickContent"/>
                                    </xsl:apply-templates>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="onclick">
                                        <xsl:value-of select="$onclickContent"/>
                                    </xsl:attribute>
                                </xsl:otherwise>
                            </xsl:choose>

                            <xsl:attribute name="href">
                                <xsl:text>#</xsl:text>
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <xsl:text>{$</xsl:text>
                                    <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                    <xsl:text>LoopCount}</xsl:text>
                                </xsl:if>
                                <xsl:value-of select="$configEntry/@name"/>
                                <xsl:text>__{</xsl:text>
                                <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $elementName]/fm:value_xpath"/>
                                <xsl:text>}__tab</xsl:text>
                            </xsl:attribute>
                            <out:if test="{$bindings/fm:xforms/fm:element[@name = $elementName]/fm:value_xpath} = {$bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath}">
                                <out:attribute name="class">
                                    <xsl:value-of select="$configEntry/fm:field/fm:input/@selected_tab_class"/>
                                </out:attribute>
                            </out:if>
                            <!-- Check if a default value has been given, and if so insert runtime check to see if it should be used -->
                            <xsl:if test="$configEntry/fm:field/fm:input/@default != ''">
                                <out:if test="(not({$bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath}) or ({$bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath} = '')) and ({$bindings/fm:xforms/fm:element[@name = $elementName]/fm:value_xpath} = '{$configEntry/fm:field/fm:input/@default}')">
                                    <out:attribute name="class">
                                        <xsl:value-of select="$configEntry/fm:field/fm:input/@selected_tab_class"/>
                                    </out:attribute>
                                </out:if>
                            </xsl:if>
                            <span class="l">
                                <span class="r">
                                    <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $elementName]/fm:text_xpath}"/>
                                </span>
                            </span>
                        </a>
                    </out:for-each>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="not(preceding-sibling::a[@_use = 'tab']) and $bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath != ''">
                    <out:variable name="currentTabValue">
                        <out:choose>
                            <out:when test="{$bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath} != ''">
                                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath}"/>
                            </out:when>
                            <out:otherwise>
                                <xsl:value-of select="$configEntry/fm:field/fm:input/@default"/>
                            </out:otherwise>
                        </out:choose>
                    </out:variable>
                </xsl:if>
                <xsl:copy>
                    <xsl:apply-templates select="@*[local-name() != 'onclick']"/>

                    <xsl:variable name="onclickContent">
                        <xsl:text>return </xsl:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:text>{$</xsl:text>
                            <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                            <xsl:text>LoopCount}</xsl:text>
                        </xsl:if>
                        <xsl:value-of select="$elementName"/>
                        <xsl:text>TabChange('</xsl:text>
                        <xsl:value-of select="@_value"/>
                        <xsl:text>');</xsl:text>
                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="@onclick">
                            <xsl:apply-templates select="@onclick">
                                <xsl:with-param name="extraContent" select="$onclickContent"/>
                            </xsl:apply-templates>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:attribute name="onclick">
                                <xsl:value-of select="$onclickContent"/>
                            </xsl:attribute>
                        </xsl:otherwise>
                    </xsl:choose>

                    <xsl:attribute name="href">
                        <xsl:text>#</xsl:text>
                        <xsl:if test="ancestor::repeat_code_placeholder">
                            <xsl:text>{$</xsl:text>
                            <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                            <xsl:text>LoopCount}</xsl:text>
                        </xsl:if>
                        <xsl:value-of select="substring(@href, 2)" />
                    </xsl:attribute>
                    <xsl:choose>
                        <xsl:when test="$bindings/fm:xforms/fm:element[@name = $elementName]/fm:transform_xpath != ''">
                            <out:if test="$currentTabValue = '{@_value}'">
                                <out:attribute name="class">
                                    <xsl:value-of select="$configEntry/fm:field/fm:input/@selected_tab_class"/>
                                </out:attribute>
                            </out:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:if test="$configEntry/fm:field/fm:input/@default[(. != '') and (. = current()/@_value)]" >
                                <xsl:attribute name="class">
                                    <xsl:value-of select="$configEntry/fm:field/fm:input/@selected_tab_class"/>
                                </xsl:attribute>
                            </xsl:if>
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="a[@_use = 'noscript_tab']" mode="copycontrol" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="controlName" select="substring-before(@id, '__')"/>
        <xsl:variable name="controlValue" select="substring-before(substring-after(@id, '__'), '__')"/>
        <xsl:variable name="controlField" select="$formStructure/fm:structure//fm:element[@name = $controlName and not(ancestor::fm:path)]"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="node()"/>
            <!--<xsl:value-of select="concat($controlName, '**', $controlValue, '**', $controlField/fm:field/fm:input/@options)"/>-->
            <xsl:if test="$controlField/fm:field/fm:input/@options = 'dynamic'">
                <span class="l">
                    <span class="r">
                        <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $controlName]/fm:repeat_xpath}">
                            <out:if test="{$bindings/fm:xforms/fm:element[@name = $controlName]/fm:value_xpath} = '{$controlValue}'">
                                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $controlName]/fm:text_xpath}"/>
                            </out:if>
                        </out:for-each>
                    </span>
                </span>
            </xsl:if>
        </xsl:copy>
    </xsl:template>
    <!-- Creates the template calls to output the converted value for the given configElement or just outputs the value if none required -->
    <xsl:template name="convertValue" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <!-- check if the value should be converted in any way -->
        <xsl:choose>
            <xsl:when test="$configElement/fm:constraints/fm:type = 'date'">
                <xsl:choose>
                    <xsl:when test="$configElement/fm:data_formats/fm:date[(@data != @display) and (@data != '') and (@display != '')]">

                        <xsl:choose>
                            <xsl:when test="./@_selectBoxFormat[.!='']">
                                <xsl:variable name="bindingXPath">
                                    <xsl:choose>
                                        <xsl:when test="$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath != ''" >
                                            <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath" />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:text>enter_xpath_here</xsl:text>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                <out:call-template name="create-date-options">
                                    <out:with-param name="source_string" select="{$bindingXPath}"/>
                                    <out:with-param name="source_pattern">
                                        <xsl:value-of select="$configElement/fm:data_formats/fm:date/@data"/>
                                    </out:with-param>
                                    <out:with-param name="target_pattern">
                                        <xsl:value-of select="./@_selectBoxFormat"/>
                                    </out:with-param>
                                </out:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <out:if test="({$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath} != '')">
                                    <out:call-template name="parse-format-date">
                                        <out:with-param name="source_string" select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath}"/>
                                        <out:with-param name="source_pattern">
                                            <xsl:value-of select="$configElement/fm:data_formats/fm:date/@data"/>
                                        </out:with-param>
                                        <out:with-param name="target_pattern">
                                            <xsl:choose>
                                                <xsl:when test="./@_display_date_format[.!='']">
                                                    <xsl:value-of select="./@_display_date_format"/>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:value-of select="$configElement/fm:data_formats/fm:date/@display"/>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </out:with-param>
                                    </out:call-template>
                                </out:if>
                            </xsl:otherwise>
                        </xsl:choose>

                    </xsl:when>
                    <xsl:otherwise>
                        <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath}"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$configElement/fm:constraints/fm:type = 'string'">
                <xsl:choose>
                    <xsl:when test="($configElement/fm:data_formats/fm:case[@display != '']) and ($configElement/fm:data_formats/fm:case[@display != 'preserve'])">
                        <out:call-template name="convert-case">
                            <out:with-param name="toconvert" select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath}"/>
                            <out:with-param name="conversion">
                                <xsl:value-of select="$configElement/fm:data_formats/fm:case/@display"/>
                            </out:with-param>
                        </out:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath}"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:transform_xpath}"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Match all components for showing labels, ie that have an id ending with '_label'
         Used to check if labels should be dynamicly generated-->
    <xsl:template match="*[substring(@id, string-length(@id) - string-length('_label') + 1) = '_label']" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elementName" select="substring(@id, 1, string-length(@id) - string-length('_label'))"/>
        <xsl:choose>
            <!-- Check if this label is for a group or element to apply a dynamic label value -->
            <xsl:when test="$formStructure/fm:structure//fm:element[@name = $elementName and not(ancestor::fm:path)]">
                <!-- Is an element label -->
                <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementName and not(ancestor::fm:path)]"/>

                <xsl:copy>
                    <xsl:apply-templates select="@*"/>

                    <!-- Check for dynamic label styling -->
                    <xsl:if test="$configElement/fm:field/fm:label/@options = 'dynamic'" >
                        <out:attribute name="class">
                            <xsl:if test="@class != ''">
                                <out:text>
                                    <xsl:value-of select="@class"/>
                                </out:text>
                            </xsl:if>
                            <xsl:call-template name="outputDynamicStylingImpl">
                                <xsl:with-param name="configElement" select="$configElement/fm:field/fm:label"/>
                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:element[@name = $elementName]"/>
                                <xsl:with-param name="styleType">label</xsl:with-param>
                            </xsl:call-template>
                        </out:attribute>
                    </xsl:if>

                    <!-- Check if the label is for a sorted element
                         TODO: This area needs to be updated.  This will currently override any other dynamic styling defined!!-->
                    <xsl:apply-templates select="." mode="checkSorted"/>
                    <xsl:choose>
                        <xsl:when test="$configElement/fm:field/fm:label/@type = 'dynamic'">
                            <!-- ensure any needed markers get carried across -->
                            <xsl:apply-templates select="*[@_location = 'before_label']"/>
                            <xsl:if test="$bindings/fm:xforms/fm:element[@name = $elementName]/fm:label_xpath != ''">
                                <out:value-of select="{$bindings/fm:xforms/fm:element[@name = $elementName]/fm:label_xpath}"/>
                            </xsl:if>
                            <!-- ensure any needed markers get carried across -->
                            <xsl:apply-templates select="*[@_location = 'after_label']"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:apply-templates/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:copy>
            </xsl:when>
            <xsl:when test="$formStructure/fm:structure//fm:group[@name = $elementName]">
                <!-- Is a group label -->
                <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:group[@name = $elementName]"/>

                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <!-- Check for dynamic label styling -->
                    <xsl:if test="$configElement/fm:label/@options = 'dynamic'" >
                        <out:attribute name="class">
                            <xsl:if test="@class != ''">
                                <out:text>
                                    <xsl:value-of select="@class"/>
                                </out:text>
                            </xsl:if>
                            <xsl:call-template name="outputDynamicStylingImpl">
                                <xsl:with-param name="configElement" select="$configElement/fm:label"/>
                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $elementName]"/>
                                <xsl:with-param name="styleType">grouplabel</xsl:with-param>
                            </xsl:call-template>
                        </out:attribute>
                    </xsl:if>
                    <!-- Check if the label is for a sorted element -->
                    <xsl:apply-templates select="." mode="checkSorted"/>
                    <!-- Output dynamic label value if needed -->
                    <xsl:choose>
                        <xsl:when test="$configElement/fm:label/@type = 'dynamic'">
                            <xsl:if test="$bindings/fm:xforms/fm:group[@name = $elementName]/fm:label_xpath != ''">
                                <out:value-of select="{$bindings/fm:xforms/fm:group[@name = $elementName]/fm:label_xpath}"/>
                            </xsl:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:apply-templates/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:apply-templates/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Match all labels to check if they are in a sorted repeat, and if so check if the current label is for the sorted entry,
         in order to apply the sorted styling to it -->
    <xsl:template match="*[substring(@id, string-length(@id) - string-length('_label') + 1) = '_label']" mode="checkSorted" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elementID" select="substring(@id, 1, string-length(@id) - string-length('_label'))"/>
        <!-- Check if it is a group or element label -->
        <xsl:choose>
            <xsl:when test="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]">
                <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]"/>
                <!-- Check if the elemnt is within a repeat -->
                <xsl:if test="$configElement/ancestor::fm:repeat">
                    <!-- check if the repeat is sorted (The predicate selects the first repeat ancestor in 'reverse document order' as its part of a location step!) -->
                    <xsl:if test="$configElement/ancestor::fm:repeat[1]/fm:sort">
                        <xsl:variable name="repeatEntry" select="$configElement/ancestor::fm:repeat[1]"/>
                        <!-- Check that a sorted style or class is specified -->
                        <xsl:if test="($repeatEntry/fm:sort/@sorted_column_style != '') or ($repeatEntry/fm:sort/@sorted_column_class != '')">
                            <!-- check required xpaths have been specified -->
                            <xsl:if test="($bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath != '') and ($bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:sort_xpath != '') and ($bindings/fm:xforms/fm:element[@name = $elementID]/fm:transform_xpath != '')">
                                <!-- Check if the parent group (if present) has a 'table' gen_method, and is the only child of the repeat.
                                     If so dont generate highlighting check if the element xpaths make use of the repeat variables.
                                     This is because the labels are actually the table headings in this case, and are outside of the repeat
                                     Therefore these variables will not yet have been defined. -->
                                <xsl:variable name="ancestorRepeat" select="$configElement/../.."/>
                                <xsl:choose>
                                    <xsl:when test="(local-name($configElement/..) = 'group' and $configElement/../@gen_method = 'table') and (local-name($ancestorRepeat) = 'repeat') and (count($ancestorRepeat/*[local-name() = 'group' or local-name() = 'element' or local-name() = 'repeat']) = 1)">
                                        <!-- <xsl:when test="$configElement/../*[local-name() = 'group' and @gen_method = 'table']/../*[local-name() = 'repeat' and count(*) = 1]"> -->
                                        <xsl:if test="not(contains($bindings/fm:xforms/fm:element[@name = $elementID]/fm:transform_xpath, concat('$', $repeatEntry/@name, 'LoopEntry')))">
                                            <!-- Evaluate repeat sort_xpath against element transform_xpath at runtime-->
                                            <out:if test="name({$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath}/{$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:sort_xpath}) = name({$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath}/{$bindings/fm:xforms/fm:element[@name = $elementID]/fm:transform_xpath})">
                                                <!-- Change style attributes -->
                                                <xsl:if test="$repeatEntry/fm:sort/@sorted_column_style != ''">
                                                    <out:attribute name="style" namespace="">
                                                        <xsl:value-of select="$repeatEntry/fm:sort/@sorted_column_style"/>
                                                    </out:attribute>
                                                </xsl:if>
                                                <xsl:if test="$repeatEntry/fm:sort/@sorted_column_class != ''">
                                                    <out:attribute name="class" namespace="">
                                                        <!-- <xsl:text>tableColumnHeading </xsl:text> -->
                                                        <xsl:value-of select="$repeatEntry/fm:sort/@sorted_column_class"/>
                                                    </out:attribute>
                                                </xsl:if>
                                            </out:if>
                                        </xsl:if>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <!-- Evaluate repeat sort_xpath against element transform_xpath at runtime-->
                                        <out:if test="name({$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath}/{$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:sort_xpath}) = name({$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:repeat_xpath}/{$bindings/fm:xforms/fm:element[@name = $elementID]/fm:transform_xpath})">
                                            <!-- Change style attributes -->
                                            <xsl:if test="$repeatEntry/fm:sort/@sorted_column_style != ''">
                                                <out:attribute name="style" namespace="">
                                                    <xsl:value-of select="$repeatEntry/fm:sort/@sorted_column_style"/>
                                                </out:attribute>
                                            </xsl:if>
                                            <xsl:if test="$repeatEntry/fm:sort/@sorted_column_class != ''">
                                                <out:attribute name="class" namespace="">
                                                    <!-- <xsl:text>tableColumnHeading </xsl:text> -->
                                                    <xsl:value-of select="$repeatEntry/fm:sort/@sorted_column_class"/>
                                                </out:attribute>
                                            </xsl:if>
                                        </out:if>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:if>
                        </xsl:if>
                    </xsl:if>
                </xsl:if>
            </xsl:when>
            <xsl:when test="$formStructure/fm:structure//fm:group[@name = $elementID]">
                <!-- A group does not represent a specific section of the instance so not possible to sort by group -->
                <!-- TODO: Should really check whether we are sorting on any elements within this group, in which case
                     the group should probably be highlighted! -->
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    <!-- Match all containers, ie that have an id ending with '_container'
         Used to check if container is within a repeat, and if so prepend loopCount to the ids so that error highlighting works
         Also decides whether the style of the container should be changed to produce alternate styling -->
    <xsl:template match="*[substring(@id, string-length(@id) - string-length('_container') + 1) = '_container']" xmlns:fm="http://www.hyfinity.com/formmaker">

        <!-- Check for a label container to see if any dynamic styling needs to be output, this is separated
             out because a label container could be for a group or an element-->
        <xsl:choose>
            <xsl:when test="substring(@id, string-length(@id) - string-length('_label_container') + 1) = '_label_container'">
                <xsl:variable name="elementID" select="substring(@id, 1, string-length(@id) - string-length('_label_container'))"/>
                <xsl:choose>
                    <xsl:when test="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]" >
                        <!-- Is an element label -->
                        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]"/>
                        <xsl:copy>
                            <xsl:apply-templates select="@*">
                                <xsl:with-param name="configElement" select="$configElement"/>
                            </xsl:apply-templates>

                            <xsl:if test="$configElement/fm:field/fm:label/fm:container/@options = 'dynamic'">
                                <out:attribute name="class">
                                    <out:text><xsl:value-of select="@class"/></out:text>
                                    <xsl:call-template name="outputDynamicStylingImpl">
                                        <xsl:with-param name="configElement" select="$configElement/fm:field/fm:label/fm:container"/>
                                        <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:element[@name = $configElement/@name]"/>
                                        <xsl:with-param name="styleType">labelcontainer</xsl:with-param>
                                    </xsl:call-template>
                                </out:attribute>
                            </xsl:if>

                            <xsl:apply-templates/>
                        </xsl:copy>
                    </xsl:when>
                    <xsl:when test="$formStructure/fm:structure//fm:group[@name = $elementID]" >
                        <!-- Is a group label -->
                        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:group[@name = $elementID]"/>

                        <!-- Check if this label needs to be hidden
                             This is needed if the group has hide enabled, and the label is not contained within the actual
                             group container -->
                        <xsl:choose>
                            <xsl:when test="($configElement/@hide = 'true') and ($bindings/fm:xforms/fm:group[@name = $elementID]/fm:hide_xpath != '') and
                                               not(ancestor::*[@id = $configElement/@name])">
                                <!-- label needs to be conditionally hidden -->

                                <!-- Check if it is in a repeat table, to determine whether the element should be removed completely
                                     or if the td element if present should be retained-->
                                <xsl:choose>
                                    <xsl:when test="$configElement/parent::fm:group[(@gen_method='table') or (@gen_method='horizontal_table')]/parent::fm:repeat" >
                                        <!-- Check if the xpath is absolute or not -->
                                        <xsl:variable name="isAbsolute" select="starts-with($bindings/fm:xforms/fm:group[@name = $elementID]/fm:hide_xpath, '/')"/>
                                        <xsl:choose>
                                            <xsl:when test="$isAbsolute" >
                                                <xsl:apply-templates select="." mode="process-group-hide">
                                                    <xsl:with-param name="leave-tds">false</xsl:with-param>
                                                    <xsl:with-param name="configGroup" select="$configElement" />
                                                </xsl:apply-templates>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:apply-templates select="." mode="process-group-hide">
                                                    <xsl:with-param name="leave-tds">true</xsl:with-param>
                                                    <xsl:with-param name="configGroup" select="$configElement" />
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:apply-templates select="." mode="process-group-hide">
                                            <xsl:with-param name="leave-tds">true</xsl:with-param>
                                            <xsl:with-param name="configGroup" select="$configElement" />
                                        </xsl:apply-templates>
                                    </xsl:otherwise>
                                </xsl:choose>

                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:copy>
                                    <xsl:apply-templates select="@*">
                                        <xsl:with-param name="configElement" select="$configElement"/>
                                    </xsl:apply-templates>

                                    <xsl:if test="$configElement/fm:label/fm:container/@options = 'dynamic'">
                                        <out:attribute name="class">
                                            <out:text><xsl:value-of select="@class"/></out:text>
                                            <xsl:call-template name="outputDynamicStylingImpl">
                                                <xsl:with-param name="configElement" select="$configElement/fm:label/fm:container"/>
                                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $configElement/@name]"/>
                                                <xsl:with-param name="styleType">grouplabelcontainer</xsl:with-param>
                                            </xsl:call-template>
                                        </out:attribute>
                                    </xsl:if>

                                    <xsl:apply-templates/>
                                </xsl:copy>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:apply-templates select="@*"/>
                            <xsl:apply-templates/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>

            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="elementID" select="substring(@id, 1, string-length(@id) - string-length('_container'))"/>
                <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]"/>
                <xsl:copy>
                    <xsl:apply-templates select="@*">
                        <xsl:with-param name="configElement" select="$configElement"/>
                    </xsl:apply-templates>
                    <xsl:choose>
                        <xsl:when test="substring(@id, string-length(@id) - string-length('_message_container') + 1) = '_message_container'">
                            <xsl:apply-templates select="." mode="checkDynamicMessage"/>
                        </xsl:when>

                        <xsl:otherwise>
                            <xsl:apply-templates select="." mode="checkStyle"/>
                            <xsl:apply-templates/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:copy>
                <!-- If this is the container for a radio or multiple checkbox type control, check if we need to output a noscript submit button here.
                     This is done here because we do not want multiple submit buttons output for each radio/checkbox in the group. -->
                <xsl:if test="substring(@id, string-length(@id) - string-length('_message_container') + 1) != '_message_container'">
                    <xsl:if test="($configElement/fm:field/fm:input = 'radio' or $configElement/fm:field/fm:input = 'multipleCheckbox') and $configElement/fm:events/*/fm:submit">
                        <noscript>
                            <input name="xgate_action_{($configElement/fm:events/*/fm:submit)[1]/@target}" id="{$configElement/@name}_submit_button" type="submit" value="Submit">
                                <xsl:if test="ancestor::repeat_code_placeholder">
                                    <xsl:attribute name="id"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="@name"/><xsl:text>_submit_button</xsl:text></xsl:attribute>
                                </xsl:if>
                            </input>
                        </noscript>
                    </xsl:if>
                </xsl:if>

            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- handle group label hiding -->
    <xsl:template match="*" mode="process-group-hide" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="leave-tds">true</xsl:param>
        <xsl:param name="configGroup"/>
        <xsl:choose>
            <!-- if the label container is a td, then we dont want to remove it completely
                 Instead still output the td, and any stlying on it, but dont output any contents. -->
            <xsl:when test="($leave-tds = 'true') and ((local-name(.) = 'td') or (local-name(.) = 'th'))">
                <xsl:copy>
                    <xsl:apply-templates select="@*">
                        <xsl:with-param name="configElement" select="$configGroup"/>
                    </xsl:apply-templates>

                    <xsl:if test="$configGroup/fm:label/fm:container/@options = 'dynamic'">
                        <out:attribute name="class">
                            <out:text><xsl:value-of select="@class"/></out:text>
                            <xsl:call-template name="outputDynamicStylingImpl">
                                <xsl:with-param name="configElement" select="$configGroup/fm:label/fm:container"/>
                                <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $configGroup/@name]"/>
                                <xsl:with-param name="styleType">grouplabelcontainer</xsl:with-param>
                            </xsl:call-template>
                        </out:attribute>
                    </xsl:if>

                    <out:choose>
                        <out:when test="not({$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath})">
                            <xsl:apply-templates />
                        </out:when>
                        <out:otherwise>&#160;</out:otherwise>
                    </out:choose>
                </xsl:copy>
            </xsl:when>
            <!-- otherwise, we just remove the whole container -->
            <xsl:otherwise>
                <out:if test="not({$bindings/fm:xforms/fm:group[@name = $configGroup/@name]/fm:hide_xpath})">
                    <xsl:copy>
                        <xsl:apply-templates select="@*">
                            <xsl:with-param name="configElement" select="$configGroup"/>
                        </xsl:apply-templates>

                        <xsl:if test="$configGroup/fm:label/fm:container/@options = 'dynamic'">
                            <out:attribute name="class">
                                <out:text><xsl:value-of select="@class"/></out:text>
                                <xsl:call-template name="outputDynamicStylingImpl">
                                    <xsl:with-param name="configElement" select="$configGroup/fm:label/fm:container"/>
                                    <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:group[@name = $configGroup/@name]"/>
                                    <xsl:with-param name="styleType">grouplabelcontainer</xsl:with-param>
                                </xsl:call-template>
                            </out:attribute>
                        </xsl:if>

                        <xsl:apply-templates />
                    </xsl:copy>
                </out:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Check every message container to see if it should be given a dynamic message from the server -->
    <xsl:template match="*[substring(@id, string-length(@id) - string-length('_message_container') + 1) = '_message_container']" mode="checkDynamicMessage" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elementID" select="substring(@id, 1, string-length(@id) - string-length('_message_container'))"/>
        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]"/>
        <xsl:choose>
            <!-- check if a server message should be shown if present -->
            <xsl:when test="$configElement/fm:error/fm:server/@use = 'true'">
                <!-- check if the server message exists -->
                <xsl:if test="$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath != ''">
                    <out:if test="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath} != ''">
                        <!-- set server message attribute -->
                        <!--<out:attribute name="_serverMessage" namespace="">
                            <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath}">
                                <out:value-of select="."/>
                                <out:text>&lt;br/&gt;</out:text>
                            </out:for-each>
                        </out:attribute>-->
                        <!--<out:attribute name="_serverMessageShow" namespace="">true</out:attribute>-->
                        <!-- <out:attribute name="_showing" namespace="">server</out:attribute> -->
                        <!--<out:attribute name="_serverMessageRetain" namespace="">
                            <xsl:value-of select="$configElement/fm:error/fm:server/@retain"/>
                        </out:attribute>-->
                        <!-- set the container styling to the server message styles -->
                        <xsl:if test="$configElement/fm:error/fm:server/@style != ''">
                            <out:attribute name="style" namespace="">
                                <xsl:value-of select="$configElement/fm:error/fm:server/@style"/>
                            </out:attribute>
                            <!--<out:attribute name="_serverStyle" namespace="">
                                <xsl:value-of select="$configElement/fm:error/fm:server/@style"/>
                            </out:attribute>-->
                        </xsl:if>
                        <xsl:if test="$configElement/fm:error/fm:server/@class != ''">
                            <out:attribute name="class" namespace="">
                                <xsl:value-of select="$configElement/fm:error/fm:server/@class"/>
                            </out:attribute>
                            <!--<out:attribute name="_serverClass" namespace="">
                                <xsl:value-of select="$configElement/fm:error/fm:server/@class"/>
                            </out:attribute>-->
                        </xsl:if>
                        <!-- check the message display format for this form and put the server message in the relevant place -->
                        <xsl:choose>
                            <xsl:when test="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method = 'text'">
                                <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath}">
                                    <out:value-of select="."/>
                                    <br/>
                                </out:for-each>
                            </xsl:when>
                            <xsl:when test="$formStructure/fm:structure/fm:form_details/fm:error_highlight_method = 'tooltip'">
                                <!--<out:attribute name="_tipMessage" namespace="">
                                    <out:for-each select="{$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:server_message_xpath}">
                                        <out:value-of select="."/>
                                        <out:text>&lt;br/&gt;</out:text>
                                    </out:for-each>
                                </out:attribute>-->
                                <!-- set the event handlers to show the tooltip -->
                                <out:attribute name="onmousemove" namespace="">
                                    <xsl:text>hyf.util.showTipMessage(arguments[0]);</xsl:text>
                                </out:attribute>
                                <out:attribute name="onmouseout" namespace="">
                                    <xsl:text>hyf.util.hideTipMessage(arguments[0]);</xsl:text>
                                </out:attribute>
                                <xsl:text>&#160;&#160;&#160;&#160;&#160;&#160;</xsl:text>
                            </xsl:when>
                        </xsl:choose>
                    </out:if>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Check if the style for this container should be changed.  ie for alternating styles on lines in a table like structure  -->
    <xsl:template match="*[substring(@id, string-length(@id) - string-length('_container') + 1) = '_container']" mode="checkStyle" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:variable name="elementID" select="substring(@id, 1, string-length(@id) - string-length('_container'))"/>
        <xsl:variable name="configElement" select="$formStructure/fm:structure//fm:element[@name = $elementID and not(ancestor::fm:path)]"/>
        <!-- only relevant if within a repeat -->
        <xsl:choose>
            <xsl:when test="$configElement/ancestor::fm:repeat">
                <xsl:variable name="repeatEntry" select="$configElement/ancestor::fm:repeat[1]"/>
                <!-- Check row styling -->
                <xsl:choose>
                    <xsl:when test="$repeatEntry/fm:row_styling/@alternate_styles = 'yes'">
                        <xsl:call-template name="yesRowStyling">
                            <xsl:with-param name="configElement" select="$configElement"/>
                            <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:when test="$repeatEntry/fm:row_styling/@alternate_styles = 'dynamic'">
                        <xsl:if test="$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:row_style_change_xpath != ''">
                            <out:choose>
                                <out:when test="{$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:row_style_change_xpath}">
                                    <xsl:call-template name="yesRowStyling">
                                        <xsl:with-param name="configElement" select="$configElement"/>
                                        <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                                    </xsl:call-template>
                                </out:when>
                                <out:otherwise>
                                    <xsl:call-template name="noRowStyling">
                                        <xsl:with-param name="configElement" select="$configElement"/>
                                        <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                                    </xsl:call-template>
                                </out:otherwise>
                            </out:choose>
                        </xsl:if>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- no row styling -->
                        <xsl:call-template name="noRowStyling">
                            <xsl:with-param name="configElement" select="$configElement"/>
                            <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <!-- Add container dynamic styling -->
                <xsl:call-template name="checkContainerDynamicStyle">
                    <xsl:with-param name="configElement" select="$configElement"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Check if column styling should be applied to the element as well as row styling -->
    <xsl:template name="yesRowStyling" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="repeatEntry"/>
        <xsl:choose>
            <xsl:when test="$repeatEntry/fm:column_styling/@alternate_styles = 'yes'">
                <xsl:call-template name="setRowColumnStyling">
                    <xsl:with-param name="configElement" select="$configElement"/>
                    <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="$repeatEntry/fm:column_styling/@alternate_styles = 'dynamic'">
                <xsl:if test="$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:column_style_change_xpath != ''">
                    <out:choose>
                        <out:when test="{$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:column_style_change_xpath}">
                            <xsl:call-template name="setRowColumnStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                                <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                            </xsl:call-template>
                        </out:when>
                        <out:otherwise>
                            <xsl:call-template name="setRowStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                                <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                            </xsl:call-template>
                        </out:otherwise>
                    </out:choose>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="setRowStyling">
                    <xsl:with-param name="configElement" select="$configElement"/>
                    <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Check if column styling should be applied to the element given row styling is not -->
    <xsl:template name="noRowStyling" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="repeatEntry"/>
        <xsl:choose>
            <xsl:when test="$repeatEntry/fm:column_styling/@alternate_styles = 'yes'">
                <xsl:call-template name="setColumnStyling">
                    <xsl:with-param name="configElement" select="$configElement"/>
                    <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="$repeatEntry/fm:column_styling/@alternate_styles = 'dynamic'">
                <xsl:if test="$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:column_style_change_xpath != ''">
                    <out:choose>
                        <out:when test="{$bindings/fm:xforms/fm:repeat[@name = $repeatEntry/@name]/fm:column_style_change_xpath}">
                            <xsl:call-template name="setColumnStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                                <xsl:with-param name="repeatEntry" select="$repeatEntry"/>
                            </xsl:call-template>
                        </out:when>
                        <out:otherwise>
                            <!-- Neither row or column styling -->
                        </out:otherwise>
                    </out:choose>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <!-- Add container dynamic styling -->
                <xsl:call-template name="checkContainerDynamicStyle">
                    <xsl:with-param name="configElement" select="$configElement"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- set the CSS attributes for the element when row styling should be applied -->
    <xsl:template name="setRowStyling" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="repeatEntry"/>
        <!-- set default styles -->
        <out:attribute name="style" namespace="">
            <out:text>
                <xsl:value-of select="$repeatEntry/fm:row_styling/@style1"/>
                <!-- add element styling -->
                <xsl:text> </xsl:text>
                <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
            </out:text>
        </out:attribute>
        <out:attribute name="class" namespace="">
            <out:text>
                <xsl:value-of select="$repeatEntry/fm:row_styling/@class1"/>
                <!-- add element styling -->
                <xsl:text> </xsl:text>
                <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
            </out:text>
            <!-- add dynamic element styling -->
            <xsl:call-template name="outputDynamicStyling">
                <xsl:with-param name="configElement" select="$configElement"/>
            </xsl:call-template>
        </out:attribute>
        <!-- create runtime check to change styles -->
        <xsl:variable name="row_change_period">
            <xsl:choose>
                <xsl:when test="string(number($repeatEntry/fm:row_styling/@change_period))='NaN'">1</xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$repeatEntry/fm:row_styling/@change_period"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <out:if test="(ceiling(position() div {$row_change_period}) mod 2) = 0">
            <out:attribute name="style" namespace="">
                <out:text>
                    <xsl:value-of select="$repeatEntry/fm:row_styling/@style2"/>
                    <!-- add element styling -->
                    <xsl:text> </xsl:text>
                    <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                </out:text>
            </out:attribute>
            <out:attribute name="class" namespace="">
                <out:text>
                    <xsl:value-of select="$repeatEntry/fm:row_styling/@class2"/>
                    <!-- add element styling -->
                    <xsl:text> </xsl:text>
                    <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                </out:text>
                <!-- add dynamic element styling -->
                <xsl:call-template name="outputDynamicStyling">
                    <xsl:with-param name="configElement" select="$configElement"/>
                </xsl:call-template>
            </out:attribute>
        </out:if>
    </xsl:template>
    <!-- set the CSS attributes for the element when colum styling should be applied -->
    <xsl:template name="setColumnStyling" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="repeatEntry"/>
        <xsl:variable name="col_change_period">
            <xsl:choose>
                <xsl:when test="string(number($repeatEntry/fm:column_styling/@change_period))='NaN'">1</xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$repeatEntry/fm:column_styling/@change_period"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="(ceiling(count($configElement/preceding-sibling::*) div $col_change_period) mod 2) = 0">
                <out:attribute name="style" namespace="">
                    <out:text>
                        <xsl:value-of select="$repeatEntry/fm:column_styling/@style2"/>
                        <!-- add element styling -->
                        <xsl:text> </xsl:text>
                        <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                    </out:text>
                </out:attribute>
                <out:attribute name="class" namespace="">
                    <out:text>
                        <xsl:value-of select="$repeatEntry/fm:column_styling/@class2"/>
                        <!-- add element styling -->
                        <xsl:text> </xsl:text>
                        <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                    </out:text>
                    <!-- add dynamic element styling -->
                    <xsl:call-template name="outputDynamicStyling">
                        <xsl:with-param name="configElement" select="$configElement"/>
                    </xsl:call-template>
                </out:attribute>
            </xsl:when>
            <xsl:otherwise>
                <out:attribute name="style" namespace="">
                    <out:text>
                        <xsl:value-of select="$repeatEntry/fm:column_styling/@style1"/>
                        <!-- add element styling -->
                        <xsl:text> </xsl:text>
                        <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                    </out:text>
                </out:attribute>
                <out:attribute name="class" namespace="">
                    <out:text>
                        <xsl:value-of select="$repeatEntry/fm:column_styling/@class1"/>
                        <!-- add element styling -->
                        <xsl:text> </xsl:text>
                        <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                    </out:text>
                    <!-- add dynamic element styling -->
                    <xsl:call-template name="outputDynamicStyling">
                        <xsl:with-param name="configElement" select="$configElement"/>
                    </xsl:call-template>
                </out:attribute>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- set the CSS attributes for the element when both row and colum styling should be applied -->
    <xsl:template name="setRowColumnStyling" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="repeatEntry"/>
        <xsl:variable name="row_change_period">
            <xsl:choose>
                <xsl:when test="string(number($repeatEntry/fm:row_styling/@change_period))='NaN'">1</xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$repeatEntry/fm:row_styling/@change_period"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="col_change_period">
            <xsl:choose>
                <xsl:when test="string(number($repeatEntry/fm:column_styling/@change_period))='NaN'">1</xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$repeatEntry/fm:column_styling/@change_period"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- check which column style to use -->
        <xsl:choose>
            <!-- check if second column style should be used -->
            <xsl:when test="(ceiling(count($configElement/preceding-sibling::*) div $col_change_period) mod 2) = 0">
                <!-- create runtime check to determine row style to use-->
                <out:choose>
                    <out:when test="(ceiling(position() div {$row_change_period}) mod 2) = 0">
                        <!-- use second row style -->
                        <out:attribute name="style" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@style2"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@style2"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                            </out:text>
                        </out:attribute>
                        <out:attribute name="class" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@class2"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@class2"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                            </out:text>
                            <!-- add dynamic element styling -->
                            <xsl:call-template name="outputDynamicStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                            </xsl:call-template>
                        </out:attribute>
                    </out:when>
                    <out:otherwise>
                        <!-- use first row style -->
                        <out:attribute name="style" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@style1"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@style2"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                            </out:text>
                        </out:attribute>
                        <out:attribute name="class" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@class1"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@class2"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                            </out:text>
                            <!-- add dynamic element styling -->
                            <xsl:call-template name="outputDynamicStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                            </xsl:call-template>
                        </out:attribute>
                    </out:otherwise>
                </out:choose>
            </xsl:when>
            <xsl:otherwise>
                <!-- use first column style -->
                <!-- create runtime check to determine row style to use-->
                <out:choose>
                    <out:when test="(ceiling(position() div {$row_change_period}) mod 2) = 0">
                        <!-- use second row style -->
                        <out:attribute name="style" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@style2"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@style1"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                            </out:text>
                        </out:attribute>
                        <out:attribute name="class" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@class2"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@class1"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                            </out:text>
                            <!-- add dynamic element styling -->
                            <xsl:call-template name="outputDynamicStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                            </xsl:call-template>
                        </out:attribute>
                    </out:when>
                    <out:otherwise>
                        <!-- use first row style -->
                        <out:attribute name="style" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@style1"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@style1"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@style"/>
                            </out:text>
                        </out:attribute>
                        <out:attribute name="class" namespace="">
                            <out:text>
                                <xsl:value-of select="$repeatEntry/fm:row_styling/@class1"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$repeatEntry/fm:column_styling/@class1"/>
                                <!-- add element styling -->
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$configElement/fm:field/fm:style/@class"/>
                            </out:text>
                            <!-- add dynamic element styling -->
                            <xsl:call-template name="outputDynamicStyling">
                                <xsl:with-param name="configElement" select="$configElement"/>
                            </xsl:call-template>
                        </out:attribute>
                    </out:otherwise>
                </out:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Checks if the given element should have a runtime check to disable it under certain data situations
         If so creates the commands to disable the element, and switch the style to the disabled style if specifiied -->
    <xsl:template name="checkElementDisable" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>

        <xsl:variable name="disableXPath">
            <xsl:choose>
                <xsl:when test="$configElement/@disable = 'true'">
                    <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = $configElement/@name]/fm:disable_xpath" />
                </xsl:when>
                <xsl:when test="$configElement/ancestor::fm:group[@disable = 'true']">
                    <xsl:value-of select="$bindings/fm:xforms/fm:group[@name = $configElement/ancestor::fm:group[@disable = 'true'][1]/@name]/fm:disable_xpath" />
                </xsl:when>
            </xsl:choose>
        </xsl:variable>


        <xsl:if test="$disableXPath != ''">
            <out:if test="{$disableXPath}">
                <xsl:choose>
                    <xsl:when test="$configElement/@disable_method = 'read_only'">
                        <out:attribute name="readonly" namespace="">true</out:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <out:attribute name="disabled" namespace="">disabled</out:attribute>
                    </xsl:otherwise>
                </xsl:choose>
                <!-- if the field is a button (anchor), we need to disable all the event triggers as well -->
                <xsl:if test="$configElement/fm:field/fm:input = 'button'">
                    <xsl:for-each select="$configElement/fm:events/fm:event">
                        <xsl:variable name="eventFunction" select="$eventConfig/fm:event_config/fm:events/*/*/fm:event[@id = current()/@id]/@function_name"/>


                        <xsl:choose>
                            <xsl:when test="$eventFunction = 'href'">
                                <out:attribute name="href" namespace="">#</out:attribute>
                            </xsl:when>
                            <xsl:when test="$eventFunction = 'onclick' or $eventFunction = 'onkeypress'">
                                <out:attribute name="{$eventFunction}" namespace=""/>
                            </xsl:when>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:if>
                <!-- Check if a different style has been specified for when the element is disabled -->
                <!-- Find the static disabled class specified-->
                <xsl:variable name="disabledClass" select="$configElement/fm:field/fm:disabled_style/@class"/>

                <xsl:if test="($disabledClass != '') or ($configElement/fm:field/fm:disabled_style/@options = 'dynamic')">
                    <out:attribute name="class" namespace="">
                        <xsl:if test="$disabledClass != ''">
                            <out:text>
                                <xsl:value-of select="$disabledClass"/>
                                <xsl:text> </xsl:text>
                            </out:text>
                        </xsl:if>

                        <xsl:call-template name="outputDynamicStylingImpl">
                            <xsl:with-param name="configElement" select="$configElement/fm:field/fm:disabled_style"/>
                            <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:element[@name = $configElement/@name]"/>
                            <xsl:with-param name="styleType">disabled</xsl:with-param>
                        </xsl:call-template>

                    </out:attribute>
                </xsl:if>
                <!-- Find the static disabled style specified, checking for both old and new meta data formats-->
                <xsl:variable name="disabledStyle">
                    <xsl:choose>
                        <xsl:when test="$configElement/fm:field/fm:disabled_style/@style">
                            <xsl:value-of select="$configElement/fm:field/fm:disabled_style/@style"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$configElement/fm:field/fm:element_style/@disabled_style"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$disabledStyle != ''">
                    <out:attribute name="style" namespace="">
                        <xsl:value-of select="$disabledStyle"/>
                    </out:attribute>
                </xsl:if>
            </out:if>
        </xsl:if>
    </xsl:template>
    <!-- Checks if the given element should have a runtime style xpath inserted -->
    <xsl:template name="checkElementDynamicStyle" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:if test="$configElement/fm:field/fm:element_style/@options = 'dynamic'">
            <out:attribute name="class">
                <!--<xsl:value-of select="$configElement/fm:field/fm:element_style/@class"/>-->
                <out:text><xsl:value-of select="@class"/></out:text>
                <!-- handle the dynamic styling -->
                <xsl:call-template name="outputDynamicStylingImpl">
                    <xsl:with-param name="configElement" select="$configElement/fm:field/fm:element_style"/>
                    <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:element[@name = $configElement/@name]"/>
                    <xsl:with-param name="styleType">element</xsl:with-param>
                </xsl:call-template>
            </out:attribute>
        </xsl:if>
    </xsl:template>
    <!-- Checks if the given container should have a runtime style xpath inserted -->
    <xsl:template name="checkContainerDynamicStyle" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:if test="$configElement/fm:field/fm:style/@options = 'dynamic'">
            <out:attribute name="class">
                <!--<xsl:value-of select="$configElement/fm:field/fm:style/@class"/>-->
                <out:text><xsl:value-of select="@class"/></out:text>
                <xsl:call-template name="outputDynamicStyling">
                    <xsl:with-param name="configElement" select="$configElement"/>
                </xsl:call-template>
            </out:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputDynamicStyling" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>

        <xsl:call-template name="outputDynamicStylingImpl">
            <xsl:with-param name="configElement" select="$configElement/fm:field/fm:style"/>
            <xsl:with-param name="bindingsElement" select="$bindings/fm:xforms/fm:element[@name = $configElement/@name]"/>
            <xsl:with-param name="styleType">background</xsl:with-param>
        </xsl:call-template>

    </xsl:template>

    <!-- creates the out:if xsl statements that will output the required class name values
        These need to be embeded within an out:attribute or appropriate tag -->
    <xsl:template name="outputDynamicStylingImpl" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="configElement"/>
        <xsl:param name="bindingsElement"/>
        <xsl:param name="styleType"/>
        <xsl:if test="$configElement/@options = 'dynamic'">
            <!-- handle the new type of dynamic styling -->
            <xsl:for-each select="$configElement/fm:class">
                <xsl:if test="$bindingsElement/fm:dynamic_style_xpath[@style_type = $styleType and @style_id=current()/@id] != ''">
                    <xsl:choose>
                        <xsl:when test="@type = 'static'">
                            <out:if test="{$bindingsElement/fm:dynamic_style_xpath[@style_type = $styleType and @style_id=current()/@id]}">
                                <out:text>
                                    <xsl:text> </xsl:text>
                                    <xsl:value-of select="."/>
                                </out:text>
                            </out:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:if test="$bindingsElement/fm:dynamic_style_xpath_value[@style_type = $styleType and @style_id=current()/@id] != ''">
                                <out:if test="{$bindingsElement/fm:dynamic_style_xpath[@style_type = $styleType and @style_id=current()/@id]}">
                                    <out:value-of select="concat(' ', {$bindingsElement/fm:dynamic_style_xpath_value[@style_type = $styleType and @style_id=current()/@id]})"/>
                                </out:if>
                            </xsl:if>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
            </xsl:for-each>
        </xsl:if>
    </xsl:template>



    <!-- Match any event attributes using the new style events functionality
         This then adds in the calls to the generated event functions. -->
    <xsl:template match="@*[name() = $eventConfig/fm:event_config/fm:events/*/*/fm:event/@id]" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="extraContent" />

        <!-- find whether this event is for the field or its label, or a group -->
        <xsl:variable name="eventSource">
            <xsl:choose>
                <xsl:when test="substring(@id, string-length(@id) - 9) = '_container'">container</xsl:when>
                <xsl:when test="substring(../@id, string-length(../@id) - 5) = '_label'">
                    <xsl:choose>
                        <xsl:when test="$formStructure/fm:structure//fm:group[(@name = substring(current()/../@id, 1, string-length(current()/../@id) - 6))]">grouplabel</xsl:when>
                        <xsl:otherwise>label</xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="../@_use = 'group'">group</xsl:when>
                <xsl:otherwise>field</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <!-- find the name of the field this event is associated with -->
        <xsl:variable name="fieldName">
            <xsl:choose>
                <xsl:when test="$eventSource = 'label' or $eventSource = 'grouplabel'">
                    <xsl:value-of select="substring(../@id, 1, string-length(../@id) - 6)"/>
                </xsl:when>
                <xsl:when test="../@_originalFieldName">
                    <xsl:value-of select="../@_originalFieldName"/>
                </xsl:when>
                <xsl:when test="../@_element">
                    <xsl:value-of select="../@_element"/>
                </xsl:when>
                <xsl:when test="../@name">
                    <xsl:value-of select="../@name"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="../@id"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="fieldEntry" select="$formStructure/fm:structure//fm:element[($eventSource = 'field' or $eventSource = 'label') and (@name = $fieldName) and not(ancestor::fm:path)] | $formStructure/fm:structure//fm:group[($eventSource = 'group' or $eventSource = 'grouplabel') and (@name = $fieldName)]"/>

        <xsl:choose>
            <!-- if this attribute is on a container component, or we cannot find a matching structrue file entry,
                 then just keep it as is. -->
            <xsl:when test="$eventSource = 'container' or not($fieldEntry)">
                <xsl:copy>
                    <xsl:value-of select="."/>
                </xsl:copy>
            </xsl:when>
            <!--<xsl:when test="(($eventSource = 'field') and ($eventConfig/fm:event_config/fm:events/*/*/fm:event[@id = name(current())]/@function_name != 'href'))">-->
                <!-- remove the attribute, as these will be added later using attachEvent. -->
            <!--</xsl:when>-->
            <xsl:otherwise>

                <xsl:variable name="eventsContainer" select="$fieldEntry[fm:events/fm:event and ($eventSource != 'grouplabel') and ($eventSource != 'label')] | $fieldEntry/fm:label[fm:events/fm:event and ($eventSource = 'grouplabel')] | $fieldEntry/fm:field/fm:label[fm:events/fm:event and ($eventSource = 'label')]"/>

                <xsl:variable name="event" select="$eventsContainer/fm:events/fm:event[@id = name(current())]"/>

                <xsl:variable name="functionName" select="$eventConfig/fm:event_config/fm:events/*/*/fm:event[@id = name(current())]/@function_name"/>

                <xsl:attribute name="{$functionName}">
                    <xsl:choose>
                        <!-- handle the special case where this is an href attribute with a location url.
                             In this case we do not want to use javascript, so just output the url value directly -->
                        <xsl:when test="$functionName = 'href' and $event/fm:action[@id = 'targetLocation']" >
                            <xsl:variable name="bindingsEntry" select="$bindings/fm:xforms/fm:element[($eventSource != 'group') and (@name = $fieldName)] | $bindings/fm:xforms/fm:group[($eventSource = 'group') and (@name = $fieldName)]"/>

                            <xsl:choose>
                                <xsl:when test="$event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@option = 'Dynamic'">
                                    <xsl:text>{</xsl:text>
                                    <xsl:call-template name="find-event-action-dynamic-xpath">
                                        <xsl:with-param name="param" select="$event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']"/>
                                        <xsl:with-param name="eventSource" select="$eventSource"/>
                                        <xsl:with-param name="bindingsContainer" select="$bindingsEntry"/>
                                    </xsl:call-template>
                                    <xsl:text>}</xsl:text>
                                </xsl:when>
                                <xsl:when test="$event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@option = 'Action'">
                                    <xsl:variable name="link" select="$appConfig/fm:application/fm:links/fm:link[fm:action = $event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value]"/>
                                    <xsl:choose>
                                        <xsl:when test="$link/fm:target/@type = 'Handler' and $appConfig/fm:application/fm:handlers/fm:handler[@id = $link/fm:target]/@type = 'thirdparty'">
                                            <xsl:value-of select="$appConfig/fm:application/fm:handlers/fm:handler[@id = $link/fm:target]/fm:URL" />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="$event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value"/>
                                            <xsl:text>.do</xsl:text>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value"/>
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:if test="$event/fm:action[@id = 'SetValue']">
                                <xsl:choose>
                                    <xsl:when test="contains($event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value,'?')">
                                        <out:text disable-output-escaping="yes">&amp;</out:text>
                                    </xsl:when>
                                    <xsl:otherwise>?</xsl:otherwise>
                                </xsl:choose>
                                <xsl:for-each select="$event/fm:action[@id = 'SetValue']">
                                    <xsl:if test="position() &gt; 1">
                                        <out:text disable-output-escaping="yes">&amp;</out:text>
                                    </xsl:if>
                                    <xsl:value-of select="fm:param[@name = 'TargetToSet']/@value"/>
                                    <xsl:text>=</xsl:text>
                                    <xsl:choose>
                                        <xsl:when test="fm:param[@name = 'ValueToAssign']/@option = 'Static'">
                                            <xsl:value-of select="fm:param[@name = 'ValueToAssign']/@value"/>
                                        </xsl:when>
                                        <xsl:when test="fm:param[@name = 'ValueToAssign']/@option = 'Dynamic'">
                                            <xsl:text>{</xsl:text>
                                            <xsl:call-template name="find-event-action-dynamic-xpath">
                                                <xsl:with-param name="param" select="fm:param[@name = 'ValueToAssign']"/>
                                                <xsl:with-param name="eventSource" select="$eventSource"/>
                                                <xsl:with-param name="bindingsContainer" select="$bindingsEntry"/>
                                            </xsl:call-template>
                                            <xsl:text>}</xsl:text>
                                        </xsl:when>
                                        <!-- page field or external param options are not allowed in this situation -->
                                    </xsl:choose>
                                    <!-- check if we should also output the xform xpath for this value param -->
                                    <xsl:if test="($event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@option = 'Action') and $appConfig/fm:application/fm:links/fm:link[fm:source/@type = 'Page' and fm:source = $page_id and fm:action = $event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value]/fm:submission_method = 'values'">
                                        <xsl:if test="$bindings/fm:xforms/fm:element[@name = current()/fm:param[@name = 'TargetToSet']/@value]/fm:action[@name = $event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value]/fm:xform_xpath != '' and $bindings/fm:xforms/fm:element[@name = current()/fm:param[@name = 'TargetToSet']/@value]/fm:action[@name = $event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value]/fm:xform_xpath != $blank_xpath_value">
                                            <out:text disable-output-escaping="yes">&amp;</out:text>
                                            <xsl:value-of select="fm:param[@name = 'TargetToSet']/@value"/>
                                            <xsl:text>_xpath=</xsl:text>
                                            <xsl:value-of select="$bindings/fm:xforms/fm:element[@name = current()/fm:param[@name = 'TargetToSet']/@value]/fm:action[@name = $event/fm:action[@id = 'targetLocation']/fm:param[@name = 'Target']/@value]/fm:xform_xpath"/>
                                        </xsl:if>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:if>

                        </xsl:when>
                        <xsl:when test="$event" >
                            <!-- Just call the script function, that will already have been generated -->

                            <xsl:choose>
                                <xsl:when test="$fieldEntry/fm:field/fm:input = 'button' and $functionName = 'href'">
                                    <xsl:text>javascript: </xsl:text>
                                </xsl:when>
                                <!-- Dont want to return if there is extra content,
                                     as this will stop the extra content from executing -->
                                <xsl:when test="$extraContent != ''"></xsl:when>
                                <xsl:otherwise>return </xsl:otherwise>
                            </xsl:choose>


                            <xsl:value-of select="$fieldName" />
                            <xsl:if test="$eventSource = 'label' or $eventSource = 'grouplabel'">label</xsl:if>
                            <xsl:value-of select="$functionName" />
                            <xsl:text>(</xsl:text>

                            <xsl:choose>
                                <!-- The event object doesnt exist for the anchor href so we cant pass it through
                                just use 'null' for now to save having to adjust the parameters list on the generated methods -->
                                <xsl:when test="$fieldEntry/fm:field/fm:input = 'button' and $functionName = 'href'" >null</xsl:when>
                                <xsl:otherwise>event</xsl:otherwise>
                            </xsl:choose>

                            <xsl:if test="ancestor::repeat_code_placeholder">
                                <xsl:text>, '{$</xsl:text>
                                <xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/>
                                <xsl:text>LoopCount}'</xsl:text>
                            </xsl:if>

                            <!-- Insert all the dynamic values -->
                            <xsl:for-each select="$event/fm:action/fm:param[@option = 'Dynamic']">
                                <xsl:text>, '{</xsl:text>
                                <xsl:choose>
                                    <xsl:when test="$eventSource = 'group' or $eventSource = 'grouplabel'">
                                        <xsl:call-template name="find-event-action-dynamic-xpath">
                                            <xsl:with-param name="param" select="."/>
                                            <xsl:with-param name="eventSource" select="$eventSource"/>
                                            <xsl:with-param name="bindingsContainer" select="$bindings/fm:xforms/fm:group[@name = $fieldName]"/>
                                        </xsl:call-template>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:call-template name="find-event-action-dynamic-xpath">
                                            <xsl:with-param name="param" select="."/>
                                            <xsl:with-param name="eventSource" select="$eventSource"/>
                                            <xsl:with-param name="bindingsContainer" select="$bindings/fm:xforms/fm:element[@name = $fieldName]"/>
                                        </xsl:call-template>
                                    </xsl:otherwise>
                                </xsl:choose>
                                <xsl:text>}'</xsl:text>
                            </xsl:for-each>

                            <xsl:text>);</xsl:text>
                        </xsl:when>
                    </xsl:choose>


                    <xsl:if test=". != ''">
                        <xsl:value-of select="."/>
                        <xsl:text>;</xsl:text>
                    </xsl:if>
                    <xsl:if test="$extraContent != ''">
                        <xsl:value-of select="$extraContent"/>
                    </xsl:if>
                </xsl:attribute>

            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <!-- get the id for the repeat containing the target element.
         If this repeat is the sourceRepeat, the returned id is for the specific entry in the repeat,
         otherwise just the first entry will be returned. -->
    <xsl:template name="getRepeatID" xmlns:fm="http://www.hyfinity.com/formmaker">
        <xsl:param name="sourceRepeat"/>
        <xsl:param name="targetElement"/>
        <xsl:choose>
            <xsl:when test="$targetElement/ancestor::fm:repeat">
                <!-- if the target element is in the same repeat as the source element, toggle the entry in the same
                     position in the repeat, otherwise just toggle the first one in the repeat-->
                <xsl:choose>
                    <xsl:when test="generate-id($targetElement/ancestor::fm:repeat[1]) = generate-id($sourceRepeat)">
                        <xsl:text>{$</xsl:text>
                        <xsl:value-of select="$sourceRepeat/@name"/>
                        <xsl:text>LoopCount}</xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:for-each select="$targetElement/ancestor::fm:repeat">
                            <xsl:value-of select="@name"/>
                            <xsl:text>1</xsl:text>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>

            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- Copy any custom sections to the output, taking any content from the existing output file if it exists -->
    <xsl:template match="custom">
        <xsl:choose>
            <xsl:when test="@name">
                <xsl:variable name="customID" select="@name"/>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                    <xsl:choose>
                        <xsl:when test="$existingFile//*[local-name(.) = 'custom' and @name = $customID ]">
                            <xsl:copy-of select="$existingFile//*[local-name(.) = 'custom' and @name = $customID]/*"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:copy-of select="*"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="node()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- match any id attribute to see if the repeat loopcount should be added to it. -->
    <xsl:template match="@id">
        <xsl:choose>
            <xsl:when test="ancestor::repeat_code_placeholder">
                <xsl:attribute name="id"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="."/></xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- match any for attribute (used with label tags) to see if the repeat loopcount should be added to it. -->
    <xsl:template match="@for">
        <xsl:choose>
            <xsl:when test="ancestor::repeat_code_placeholder">
                <xsl:attribute name="for"><xsl:text>{$</xsl:text><xsl:value-of select="ancestor::repeat_code_placeholder[1]/@name"/><xsl:text>LoopCount}</xsl:text><xsl:value-of select="."/></xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="."/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- match any repeat_count_placeholder tags and check whether the repeatcount value needs to be inserted -->
    <xsl:template match="repeat_count_placeholder">
        <xsl:if test="ancestor::repeat_code_placeholder">
            <out:value-of select="${ancestor::repeat_code_placeholder[1]/@name}LoopCount"/>
        </xsl:if>
    </xsl:template>
    <!-- stop outputting of the FM specific _... attributes
         These will instead be created by javascript to ensure the generated page is XHTML compliant. -->
    <xsl:template match="@*[starts-with(local-name(), '_')]">

    </xsl:template>
    <!-- Generic template that matches any attribute or node not matched above and copies it to the output
         before calling apply-templates on all children attributes/nodes -->
    <xsl:template match="@* | node()">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    <!-- **********************************************************
                    Start Utility Templates
         ********************************************************** -->
    <!-- replaces all occurrences of the given 'replace' string with the contents of the 'with'
         string in the 'text' string -->
    <xsl:template name="replace-string">
        <xsl:param name="text"/>
        <xsl:param name="replace"/>
        <xsl:param name="with"/>
        <xsl:choose>
            <xsl:when test="contains($text,$replace)">
                <xsl:value-of select="substring-before($text,$replace)"/>
                <xsl:value-of select="$with"/>
                <xsl:call-template name="replace-string">
                    <xsl:with-param name="text" select="substring-after($text,$replace)"/>
                    <xsl:with-param name="replace" select="$replace"/>
                    <xsl:with-param name="with" select="$with"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$text"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <!-- returns the contents of 'string' before the very last occurrence of
         the 'seperator' string. -->
    <xsl:template name="substring-before-last">
        <xsl:param name="string"/>
        <xsl:param name="seperator"/>
        <xsl:value-of select="substring-before($string, $seperator)"/>
        <xsl:variable name="after" select="substring-after($string, $seperator)"/>
        <xsl:if test="contains($after, $seperator)">
            <xsl:value-of select="$seperator"/>
            <xsl:call-template name="substring-before-last">
                <xsl:with-param name="string" select="$after"/>
                <xsl:with-param name="seperator" select="$seperator"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    <!-- returns the contents of 'string' after the very last occurrence of
         the 'seperator' string. -->
    <xsl:template name="substring-after-last">
        <xsl:param name="string"/>
        <xsl:param name="seperator"/>
        <xsl:variable name="after" select="substring-after($string, $seperator)"/>
        <xsl:choose>
            <xsl:when test="contains($after, $seperator)">
                <xsl:call-template name="substring-after-last">
                    <xsl:with-param name="string" select="$after"/>
                    <xsl:with-param name="seperator" select="$seperator"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$after"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- compares the two XML fragmetns passed in.
         Returns an empty string if they are the same, otherwise returns 'different' -->
    <xsl:template name="compare-fragments">
        <xsl:param name="nodeA"/>
        <xsl:param name="nodeB"/>

        <xsl:choose>
            <!-- check both elements exist -->
            <xsl:when test="$nodeA and $nodeB">

                <xsl:choose>
                    <!-- Check both  have same name -->
                    <xsl:when test="name($nodeA) = name($nodeB)">
                        <!-- check attributes -->
                        <xsl:choose>
                            <xsl:when test="count($nodeA/@*) = count($nodeB/@*)">
                                <xsl:for-each select="$nodeA/@*">
                                    <xsl:variable name="num" select="position()"/>
                                    <xsl:call-template name="compare-fragments">
                                        <xsl:with-param name="nodeA" select="."/>
                                        <xsl:with-param name="nodeB" select="$nodeB/@*[$num]"/>
                                    </xsl:call-template>
                                </xsl:for-each>
                            </xsl:when>
                            <xsl:otherwise>different</xsl:otherwise>
                        </xsl:choose>
                        <!-- check child elements -->
                        <xsl:choose>
                            <xsl:when test="count($nodeA/*) = count($nodeB/*)">
                                <xsl:for-each select="$nodeA/*">
                                    <xsl:variable name="num" select="position()"/>
                                    <xsl:call-template name="compare-fragments">
                                        <xsl:with-param name="nodeA" select="."/>
                                        <xsl:with-param name="nodeB" select="$nodeB/*[$num]"/>
                                    </xsl:call-template>
                                </xsl:for-each>
                            </xsl:when>
                            <xsl:otherwise>different</xsl:otherwise>
                        </xsl:choose>
                        <!-- check text content -->
                        <xsl:choose>
                            <xsl:when test="count($nodeA/text()) = count($nodeB/text())">
                                <xsl:for-each select="$nodeA/text()">
                                    <xsl:variable name="num" select="position()"/>
                                    <xsl:if test=". != $nodeB/text()[$num]">different</xsl:if>
                                </xsl:for-each>
                            </xsl:when>
                            <xsl:otherwise>different</xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>different</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>different</xsl:otherwise>
        </xsl:choose>
    </xsl:template>

</xsl:stylesheet>
